import { __decorate, __metadata, __param, __extends } from 'tslib';
import { InjectionToken, Injectable, Inject, ViewChild, ElementRef, Input, HostBinding, Component, Renderer2, HostListener, EventEmitter, Output, ContentChildren, QueryList, ChangeDetectorRef, Directive, Self, Optional, ViewChildren, ComponentFactoryResolver, ViewContainerRef, NgModule } from '@angular/core';
import { Subject, fromEvent, merge } from 'rxjs';
import { Easing } from '@tweenjs/tween.js';
import { delay, distinctUntilChanged, filter, debounceTime, tap } from 'rxjs/operators';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormControl, FormGroup, NgForm, FormsModule } from '@angular/forms';
import BScroll from '@better-scroll/core';
import Wheel from '@better-scroll/wheel';
import { CubicBezier } from 'tanbo-bezier';
import { DOCUMENT, Location, CommonModule } from '@angular/common';
import { NavigationStart, Router, PRIMARY_OUTLET, ChildrenOutletContexts, ActivatedRoute, RouteReuseStrategy } from '@angular/router';
import { trigger, transition, animate, keyframes, style, state } from '@angular/animations';

/**
 * 当前运行环境的枚举量
 */
var BrowserENV;
(function (BrowserENV) {
    BrowserENV["iphone"] = "iphone";
    BrowserENV["iphoneX"] = "iphoneX";
    BrowserENV["android"] = "android";
    BrowserENV["default"] = "default";
})(BrowserENV || (BrowserENV = {}));
/**
 * @whatItDoes 给组件、指令或服务注入当前运行环境的 token
 * 一般由框架自动检测，你可以在组件注入，并根据不同的环境，编写后续逻辑
 *
 * @howToUse
 * ```typescript
 * import { Inject } from '@angular/core';
 * import { UI_BROWSER_ENV, BrowserENV } from '@tanbo/ui-native';
 *
 * @Component({
 *   templateUrl: './demo.html'
 * })
 * export class DemoComponent {
 *   constructor(@Inject(UI_BROWSER_ENV) private env: BrowserENV) {
 *     console.log(env); // 当前环境
 *   }
 * }
 *
 * ```
 */
var UI_BROWSER_ENV = new InjectionToken('UI_BROWSER_ENV');
/**
 * @whatItDoes 给组件、指令或服务注入当前缩放系数的 token
 * @howToUse
 * ```typescript
 * import { Inject } from '@angular/core';
 * import { UI_SCREEN_SCALE } from '@tanbo/ui-native';
 *
 * @Component({
 *   templateUrl: './demo.html'
 * })
 * export class DemoComponent {
 *   constructor(@Inject(UI_SCREEN_SCALE) private env: number) {
 *     console.log(env); // 当前环境
 *   }
 * }
 *
 * ```
 */
var UI_SCREEN_SCALE = new InjectionToken('UI_SCREEN_SCALE');

var AppController = /** @class */ (function () {
    function AppController() {
        this.quitEvent = new Subject();
        this.resizeEvent = new Subject();
        this.transitionEvent = new Subject();
        this.onQuit = this.quitEvent.asObservable();
        this.onResize = this.resizeEvent.asObservable();
        this.transition = this.transitionEvent.asObservable();
    }
    AppController.prototype.quit = function () {
        this.quitEvent.next();
    };
    AppController.prototype.resize = function (env) {
        this.resizeEvent.next(env);
    };
    AppController.prototype.transitionStart = function () {
        this.transitionEvent.next(true);
    };
    AppController.prototype.transitionEnd = function () {
        this.transitionEvent.next(false);
    };
    AppController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], AppController);
    return AppController;
}());
function getDeviceType() {
    var u = navigator.userAgent;
    // android终端
    var isAndroid = u.indexOf('Android') > -1 || u.indexOf('Adr') > -1;
    // ios终端
    var isIOS = /\(i[^;]+;( U;)? CPU.+Mac OS X/.test(u);
    if (isAndroid) {
        return BrowserENV.android;
    }
    if (isIOS) {
        var screen_1 = window.screen;
        if (screen_1.width === 375 && screen_1.height === 812 ||
            screen_1.height === 375 && screen_1.width === 812 ||
            screen_1.width === 414 && screen_1.height === 896 ||
            screen_1.height === 414 && screen_1.width === 896) {
            return BrowserENV.iphoneX;
        }
        return BrowserENV.iphone;
    }
    return BrowserENV.default;
}

var UI_DO_REFRESH_DISTANCE = new InjectionToken('UI_DO_REFRESH_DISTANCE');
var PullDownRefreshController = /** @class */ (function () {
    function PullDownRefreshController(doRefreshDistance) {
        this.doRefreshDistance = doRefreshDistance;
        this.dragSource = new Subject();
        this.dragEndSource = new Subject();
        this.refreshSource = new Subject();
        this.refreshEndSource = new Subject();
        this.distance = 0;
        this.timer = null;
        this.onStateChange = this.dragSource.asObservable();
        this.onDragEnd = this.dragEndSource.asObservable();
        this.onRefresh = this.refreshSource.asObservable();
        this.onRefreshEnd = this.refreshEndSource.asObservable();
    }
    PullDownRefreshController.prototype.refreshEnd = function () {
        var _this = this;
        this.refreshEndSource.next();
        if (this.distance === 0) {
            this.dragSource.next(0);
            return;
        }
        this.timer = setTimeout(function () {
            _this.animationTo(0);
        }, 1000);
    };
    PullDownRefreshController.prototype.drag = function (n) {
        clearTimeout(this.timer);
        cancelAnimationFrame(this.animationId);
        this.distance = n;
        if (this.distance < 0) {
            this.distance = 0;
        }
        this.dragSource.next(this.distance);
    };
    PullDownRefreshController.prototype.dragEnd = function () {
        if (this.distance < this.doRefreshDistance) {
            this.animationTo(0);
        }
        else {
            this.refresh();
        }
        this.dragEndSource.next();
    };
    PullDownRefreshController.prototype.refresh = function () {
        var _this = this;
        cancelAnimationFrame(this.animationId);
        this.animationTo(this.doRefreshDistance, function () {
            _this.refreshSource.next();
        });
    };
    PullDownRefreshController.prototype.animationTo = function (target, callback) {
        var _this = this;
        var oldDistance = this.distance;
        var interval = target - oldDistance;
        if (interval === 0) {
            return;
        }
        var max = 20;
        var step = 0;
        var animationFn = function () {
            step++;
            _this.distance = Easing.Cubic.Out(step / max) * interval + oldDistance;
            _this.dragSource.next(_this.distance);
            if (step < max) {
                _this.animationId = requestAnimationFrame(animationFn);
            }
            else {
                if (callback) {
                    callback();
                }
            }
        };
        this.animationId = requestAnimationFrame(animationFn);
    };
    PullDownRefreshController.ctorParameters = function () { return [
        { type: Number, decorators: [{ type: Inject, args: [UI_DO_REFRESH_DISTANCE,] }] }
    ]; };
    PullDownRefreshController = __decorate([
        Injectable(),
        __param(0, Inject(UI_DO_REFRESH_DISTANCE)),
        __metadata("design:paramtypes", [Number])
    ], PullDownRefreshController);
    return PullDownRefreshController;
}());

var UI_DO_LOAD_DISTANCE = new InjectionToken('UI_DO_LOAD_DISTANCE');
var PullUpLoadController = /** @class */ (function () {
    function PullUpLoadController() {
        this.infiniteSource = new Subject();
        this.loadedSource = new Subject();
        this.isPrevLoaded = true;
        this.onLoading = this.infiniteSource.asObservable();
        this.onLoaded = this.loadedSource.asObservable();
    }
    PullUpLoadController.prototype.loading = function () {
        if (this.isPrevLoaded) {
            this.isPrevLoaded = false;
            this.infiniteSource.next();
        }
    };
    PullUpLoadController.prototype.loaded = function () {
        this.isPrevLoaded = true;
        this.loadedSource.next();
    };
    PullUpLoadController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], PullUpLoadController);
    return PullUpLoadController;
}());

var RefresherComponent = /** @class */ (function () {
    function RefresherComponent(pullDownRefreshController, scale, doRefreshDistance) {
        this.pullDownRefreshController = pullDownRefreshController;
        this.scale = scale;
        this.doRefreshDistance = doRefreshDistance;
        this.color = '#80848f';
        this.opacity = 1;
        this.progress = 0;
        this.isLoaded = false;
        this.subs = [];
    }
    Object.defineProperty(RefresherComponent.prototype, "height", {
        get: function () {
            return this.progress + 'px';
        },
        enumerable: true,
        configurable: true
    });
    RefresherComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.pullDownRefreshController.onStateChange.subscribe(function (n) {
            cancelAnimationFrame(_this.animationId);
            _this.progress = n;
            if (_this.isLoaded && n < _this.doRefreshDistance) {
                _this.outStage();
                if (n === 0) {
                    _this.isLoaded = false;
                }
            }
            else {
                _this.opacity = 1;
                _this.inStage();
            }
        }));
        this.subs.push(this.pullDownRefreshController.onRefresh.subscribe(function () {
            _this.animateYoYo();
        }));
        this.subs.push(this.pullDownRefreshController.onRefreshEnd.subscribe(function () {
            _this.opacity = 0.5;
            _this.isLoaded = true;
        }));
    };
    RefresherComponent.prototype.ngAfterViewInit = function () {
        this.canvasElement = this.canvas.nativeElement;
        this.containerWidth = this.canvasElement.offsetWidth;
        this.canvasElement.width = this.containerWidth * 2;
        this.context = this.canvasElement.getContext('2d');
    };
    RefresherComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
        cancelAnimationFrame(this.animationId);
    };
    RefresherComponent.prototype.inStage = function () {
        if (!this.canvasElement) {
            return;
        }
        this.canvasElement.height = this.progress * 2;
        this.context.translate(this.containerWidth, this.progress);
        this.drawProgress(this.progress);
    };
    RefresherComponent.prototype.outStage = function () {
        if (!this.canvasElement) {
            return;
        }
        this.canvasElement.height = this.progress * 2;
        var context = this.context;
        context.translate(this.containerWidth, this.progress);
        context.clearRect(-this.containerWidth, -this.progress, this.containerWidth * 2, this.progress * 2);
        context.fillStyle = this.color;
        var r = this.doRefreshDistance - this.progress;
        context.arc(0, 0, r * 10 * this.scale, 0, Math.PI * 2);
        context.fill();
        this.opacity = Easing.Cubic.Out(this.progress / this.doRefreshDistance) / 2;
    };
    RefresherComponent.prototype.animateYoYo = function () {
        var _this = this;
        var progress = this.progress;
        var max = this.doRefreshDistance;
        var yoyo = function () {
            progress += 2 * _this.scale;
            var isSingle = Math.floor(progress / max) % 2 === 1;
            var result;
            if (isSingle) {
                result = max - progress % max;
            }
            else {
                result = progress % max;
            }
            _this.drawProgress(Easing.Cubic.In(result / max) * max);
            if (_this.isLoaded && result === _this.doRefreshDistance) {
                return;
            }
            _this.animationId = requestAnimationFrame(yoyo);
        };
        this.animationId = requestAnimationFrame(yoyo);
    };
    RefresherComponent.prototype.drawProgress = function (progress) {
        var context = this.context;
        context.clearRect(-this.containerWidth, -this.progress, this.containerWidth * 2, this.progress * 2);
        this.context.fillStyle = this.color;
        var r1 = this.progress / 4;
        var r2 = this.progress / 5;
        if (r1 > 12 * this.scale) {
            r1 = 12 * this.scale;
        }
        if (r2 > 8 * this.scale) {
            r2 = 8 * this.scale;
        }
        context.beginPath();
        context.arc(0, 0, r1, 0, Math.PI * 2);
        context.closePath();
        context.fill();
        var distance = (this.doRefreshDistance - progress) / this.doRefreshDistance * 40 * this.scale;
        context.beginPath();
        context.arc(-distance, 0, r2, 0, Math.PI * 2);
        context.closePath();
        context.fill();
        context.beginPath();
        context.arc(distance, 0, r2, 0, Math.PI * 2);
        context.closePath();
        context.fill();
    };
    RefresherComponent.ctorParameters = function () { return [
        { type: PullDownRefreshController },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_DO_REFRESH_DISTANCE,] }] }
    ]; };
    __decorate([
        ViewChild('canvas', { static: true }),
        __metadata("design:type", ElementRef)
    ], RefresherComponent.prototype, "canvas", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RefresherComponent.prototype, "color", void 0);
    __decorate([
        HostBinding('style.height'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], RefresherComponent.prototype, "height", null);
    RefresherComponent = __decorate([
        Component({
            selector: 'ui-refresher',
            template: "<canvas [ngStyle]=\"{opacity: opacity}\" class=\"ui-progress-bar\" #canvas></canvas>\n<div class=\"ui-refresher-content\" [ngClass]=\"{'ui-refresher-loaded': isLoaded}\">\n  <ng-content></ng-content>\n</div>"
        }),
        __param(1, Inject(UI_SCREEN_SCALE)),
        __param(2, Inject(UI_DO_REFRESH_DISTANCE)),
        __metadata("design:paramtypes", [PullDownRefreshController, Number, Number])
    ], RefresherComponent);
    return RefresherComponent;
}());

var ScrollComponent = /** @class */ (function () {
    function ScrollComponent(renderer, elementRef, doLoadDistance, pullUpLoadController, pullDownRefreshController) {
        this.renderer = renderer;
        this.elementRef = elementRef;
        this.doLoadDistance = doLoadDistance;
        this.pullUpLoadController = pullUpLoadController;
        this.pullDownRefreshController = pullDownRefreshController;
        this.distanceY = 0;
        this.unBindFnList = [];
        this._openRefresh = false;
        this._openInfinite = false;
        this.isBindRefresh = false;
        this.isBindInfinite = false;
    }
    Object.defineProperty(ScrollComponent.prototype, "openRefresh", {
        get: function () {
            return this._openRefresh;
        },
        // 是否开启下拉刷新
        set: function (value) {
            this._openRefresh = value;
            if (value && !this.isBindRefresh) {
                this.bindingRefresher();
                this.isBindRefresh = true;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ScrollComponent.prototype, "openInfinite", {
        get: function () {
            return this._openInfinite;
        },
        // 是否开启下拉刷新
        set: function (value) {
            this._openInfinite = value;
            if (value && !this.isBindInfinite) {
                this.bindingInfinite();
                this.isBindInfinite = true;
            }
        },
        enumerable: true,
        configurable: true
    });
    ScrollComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.sub = this.pullDownRefreshController.onStateChange.subscribe(function (n) {
            _this.distanceY = n;
            _this.paddingTop = n + "px";
        });
    };
    ScrollComponent.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
        this.unBindFnList.forEach(function (item) { return item(); });
    };
    ScrollComponent.prototype.bindingInfinite = function () {
        var _this = this;
        var element = this.elementRef.nativeElement;
        this.unBindFnList.push(this.renderer.listen(element, 'scroll', function () {
            if (!_this.openInfinite) {
                return;
            }
            // 计算最大滚动距离
            var maxScrollY = Math.max(element.scrollHeight, element.offsetHeight) - element.offsetHeight;
            // 如果当前滚动距离小于上拉刷新临界值，则记录相应值，并就广播相应事件
            if (maxScrollY - element.scrollTop < _this.doLoadDistance) {
                _this.pullUpLoadController.loading();
            }
        }));
    };
    ScrollComponent.prototype.bindingRefresher = function () {
        var _this = this;
        var element = this.elementRef.nativeElement;
        var fn = this.renderer.listen(element, 'touchstart', function (ev) {
            if (!_this.openRefresh) {
                return;
            }
            var startPoint = ev.touches[0];
            var startX = startPoint.pageX;
            var startY = startPoint.pageY;
            var oldY = startY;
            var unBindTouchMoveFn;
            var unBindTouchEndFn;
            var unBindTouchCancelFn;
            var unBindFn = function () {
                unBindTouchMoveFn();
                unBindTouchEndFn();
                unBindTouchCancelFn();
                _this.pullDownRefreshController.dragEnd();
            };
            var isFirstTouching = true;
            unBindTouchMoveFn = _this.renderer.listen(element, 'touchmove', function (moveEvent) {
                var movePoint = moveEvent.touches[0];
                var moveX = movePoint.pageX;
                var moveY = movePoint.pageY;
                if (Math.abs(moveX - startX) > Math.abs(moveY - startY) && isFirstTouching) {
                    unBindFn();
                    return;
                }
                isFirstTouching = false;
                var n = moveY - oldY;
                oldY = moveY;
                if (n < 0) {
                    // 上拉
                    if (_this.distanceY > 0) {
                        _this.pullDownRefreshController.drag(n / 2 + _this.distanceY);
                        return false;
                    }
                }
                else if (element.scrollTop <= 0) {
                    // 下拉
                    _this.pullDownRefreshController.drag(n / 2 + _this.distanceY);
                }
            });
            unBindTouchEndFn = _this.renderer.listen(element, 'touchend', unBindFn);
            unBindTouchCancelFn = _this.renderer.listen(element, 'touchcancel', unBindFn);
        });
        this.unBindFnList.push(fn);
    };
    ScrollComponent.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef },
        { type: Number, decorators: [{ type: Inject, args: [UI_DO_LOAD_DISTANCE,] }] },
        { type: PullUpLoadController },
        { type: PullDownRefreshController }
    ]; };
    __decorate([
        HostBinding('style.paddingTop'),
        __metadata("design:type", String)
    ], ScrollComponent.prototype, "paddingTop", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], ScrollComponent.prototype, "openRefresh", null);
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], ScrollComponent.prototype, "openInfinite", null);
    ScrollComponent = __decorate([
        Component({
            selector: 'ui-scroll',
            template: "<ng-content></ng-content>\n"
        }),
        __param(2, Inject(UI_DO_LOAD_DISTANCE)),
        __metadata("design:paramtypes", [Renderer2,
            ElementRef, Number, PullUpLoadController,
            PullDownRefreshController])
    ], ScrollComponent);
    return ScrollComponent;
}());

var ScrollTabService = /** @class */ (function () {
    function ScrollTabService() {
        this.selectedEvent = new Subject();
        this.onSelected = this.selectedEvent.asObservable();
    }
    ScrollTabService.prototype.selected = function (c) {
        this.selectedEvent.next(c);
    };
    ScrollTabService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ScrollTabService);
    return ScrollTabService;
}());

var ScrollTabButtonComponent = /** @class */ (function () {
    function ScrollTabButtonComponent(elementRef, scrollTabService) {
        this.elementRef = elementRef;
        this.scrollTabService = scrollTabService;
        this.selected = false;
    }
    ScrollTabButtonComponent.prototype.click = function () {
        this.scrollTabService.selected(this);
    };
    ScrollTabButtonComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ScrollTabService }
    ]; };
    __decorate([
        Input(),
        HostBinding('class.ui-selected'),
        __metadata("design:type", Boolean)
    ], ScrollTabButtonComponent.prototype, "selected", void 0);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ScrollTabButtonComponent.prototype, "click", null);
    ScrollTabButtonComponent = __decorate([
        Component({
            selector: 'ui-scroll-tab-button',
            template: "<ng-content></ng-content>"
        }),
        __metadata("design:paramtypes", [ElementRef,
            ScrollTabService])
    ], ScrollTabButtonComponent);
    return ScrollTabButtonComponent;
}());

var ScrollTabComponent = /** @class */ (function () {
    function ScrollTabComponent(scrollTabService) {
        this.scrollTabService = scrollTabService;
        this.uiChange = new EventEmitter();
        this.left = 0;
        this.lineWidth = 0;
        this._index = 0;
        this.subs = [];
    }
    Object.defineProperty(ScrollTabComponent.prototype, "index", {
        get: function () {
            return this._index;
        },
        set: function (value) {
            if (this.children) {
                this.autoUpdateStyle(value);
            }
            this._index = value;
        },
        enumerable: true,
        configurable: true
    });
    ScrollTabComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        setTimeout(function () {
            _this.initStyle();
        });
        this.subs.push(this.children.changes.pipe(delay(0)).subscribe(function () {
            _this.initStyle();
        }));
        this.subs.push(this.scrollTabService.onSelected.subscribe(function (c) {
            _this.children.forEach(function (item, i) {
                if (item === c) {
                    _this.uiChange.emit(i);
                }
            });
        }));
    };
    ScrollTabComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    ScrollTabComponent.prototype.initStyle = function () {
        var _this = this;
        if (this.children.length === 0) {
            return;
        }
        var btns = this.children.toArray().map(function (item, index) {
            item.selected = index === _this.index;
            return item.elementRef.nativeElement;
        });
        var element = this.wrap.nativeElement;
        this.lineWidth = btns[this.index].offsetWidth;
        this.left = btns[this.index].offsetLeft;
        element.scrollLeft = this.left + this.lineWidth / 2 - element.offsetWidth / 2;
    };
    ScrollTabComponent.prototype.autoUpdateStyle = function (index) {
        var _this = this;
        cancelAnimationFrame(this.animationId);
        if (index === this.index) {
            return;
        }
        var btns = this.children.toArray().map(function (item, ii) {
            item.selected = ii === index;
            return item.elementRef.nativeElement;
        });
        if (!btns[this.index] || !btns[index]) {
            return;
        }
        var element = this.wrap.nativeElement;
        var oldLineWidth = btns[this.index].offsetWidth;
        var oldScrollLeft = element.scrollLeft;
        var oldLeft = this.left;
        var targetLineWidth = btns[index].offsetWidth;
        var targetScrollLeft = btns[index].offsetLeft + btns[index].offsetWidth / 2 - element.offsetWidth / 2;
        var targetLeft = btns[index].offsetLeft;
        var i = 0;
        var fn = function () {
            i++;
            var offset = Easing.Cubic.Out(i / 20);
            var scrollLeft = offset * (targetScrollLeft - oldScrollLeft);
            var width = offset * (targetLineWidth - oldLineWidth);
            var left = offset * (targetLeft - oldLeft);
            element.scrollLeft = oldScrollLeft + scrollLeft;
            _this.lineWidth = oldLineWidth + width;
            _this.left = oldLeft + left;
            if (i < 20) {
                _this.animationId = requestAnimationFrame(fn);
            }
        };
        this.animationId = requestAnimationFrame(fn);
    };
    ScrollTabComponent.ctorParameters = function () { return [
        { type: ScrollTabService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], ScrollTabComponent.prototype, "uiChange", void 0);
    __decorate([
        ContentChildren(ScrollTabButtonComponent),
        __metadata("design:type", QueryList)
    ], ScrollTabComponent.prototype, "children", void 0);
    __decorate([
        ViewChild('wrap', { static: true }),
        __metadata("design:type", ElementRef)
    ], ScrollTabComponent.prototype, "wrap", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number),
        __metadata("design:paramtypes", [Number])
    ], ScrollTabComponent.prototype, "index", null);
    ScrollTabComponent = __decorate([
        Component({
            selector: 'ui-scroll-tab',
            template: "<div class=\"ui-scroll-tab\" #wrap>\n  <div class=\"ui-scroll-tab-wrap\">\n    <ng-content></ng-content>\n  </div>\n  <div class=\"ui-scroll-tab-line-wrap\" [ngStyle]=\"{width: lineWidth + 'px', transform: 'translateX('+left+'px)'}\">\n    <div class=\"ui-scroll-tab-line\"></div>\n  </div>\n</div>",
            providers: [
                ScrollTabService
            ]
        }),
        __metadata("design:paramtypes", [ScrollTabService])
    ], ScrollTabComponent);
    return ScrollTabComponent;
}());

var CollectionItemComponent = /** @class */ (function () {
    function CollectionItemComponent() {
    }
    CollectionItemComponent = __decorate([
        Component({
            selector: 'ui-collection-item',
            template: "<ng-content></ng-content>",
            providers: [
                PullDownRefreshController,
                PullUpLoadController
            ]
        })
    ], CollectionItemComponent);
    return CollectionItemComponent;
}());

var CollectionComponent = /** @class */ (function () {
    function CollectionComponent(scale, elementRef) {
        this.scale = scale;
        this.elementRef = elementRef;
        /**
         * 拖动事件 $event 为当前拖动的进度
         */
        this.uiSliding = new EventEmitter();
        /**
         * 当完成整屏切换时，触发事件， $event 为当前拖动的进度
         */
        this.uiSlidingFinish = new EventEmitter();
        /**
         * 是否为垂直翻页，默认为 false，即为水平翻页
         */
        this.vertical = false;
        /**
         * 是否填满整个容器，默认不填充，如设置为 true，那么会填充满第一个有定位的父级元素
         */
        this.fill = true;
        this.transform = '';
        this.max = 0;
        // 记录已拖动的距离
        this.distance = 0;
        this.slidingEventSource = new Subject();
        this.subs = [];
        this._index = 0;
        this.oldDistance = 0;
        this.slidingEvent$ = this.slidingEventSource.asObservable();
    }
    Object.defineProperty(CollectionComponent.prototype, "index", {
        get: function () {
            return this._index;
        },
        /**
         * 显示第几屏的索引
         */
        set: function (value) {
            this._index = value;
            if (this.items) {
                cancelAnimationFrame(this.animationId);
                this.setStyle();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CollectionComponent.prototype, "width", {
        // 通过子级的多少，计算自身的盒子大小
        get: function () {
            return this.vertical ? 'auto' : this.childrenCount * 100 + '%';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CollectionComponent.prototype, "height", {
        get: function () {
            return this.vertical ? this.childrenCount * 100 + '%' : 'auto';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CollectionComponent.prototype, "childrenCount", {
        get: function () {
            if (this.items) {
                return this.items.length;
            }
            return 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CollectionComponent.prototype, "stepDistance", {
        get: function () {
            return this.vertical ? this.element.offsetHeight : this.element.offsetWidth;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CollectionComponent.prototype, "min", {
        get: function () {
            return -this.stepDistance * (this.childrenCount - 1);
        },
        enumerable: true,
        configurable: true
    });
    CollectionComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        this.element = this.elementRef.nativeElement;
        this.subs.push(this.slidingEvent$.pipe(distinctUntilChanged()).subscribe(function (n) {
            _this.uiSliding.emit(n);
        }));
        this.setStyle();
    };
    CollectionComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    CollectionComponent.prototype.drag = function (event) {
        if (event.first) {
            this.oldDistance = this.distance;
        }
        if ((this.vertical && event.firstDirection !== 'up' && event.firstDirection !== 'down') ||
            (!this.vertical && event.firstDirection !== 'left' && event.firstDirection !== 'right')) {
            this.touchAction = undefined;
            event.stop();
            return;
        }
        if (event.type === 'touchmove') {
            this.touchAction = this.vertical ? 'pan-y' : 'pan-x';
            var distance = this.oldDistance + (this.vertical ? event.distanceY : event.distanceX);
            if (distance > this.max) {
                distance /= 3;
            }
            else if (distance < this.min) {
                distance = this.min + (distance - this.min) / 3;
            }
            this.distance = distance;
            this.transform = "translate" + (this.vertical ? 'Y' : 'X') + "(" + distance + "px)";
            // 发送事件，并传出当前已滑动到第几屏的进度
            this.slidingEventSource.next(distance / this.stepDistance * -1);
            // event.srcEvent.preventDefault();
        }
        else if (event.type === 'touchend') {
            this.touchAction = undefined;
            if (this.distance > this.max) {
                this.autoUpdateStyle(this.max);
                return;
            }
            else if (this.distance < this.min) {
                this.autoUpdateStyle(this.min);
                return;
            }
            var translateDistance = void 0;
            var targetIndex = Math.ceil(this.distance / this.stepDistance);
            var offset = Math.abs(this.distance % this.stepDistance);
            if (event.durationTime < 200 && Math.abs(this.vertical ? event.distanceY : event.distanceX) > 50 * this.scale) {
                // swipe
                if (this.oldDistance < this.distance) {
                    translateDistance = targetIndex * this.stepDistance;
                }
                else {
                    translateDistance = targetIndex * this.stepDistance - this.stepDistance;
                }
            }
            else {
                if (offset < (this.stepDistance / 2)) {
                    translateDistance = targetIndex * this.stepDistance;
                }
                else {
                    translateDistance = targetIndex * this.stepDistance - this.stepDistance;
                }
            }
            this.autoUpdateStyle(translateDistance);
        }
    };
    CollectionComponent.prototype.setStyle = function () {
        this.distance = this.index * -1 * this.stepDistance;
        this.transform = "translate" + (this.vertical ? 'Y' : 'X') + "(" + this.distance + "px)";
    };
    CollectionComponent.prototype.autoUpdateStyle = function (translateDistance) {
        var _this = this;
        var max = 20;
        var step = 0;
        var distance = translateDistance - this.distance;
        if (distance === 0) {
            return;
        }
        var rawDistance = this.distance;
        var moveToTarget = function () {
            step++;
            var translate = rawDistance + Easing.Cubic.Out(step / max) * distance;
            _this.distance = translate;
            _this.transform = "translate" + (_this.vertical ? 'Y' : 'X') + "(" + translate + "px)";
            _this.slidingEventSource.next(translate / _this.stepDistance * -1);
            if (step < max) {
                _this.animationId = requestAnimationFrame(moveToTarget);
            }
            else {
                // 发送事件，并传出当前滑动到了第几屏
                _this.uiSlidingFinish.emit(_this.distance / _this.stepDistance * -1);
            }
        };
        this.animationId = requestAnimationFrame(moveToTarget);
    };
    CollectionComponent.ctorParameters = function () { return [
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: ElementRef }
    ]; };
    __decorate([
        ContentChildren(CollectionItemComponent),
        __metadata("design:type", QueryList)
    ], CollectionComponent.prototype, "items", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], CollectionComponent.prototype, "uiSliding", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], CollectionComponent.prototype, "uiSlidingFinish", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], CollectionComponent.prototype, "vertical", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-fill'),
        __metadata("design:type", Boolean)
    ], CollectionComponent.prototype, "fill", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number),
        __metadata("design:paramtypes", [Number])
    ], CollectionComponent.prototype, "index", null);
    CollectionComponent = __decorate([
        Component({
            selector: 'ui-collection',
            template: "<div class=\"ui-collection\"\n     (uiPan)=\"drag($event)\"\n     [ngClass]=\"{'ui-vertical': vertical}\"\n     [ngStyle]=\"{width: width, height: height, transform: transform, webkitTransform: transform, touchAction: touchAction}\">\n  <ng-content></ng-content>\n</div>"
        }),
        __param(0, Inject(UI_SCREEN_SCALE)),
        __metadata("design:paramtypes", [Number, ElementRef])
    ], CollectionComponent);
    return CollectionComponent;
}());

var FlexComponent = /** @class */ (function () {
    function FlexComponent() {
    }
    FlexComponent = __decorate([
        Component({
            selector: 'ui-flex',
            template: "<ng-content></ng-content>"
        })
    ], FlexComponent);
    return FlexComponent;
}());

var FlexItemComponent = /** @class */ (function () {
    function FlexItemComponent() {
    }
    FlexItemComponent = __decorate([
        Component({
            selector: 'ui-flex-item',
            template: "<ng-content></ng-content>"
        })
    ], FlexItemComponent);
    return FlexItemComponent;
}());

/**
 * @whatItDoes 给 `ui-select`、`ui-input[type=date]` 组件的下拉箭头的 html 元素 class 的名字的 token
 *
 * 你可以通过 provide 定义自己的下拉箭头 class
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_SELECT_ARROW_CLASSNAME } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_SELECT_ARROW_CLASSNAME,
 *     useValue: 'custom-classname'
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
var UI_SELECT_ARROW_CLASSNAME = new InjectionToken('UI_SELECT_ARROW_CLASSNAME');
/**
 * 把表单元素的输入属性转成 boolean 类型
 * '' => true
 * 0 => false
 * null => false
 * undefined => false
 *
 * @param value
 * @returns boolean
 */
function inputAttrToBoolean(value) {
    return value === '' || !!value;
}

var PickerService = /** @class */ (function () {
    function PickerService() {
        this.showEvent = new Subject();
        this.scrollEvent = new Subject();
        this.onShow = this.showEvent.asObservable();
        this.onScroll = this.scrollEvent.asObservable();
    }
    PickerService.prototype.show = function () {
        this.showEvent.next();
    };
    PickerService.prototype.scroll = function (b) {
        this.scrollEvent.next(b);
    };
    PickerService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], PickerService);
    return PickerService;
}());

var ButtonComponent = /** @class */ (function () {
    function ButtonComponent(renderer, elementRef) {
        this.renderer = renderer;
        this.elementRef = elementRef;
        this._loading = false;
    }
    Object.defineProperty(ButtonComponent.prototype, "loading", {
        get: function () {
            return this._loading;
        },
        set: function (v) {
            this._loading = inputAttrToBoolean(v);
            this.updateState();
        },
        enumerable: true,
        configurable: true
    });
    ButtonComponent.prototype.updateState = function () {
        this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', this._loading);
    };
    ButtonComponent.ctorParameters = function () { return [
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        HostBinding('class.ui-loading'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], ButtonComponent.prototype, "loading", null);
    ButtonComponent = __decorate([
        Component({
            /* tslint:disable */
            selector: '[ui-button]',
            /* tslint:enable */
            template: "<span class=\"ui-loading-bar\"></span>\n<ng-content></ng-content>"
        }),
        __metadata("design:paramtypes", [Renderer2,
            ElementRef])
    ], ButtonComponent);
    return ButtonComponent;
}());

var CheckboxComponent = /** @class */ (function () {
    function CheckboxComponent() {
        this.uiChange = new EventEmitter();
        this.checkedIcon = 'ui-icon-checkbox-checked';
        this.uncheckedIcon = 'ui-icon-checkbox-unchecked';
        this.text = '';
        this.value = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    CheckboxComponent_1 = CheckboxComponent;
    Object.defineProperty(CheckboxComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CheckboxComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CheckboxComponent.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (isChecked) {
            this._checked = inputAttrToBoolean(isChecked);
        },
        enumerable: true,
        configurable: true
    });
    CheckboxComponent.prototype.click = function () {
        if (this.disabled || this.readonly) {
            return;
        }
        this.checked = !this.checked;
        if (this.onChange) {
            this.onChange(this.checked);
        }
        if (this.onTouched) {
            this.onTouched();
        }
        this.uiChange.emit(this.checked);
    };
    CheckboxComponent.prototype.writeValue = function (value) {
        this.checked = !!value;
    };
    CheckboxComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    CheckboxComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    CheckboxComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var CheckboxComponent_1;
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], CheckboxComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "checkedIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "uncheckedIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "text", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], CheckboxComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-readonly'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], CheckboxComponent.prototype, "readonly", null);
    __decorate([
        Input(),
        HostBinding('class.ui-checked'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], CheckboxComponent.prototype, "checked", null);
    CheckboxComponent = CheckboxComponent_1 = __decorate([
        Component({
            selector: 'ui-input[type=checkbox]',
            template: "<div class=\"ui-checkbox\">\n  <span *ngIf=\"checked\" [ngClass]=\"checkedIcon\">{{ text }}</span>\n  <span *ngIf=\"!checked\" [ngClass]=\"uncheckedIcon\">{{ text }}</span>\n</div>\n<input class=\"ui-input-checkbox\"\n       type=\"button\"\n       [value]=\"value\"\n       (click)=\"click()\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: CheckboxComponent_1,
                    multi: true
                }]
        })
    ], CheckboxComponent);
    return CheckboxComponent;
}());

var UIDate = /** @class */ (function () {
    function UIDate(d) {
        this.d = d;
        if (typeof d === 'number') {
            this.date = new Date(d);
        }
        else if (typeof d === 'string') {
            this.date = this.stringToDate(d);
        }
        else if (d instanceof Date) {
            this.date = d;
        }
        else {
            this.date = new Date();
        }
    }
    Object.defineProperty(UIDate.prototype, "timestamp", {
        get: function () {
            return this.date.getTime();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "year", {
        get: function () {
            return this.date.getFullYear();
        },
        set: function (v) {
            this.date.setFullYear(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "month", {
        get: function () {
            return this.date.getMonth();
        },
        set: function (v) {
            var date = new Date(this.timestamp);
            date.setMonth(v + 1, 0);
            var day = date.getDate();
            if (day < this.day) {
                this.date.setMonth(v, day);
                return;
            }
            this.date.setMonth(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "day", {
        get: function () {
            return this.date.getDate();
        },
        set: function (v) {
            this.date.setDate(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "hours", {
        get: function () {
            return this.date.getHours();
        },
        set: function (v) {
            this.date.setHours(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "minutes", {
        get: function () {
            return this.date.getMinutes();
        },
        set: function (v) {
            this.date.setMinutes(v);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UIDate.prototype, "seconds", {
        get: function () {
            return this.date.getSeconds();
        },
        set: function (v) {
            this.date.setSeconds(v);
        },
        enumerable: true,
        configurable: true
    });
    UIDate.prototype.toStringByFormatString = function (s) {
        var date = new Date();
        date.setTime(this.timestamp);
        var year = date.getFullYear();
        var month = date.getMonth() + 1;
        var day = date.getDate();
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var seconds = date.getSeconds();
        return s.replace(/[yMdhms]+/g, function (str) {
            switch (str) {
                case 'yy':
                    return year ? toDouble(year % 100) : '';
                case 'yyyy':
                    return year + '';
                case 'M':
                    return month + '';
                case 'MM':
                    return toDouble(month);
                case 'd':
                    return day + '';
                case 'dd':
                    return toDouble(day);
                case 'h':
                    return hours + '';
                case 'hh':
                    return toDouble(hours);
                case 'm':
                    return minutes + '';
                case 'mm':
                    return toDouble(minutes);
                case 's':
                    return seconds + '';
                case 'ss':
                    return toDouble(seconds);
                default:
                    return str;
            }
        });
    };
    UIDate.prototype.stringToDate = function (s) {
        var dateArr = (s.match(/\d+/g) || []).map(function (item) {
            return +item;
        });
        var arr = [0, 0, 1, 0, 0, 0];
        dateArr = dateArr.concat(arr.slice(dateArr.length));
        var date = new Date();
        date.setFullYear(dateArr[0]);
        date.setMonth(dateArr[1] - 1);
        date.setDate(dateArr[2]);
        date.setHours(dateArr[3]);
        date.setMinutes(dateArr[4]);
        date.setSeconds(dateArr[5]);
        date.setMilliseconds(0);
        return date;
    };
    return UIDate;
}());
/**
 * 给0-9的数字补零，转为00、01……
 * @param n 0-9的数字
 */
function toDouble(n) {
    return n > 9 ? n + '' : '0' + n;
}

var DateComponent = /** @class */ (function () {
    function DateComponent(arrowIcon, pickerService) {
        this.pickerService = pickerService;
        this.uiChange = new EventEmitter();
        this.title = '';
        this.format = 'yyyy-MM-dd';
        this.displayFormat = '';
        this.placeholder = '';
        this.arrowIconClassName = '';
        this.focus = false;
        this.years = [];
        this.months = [];
        this.days = [];
        this.hours = [];
        this.minutes = [];
        this.seconds = [];
        this.displayValue = '';
        this._disabled = false;
        this._readonly = false;
        this.timer = null;
        this.isScrolling = false;
        this.arrowIconClassName = arrowIcon;
        this.currentDate = new UIDate();
        var d = new Date(this.currentDate.timestamp);
        d.setFullYear(d.getFullYear() + 20);
        this._maxDate = new UIDate(d);
        d = new Date(this.currentDate.timestamp);
        d.setFullYear(d.getFullYear() - 80);
        this._minDate = new UIDate(d);
    }
    DateComponent_1 = DateComponent;
    Object.defineProperty(DateComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this._value = value;
            if (value === '' || value === null || value === undefined) {
                this.displayValue = '';
            }
            else {
                this.currentDate = new UIDate(value);
                this.displayValue = this.currentDate.toStringByFormatString(this.displayFormat || this.format);
                this.initYears();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateComponent.prototype, "maxDate", {
        set: function (value) {
            this._maxDate = new UIDate(value);
            this.initYears();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateComponent.prototype, "minDate", {
        set: function (value) {
            this._minDate = new UIDate(value);
            this.initYears();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    DateComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.sub = this.pickerService.onScroll.subscribe(function (b) {
            _this.isScrolling = b;
        });
        this.initYears();
    };
    DateComponent.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    DateComponent.prototype.initYears = function () {
        if (/yy|yyyy/.test(this.format)) {
            this.years.length = 0;
            for (var i = this._minDate.year; i <= this._maxDate.year; i++) {
                this.years.push({
                    value: i,
                    text: i + '年'
                });
            }
        }
        this.initMonths();
    };
    DateComponent.prototype.initMonths = function () {
        if (/M|MM/.test(this.format)) {
            this.months.length = 0;
            var startMonth = 0;
            var endMonth = 11;
            if (this.currentDate.year <= this._minDate.year) {
                startMonth = this._minDate.month;
                if (this.currentDate.month < startMonth) {
                    this.currentDate.month = startMonth;
                }
            }
            if (this.currentDate.year >= this._maxDate.year) {
                endMonth = this._maxDate.month;
                if (this.currentDate.month > endMonth) {
                    this.currentDate.month = endMonth;
                }
            }
            for (; startMonth <= endMonth; startMonth++) {
                this.months.push({
                    value: startMonth,
                    text: startMonth + 1 + '月'
                });
            }
            this.months = this.months.slice(); // 数组复制触发视图变更
        }
        this.initDays();
    };
    DateComponent.prototype.initDays = function () {
        if (/d|dd/.test(this.format)) {
            this.days.length = 0;
            // 获取当前选中月总共有多少天
            var date = new Date();
            // 设置为当月1日，防止溢出
            date.setDate(1);
            date.setFullYear(this.currentDate.year);
            date.setMonth(this.currentDate.month + 1);
            date.setDate(0);
            var startDay = 1;
            var endDay = date.getDate();
            if (this.currentDate.year <= this._minDate.year) {
                if (this.currentDate.month <= this._minDate.month) {
                    startDay = this._minDate.day;
                }
            }
            if (this.currentDate.year >= this._maxDate.year) {
                if (this.currentDate.month >= this._maxDate.month) {
                    endDay = this._maxDate.day;
                }
            }
            for (; startDay <= endDay; startDay++) {
                this.days.push({
                    text: startDay + '日',
                    value: startDay
                });
            }
            this.days = this.days.slice(); // 数组复制触发视图变更
        }
    };
    DateComponent.prototype.show = function () {
        var _this = this;
        clearTimeout(this.timer);
        if (this.disabled || this.readonly) {
            return;
        }
        this.focus = true;
        this.timer = setTimeout(function () {
            _this.pickerService.show();
        });
    };
    DateComponent.prototype.selected = function () {
        if (!this.focus || this.isScrolling) {
            return;
        }
        var value;
        if (this.format) {
            value = this.currentDate.toStringByFormatString(this.format);
        }
        else {
            value = this.currentDate.timestamp;
        }
        this.displayValue = this.currentDate.toStringByFormatString(this.displayFormat || this.format);
        if (this.onChange) {
            this.onChange(value);
        }
        if (this.onTouched) {
            this.onTouched();
        }
        this._value = value;
        this.uiChange.emit(value);
        this.hide();
    };
    DateComponent.prototype.cellSelected = function (cell, type) {
        switch (type) {
            case 'year':
                this.currentDate.year = +cell.value;
                this.initMonths();
                break;
            case 'month':
                this.currentDate.month = +cell.value;
                this.initDays();
                break;
            case 'day':
                this.currentDate.day = +cell.value;
                break;
            case 'hours':
                this.currentDate.hours = +cell.value;
                break;
            case 'minutes':
                this.currentDate.minutes = +cell.value;
                break;
            case 'seconds':
                this.currentDate.seconds = +cell.value;
                break;
        }
    };
    DateComponent.prototype.hide = function () {
        clearTimeout(this.timer);
        this.focus = false;
    };
    DateComponent.prototype.writeValue = function (value) {
        this.value = value;
    };
    DateComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    DateComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    DateComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var DateComponent_1;
    DateComponent.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
        { type: PickerService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], DateComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "title", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "format", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "displayFormat", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "placeholder", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DateComponent.prototype, "arrowIconClassName", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateComponent.prototype, "value", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateComponent.prototype, "maxDate", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateComponent.prototype, "minDate", null);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-readonly'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateComponent.prototype, "readonly", null);
    DateComponent = DateComponent_1 = __decorate([
        Component({
            selector: 'ui-input[type=date]',
            template: "<div class=\"ui-date-fixed\" (click)=\"show()\">\n  <div class=\"ui-date-input-wrap\">\n    <input class=\"ui-date-input\" [disabled]=\"disabled\"\n           [attr.name]=\"name\"\n           [attr.id]=\"forId\"\n           [placeholder]=\"placeholder\"\n           readonly\n           type=\"text\"\n           [value]=\"displayValue\">\n    <div class=\"ui-date-input-mask\"></div>\n  </div>\n  <div class=\"ui-date-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hide()\">\n  <div class=\"ui-picker\" uiStopPropagation>\n    <ui-flex class=\"ui-picker-header\">\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"hide()\">\u53D6\u6D88</button>\n      <ui-flex-item class=\"ui-picker-title\">{{ title }}</ui-flex-item>\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"selected()\">\u786E\u8BA4</button>\n    </ui-flex>\n\n    <!-- *ngIf=\"focus\" \u89E3\u51B3dom\u9020\u6210\u7684\u5361\u987F\u95EE\u9898 -->\n    <div *ngIf=\"focus\" class=\"ui-picker-selector\">\n      <div class=\"ui-picker-list\">\n        <ui-picker-column *ngIf=\"years.length\" (uiSelected)=\"cellSelected($event, 'year')\"\n                          [cells]=\"years\" [value]=\"{value: currentDate.year}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"months.length\" (uiSelected)=\"cellSelected($event, 'month')\"\n                          [cells]=\"months\" [value]=\"{value: currentDate.month}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"days.length\" (uiSelected)=\"cellSelected($event, 'day')\"\n                          [cells]=\"days\" [value]=\"{value: currentDate.day}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"hours.length\" (uiSelected)=\"cellSelected($event, 'hours')\"\n                          [cells]=\"hours\" [value]=\"{value: currentDate.hours}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"minutes.length\" (uiSelected)=\"cellSelected($event, 'minutes')\"\n                          [cells]=\"minutes\" [value]=\"{value: currentDate.minutes}\"></ui-picker-column>\n        <ui-picker-column *ngIf=\"seconds.length\" (uiSelected)=\"cellSelected($event, 'seconds')\"\n                          [cells]=\"seconds\" [value]=\"{value: currentDate.seconds}\"></ui-picker-column>\n      </div>\n    </div>\n  </div>\n</ui-action-sheet>\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: DateComponent_1,
                    multi: true
                }, PickerService]
        }),
        __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
        __metadata("design:paramtypes", [String, PickerService])
    ], DateComponent);
    return DateComponent;
}());

var SelectService = /** @class */ (function () {
    function SelectService() {
        this.checkedSource = new Subject();
        this.onChecked = this.checkedSource.asObservable();
    }
    SelectService.prototype.checked = function (option) {
        this.checkedSource.next(option);
    };
    SelectService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], SelectService);
    return SelectService;
}());

var OptionComponent = /** @class */ (function () {
    function OptionComponent(elementRef, selectService) {
        this.elementRef = elementRef;
        this.selectService = selectService;
        this.uiChecked = new EventEmitter();
        this.value = '';
        this._disabled = false;
        this._selected = false;
    }
    Object.defineProperty(OptionComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(OptionComponent.prototype, "selected", {
        get: function () {
            return this._selected;
        },
        set: function (isSelected) {
            this._selected = inputAttrToBoolean(isSelected);
        },
        enumerable: true,
        configurable: true
    });
    OptionComponent.prototype.click = function () {
        if (!this.disabled) {
            this.selectService.checked(this);
            this.uiChecked.emit(this);
        }
    };
    OptionComponent.prototype.ngAfterViewInit = function () {
        this.nativeElement = this.elementRef.nativeElement;
    };
    OptionComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: SelectService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], OptionComponent.prototype, "uiChecked", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], OptionComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], OptionComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-selected'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], OptionComponent.prototype, "selected", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], OptionComponent.prototype, "click", null);
    OptionComponent = __decorate([
        Component({
            selector: 'ui-option',
            template: "<ng-content></ng-content>"
        }),
        __metadata("design:paramtypes", [ElementRef,
            SelectService])
    ], OptionComponent);
    return OptionComponent;
}());

var PickerComponent = /** @class */ (function () {
    function PickerComponent(arrowIcon, pickerService) {
        this.pickerService = pickerService;
        this.uiChange = new EventEmitter();
        this.arrowIconClassName = '';
        this.columnSize = 3;
        this.placeholder = '';
        this.value = [];
        this.title = '';
        this.focus = false;
        this.list = [];
        this._value = [];
        this._disabled = false;
        this._readonly = false;
        this.timer = null;
        this.isScrolling = false;
        this.arrowIconClassName = arrowIcon;
    }
    PickerComponent_1 = PickerComponent;
    Object.defineProperty(PickerComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PickerComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PickerComponent.prototype, "data", {
        get: function () {
            return this._data;
        },
        set: function (list) {
            var _this = this;
            this._data = list;
            if (this.value.length >= this.columnSize) {
                this.value.forEach(function (item) { return _this._value.push(item); });
                this.makeList(0, list);
            }
            else {
                this.list = [list];
                if (list.length) {
                    this.cellSelected(list[0], 0);
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    PickerComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.sub = this.pickerService.onScroll.subscribe(function (b) {
            _this.isScrolling = b;
        });
    };
    PickerComponent.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    PickerComponent.prototype.click = function () {
        var _this = this;
        clearTimeout(this.timer);
        if (this.disabled || this.readonly) {
            return;
        }
        this.focus = true;
        this.timer = setTimeout(function () {
            _this.pickerService.show();
        });
    };
    PickerComponent.prototype.hide = function () {
        clearTimeout(this.timer);
        this.focus = false;
    };
    PickerComponent.prototype.cellSelected = function (cell, index) {
        this._value[index] = cell;
        this._value.length = index + 1;
        var children = cell.children;
        var i = index;
        while (children) {
            this._value[++i] = children[0];
            children = children[0].children;
        }
        this.makeList(index + 1, cell.children);
    };
    PickerComponent.prototype.selected = function () {
        if (!this.focus || this.isScrolling) {
            return;
        }
        if (this.onChange) {
            this.onChange(this._value);
        }
        if (this.onTouched) {
            this.onTouched(this._value);
        }
        this.value = this._value;
        this.uiChange.emit(this._value);
        this.hide();
    };
    PickerComponent.prototype.writeValue = function (value) {
        var _this = this;
        this.value = value;
        if (value && Array.isArray(value)) {
            this.value.forEach(function (item) { return _this._value.push(item); });
            this.makeList(0, this._data);
        }
    };
    PickerComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    PickerComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    PickerComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    PickerComponent.prototype.makeList = function (startIndex, list) {
        var _this = this;
        this.list.length = startIndex;
        if (list) {
            var fn_1 = function (parentList, child, index) {
                _this.list.push(child);
                parentList.forEach(function (item) {
                    if (_this._value[index] && item.value === _this._value[index].value && item.children) {
                        var nextChild = [];
                        fn_1(item.children, nextChild, index + 1);
                    }
                    child.push(item);
                });
            };
            fn_1(list, [], startIndex);
        }
        this.completionListLength();
    };
    PickerComponent.prototype.completionListLength = function () {
        for (var i = this.list.length; i < this.columnSize; i++) {
            this.list.push([]);
        }
    };
    var PickerComponent_1;
    PickerComponent.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
        { type: PickerService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], PickerComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], PickerComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PickerComponent.prototype, "arrowIconClassName", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PickerComponent.prototype, "columnSize", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PickerComponent.prototype, "placeholder", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], PickerComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], PickerComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PickerComponent.prototype, "title", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], PickerComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], PickerComponent.prototype, "readonly", null);
    __decorate([
        Input(),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], PickerComponent.prototype, "data", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], PickerComponent.prototype, "click", null);
    PickerComponent = PickerComponent_1 = __decorate([
        Component({
            selector: 'ui-picker',
            template: "<div class=\"ui-picker-fixed\">\n  <div class=\"ui-picker-input-wrap\">\n    <div class=\"ui-picker-content\" [ngClass]=\"{'ui-show': value && value.length}\">\n      <ng-content></ng-content>\n    </div>\n    <input [ngClass]=\"{'ui-show': !value || !value.length}\"\n           type=\"text\" class=\"ui-picker-input\"\n           [attr.name]=\"name\" [attr.id]=\"forId\"\n           [disabled]=\"disabled\" readonly\n           [placeholder]=\"placeholder\">\n    <div class=\"ui-picker-input-mask\"></div>\n  </div>\n  <div class=\"ui-picker-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hide()\" uiStopPropagation>\n  <div class=\"ui-picker\" uiStopPropagation>\n    <ui-flex class=\"ui-picker-header\">\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"hide()\">\u53D6\u6D88</button>\n      <ui-flex-item class=\"ui-picker-title\">{{ title }}</ui-flex-item>\n      <button class=\"ui-picker-action\" type=\"button\" (click)=\"selected()\">\u786E\u8BA4</button>\n    </ui-flex>\n    <div class=\"ui-picker-selector\" *ngIf=\"focus\">\n      <div class=\"ui-picker-list\">\n        <!--\n        <ui-picker-column *ngFor=\"let column of list; let i = index\"\n                          (uiSelected)=\"cellSelected($event, i)\"\n                          [cells]=\"column\" [value]=\"value[i]\"></ui-picker-column>\n        -->\n        <!-- ngFor \u4F1A\u5BFC\u81F4\u56DE\u586B\u9009\u4E2Dbug -->\n        <ui-picker-column *ngIf=\"list.length && list[0]\" (uiSelected)=\"cellSelected($event, 0)\"\n                          [cells]=\"list[0]\" [value]=\"value[0] || list[0][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[1]\" (uiSelected)=\"cellSelected($event, 1)\"\n                          [cells]=\"list[1]\" [value]=\"value[1] || list[1][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[2]\" (uiSelected)=\"cellSelected($event, 2)\"\n                          [cells]=\"list[2]\" [value]=\"value[2] || list[2][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[3]\" (uiSelected)=\"cellSelected($event, 3)\"\n                          [cells]=\"list[3]\" [value]=\"value[3] || list[3][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[4]\" (uiSelected)=\"cellSelected($event, 4)\"\n                          [cells]=\"list[4]\" [value]=\"value[4] || list[4][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[5]\" (uiSelected)=\"cellSelected($event, 5)\"\n                          [cells]=\"list[5]\" [value]=\"value[5] || list[5][0]\"></ui-picker-column>\n\n        <ui-picker-column *ngIf=\"list.length && list[6]\" (uiSelected)=\"cellSelected($event, 6)\"\n                          [cells]=\"list[6]\" [value]=\"value[6] || list[6][0]\"></ui-picker-column>\n      </div>\n    </div>\n  </div>\n</ui-action-sheet>\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: PickerComponent_1,
                    multi: true
                },
                PickerService
            ]
        }),
        __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
        __metadata("design:paramtypes", [String, PickerService])
    ], PickerComponent);
    return PickerComponent;
}());

BScroll.use(Wheel);
var PickerColumnComponent = /** @class */ (function () {
    function PickerColumnComponent(elementRef, scale, pickerService) {
        this.elementRef = elementRef;
        this.scale = scale;
        this.pickerService = pickerService;
        this._cells = [];
        this.uiSelected = new EventEmitter();
        this.cellHeight = 36;
        this.distanceTop = 0;
        this.subs = [];
    }
    Object.defineProperty(PickerColumnComponent.prototype, "cells", {
        get: function () {
            return this._cells;
        },
        set: function (v) {
            var _this = this;
            this._cells = v || [];
            /** 视图更新，重新计算 BetterScroll */
            if (this.scrollInstance) {
                setTimeout(function () {
                    console.log(v);
                    _this.scrollInstance.refresh();
                }, 500);
            }
        },
        enumerable: true,
        configurable: true
    });
    PickerColumnComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.initScroll();
        this.subs.push(this.pickerService.onShow.subscribe(function () {
            if (_this.scrollInstance) {
                _this.scrollInstance.refresh();
                var selectedIndex = 0;
                for (var i = 0; i < _this.cells.length; i++) {
                    if (_this.cells[i].value === _this.value.value) {
                        selectedIndex = i;
                        break;
                    }
                }
                _this.scrollInstance.scrollTo(0, -selectedIndex * (_this.cellHeight * _this.scale));
                return;
            }
            _this.initScroll();
        }));
    };
    PickerColumnComponent.prototype.initScroll = function () {
        var _this = this;
        this.scrollInstance = new BScroll(this.elementRef.nativeElement, {
            wheel: {
                selectedIndex: 0,
                rotate: 0,
                wheelWrapperClass: 'ui-picker-column',
                wheelItemClass: 'ui-picker-cell'
            },
            probeType: 1
        });
        this.scrollInstance.on('scrollStart', function () {
            _this.pickerService.scroll(true);
        });
        this.scrollInstance.on('scrollEnd', function (position) {
            _this.pickerService.scroll(false);
            var value = _this.cells[Math.floor(Math.abs(position.y / (_this.cellHeight * _this.scale)))];
            if (value.disabled) {
                return;
            }
            _this.uiSelected.emit(value);
        });
    };
    PickerColumnComponent.prototype.ngOnDestroy = function () {
        if (this.scrollInstance) {
            this.scrollInstance.destroy();
        }
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    PickerColumnComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: PickerService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], PickerColumnComponent.prototype, "uiSelected", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], PickerColumnComponent.prototype, "cells", null);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], PickerColumnComponent.prototype, "cellHeight", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], PickerColumnComponent.prototype, "value", void 0);
    PickerColumnComponent = __decorate([
        Component({
            selector: 'ui-picker-column',
            template: "<div class=\"ui-picker-column\"\n     [ngStyle]=\"{transform: 'translateY(' + distanceTop + 'px)'}\">\n  <button type=\"button\"\n          *ngFor=\"let cell of cells\"\n          [disabled]=\"cell.disabled\"\n          class=\"ui-picker-cell\">{{ cell.text }}\n  </button>\n</div>\n"
        }),
        __param(1, Inject(UI_SCREEN_SCALE)),
        __metadata("design:paramtypes", [ElementRef, Number, PickerService])
    ], PickerColumnComponent);
    return PickerColumnComponent;
}());

var RadioStateService = /** @class */ (function () {
    function RadioStateService() {
        this.stateSource = new Subject();
        this.state = this.stateSource.asObservable();
    }
    RadioStateService.prototype.publishEvent = function () {
        this.stateSource.next();
    };
    RadioStateService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], RadioStateService);
    return RadioStateService;
}());

var RadioComponent = /** @class */ (function () {
    function RadioComponent(radioStateService) {
        this.radioStateService = radioStateService;
        this.uiChange = new EventEmitter();
        this.checkedIcon = 'ui-icon-radio-checked';
        this.uncheckedIcon = 'ui-icon-radio-unchecked';
        this.value = '';
        this.text = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    RadioComponent_1 = RadioComponent;
    Object.defineProperty(RadioComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RadioComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RadioComponent.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (isChecked) {
            this._checked = inputAttrToBoolean(isChecked);
        },
        enumerable: true,
        configurable: true
    });
    RadioComponent.prototype.click = function () {
        if (this.disabled || this.readonly) {
            return;
        }
        this.rawInput.nativeElement.checked = true;
        if (this.onChange) {
            this.onChange(this.value);
        }
        if (this.onTouched) {
            this.onTouched(this.value);
        }
        // 当自身被点击时，发布事件，更新其它radio状态
        this.radioStateService.publishEvent();
        this.uiChange.emit(this.value);
    };
    RadioComponent.prototype.ngOnInit = function () {
        var _this = this;
        // 当某一个radio被点击时，更新其它radio状态
        this.sub = this.radioStateService.state.subscribe(function () {
            _this.checked = _this.rawInput.nativeElement.checked;
        });
    };
    RadioComponent.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    RadioComponent.prototype.writeValue = function (value) {
        if (typeof value === 'number') {
            this.checked = this.value === ('' + value);
        }
        else {
            this.checked = this.value === value;
        }
    };
    RadioComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    RadioComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    RadioComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var RadioComponent_1;
    RadioComponent.ctorParameters = function () { return [
        { type: RadioStateService }
    ]; };
    __decorate([
        ViewChild('rawInput', { static: true }),
        __metadata("design:type", ElementRef)
    ], RadioComponent.prototype, "rawInput", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], RadioComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "checkedIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "uncheckedIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RadioComponent.prototype, "text", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RadioComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-readonly'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RadioComponent.prototype, "readonly", null);
    __decorate([
        Input(),
        HostBinding('class.ui-checked'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RadioComponent.prototype, "checked", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], RadioComponent.prototype, "click", null);
    RadioComponent = RadioComponent_1 = __decorate([
        Component({
            selector: 'ui-input[type=radio]',
            template: "<div class=\"ui-radio\">\n  <span *ngIf=\"checked\" [ngClass]=\"checkedIcon\">{{ text }}</span>\n  <span *ngIf=\"!checked\" [ngClass]=\"uncheckedIcon\">{{ text }}</span>\n</div>\n<input #rawInput class=\"ui-input-radio\"\n       type=\"radio\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: RadioComponent_1,
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [RadioStateService])
    ], RadioComponent);
    return RadioComponent;
}());

var RangeComponent = /** @class */ (function () {
    function RangeComponent(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.uiChange = new EventEmitter();
        this.showProgress = false;
        this.position = 50;
        this.isTouching = false;
        this._disabled = false;
        this._readonly = false;
        this._min = 0;
        this._max = 100;
        this._step = 1;
        this._value = 50;
    }
    RangeComponent_1 = RangeComponent;
    Object.defineProperty(RangeComponent.prototype, "disabled", {
        get: function () {
            return this._readonly;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeComponent.prototype, "min", {
        get: function () {
            return this._min;
        },
        set: function (min) {
            var v = RangeComponent_1.toNumber(min);
            if (!isNaN(v)) {
                this._min = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeComponent.prototype, "max", {
        get: function () {
            return this._max;
        },
        set: function (max) {
            var v = RangeComponent_1.toNumber(max);
            if (!isNaN(v)) {
                this._max = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeComponent.prototype, "step", {
        get: function () {
            return this._step;
        },
        set: function (step) {
            var v = RangeComponent_1.toNumber(step);
            if (!isNaN(v)) {
                this._step = v;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RangeComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            var v = RangeComponent_1.toNumber(value);
            if (!isNaN(v)) {
                this._value = v;
                if (this.min <= this.max) {
                    if (v < this.min) {
                        v = this.min;
                    }
                    else if (v > this.max) {
                        v = this.max;
                    }
                    this.position = (v - this.min) / (this.max - this.min) * 100;
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    RangeComponent.toNumber = function (value) {
        if (typeof value === 'number') {
            return value;
        }
        return Number(value);
    };
    RangeComponent.prototype.drag = function (event) {
        var _this = this;
        if (this.readonly || this.disabled) {
            return;
        }
        this.isTouching = true;
        var unbindTouchEndFn;
        var unbindTouchCancelFn;
        var unbindTouchMoveFn;
        if (this.min >= this.max) {
            return;
        }
        var element = event.target;
        unbindTouchEndFn = this.renderer.listen(element, 'touchend', function () {
            _this.isTouching = false;
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
        });
        unbindTouchCancelFn = this.renderer.listen(element, 'touchcancel', function () {
            _this.isTouching = false;
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
        });
        var section = this.max - this.min;
        var maxWidth = this.elementRef.nativeElement.offsetWidth;
        var nowWidth = this.rangeBar.nativeElement.offsetWidth;
        var oldX = event.touches[0].clientX;
        var oldValue = this.value;
        unbindTouchMoveFn = this.renderer.listen(element, 'touchmove', function (ev) {
            var dragDistance = ev.touches[0].clientX - oldX;
            var proportion = (nowWidth + dragDistance) / maxWidth;
            var temporaryValue = Math.floor(section * proportion / _this.step) * _this.step;
            var value = _this.min + temporaryValue;
            if (value < _this.min) {
                value = _this.min;
            }
            else if (value > _this.max) {
                value = _this.max - (_this.max - _this.min) % _this.step;
            }
            if (value !== oldValue) {
                _this.value = value;
                oldValue = value;
                if (_this.onChange) {
                    _this.onChange(value);
                }
                if (_this.onTouched) {
                    _this.onTouched(value);
                }
                _this.uiChange.emit(value);
            }
            ev.stopPropagation();
        });
    };
    RangeComponent.prototype.writeValue = function (value) {
        this.value = value;
    };
    RangeComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    RangeComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    RangeComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var RangeComponent_1;
    RangeComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    __decorate([
        ViewChild('rangeBar', { static: true }),
        __metadata("design:type", ElementRef)
    ], RangeComponent.prototype, "rangeBar", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], RangeComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RangeComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RangeComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RangeComponent.prototype, "showProgress", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-readonly'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "readonly", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "min", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "max", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "step", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RangeComponent.prototype, "value", null);
    RangeComponent = RangeComponent_1 = __decorate([
        Component({
            selector: 'ui-input[type=range]',
            template: "<div class=\"ui-range\">\n  <div class=\"ui-range-progress-bar\" #rangeBar [ngStyle]=\"{width: position + '%'}\">\n    <div class=\"ui-range-btn\">\n      <span class=\"ui-range-progress\" *ngIf=\"showProgress\" [ngClass]=\"{'ui-touched': isTouching}\">{{ value }}</span>\n      <button class=\"ui-range-btn-raw\" type=\"button\" (touchstart)=\"drag($event)\">\n      </button>\n    </div>\n  </div>\n</div>\n<input class=\"ui-input-range-raw\"\n       type=\"range\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [max]=\"max\"\n       [min]=\"min\"\n       [step]=\"step\"\n       [readonly]=\"readonly\">",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: RangeComponent_1,
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [ElementRef,
            Renderer2])
    ], RangeComponent);
    return RangeComponent;
}());

var SelectComponent = /** @class */ (function () {
    function SelectComponent(arrowIcon, selectService, changeDetectorRef) {
        this.selectService = selectService;
        this.changeDetectorRef = changeDetectorRef;
        this.uiChange = new EventEmitter();
        this.placeholder = '';
        this.selectedIndex = 0;
        this.cancelText = '取消';
        this.arrowIconClassName = '';
        this.focus = false;
        this.text = '';
        this._disabled = false;
        this._readonly = false;
        this.value = '';
        this.subs = [];
        this.isWrite = false;
        this.arrowIconClassName = arrowIcon;
    }
    SelectComponent_1 = SelectComponent;
    Object.defineProperty(SelectComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    SelectComponent.getTextByElement = function (element) {
        if (element) {
            return element.innerText.replace(/^[\s\n\t\r]+|[\s\n\t\r]+$/g, '');
        }
        return '';
    };
    SelectComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        if (!this.isWrite) {
            this.updateBySelf();
        }
        this.subs.push(this.selectService.onChecked.subscribe(function (option) {
            _this.options.forEach(function (op, index) {
                if (op === option) {
                    if (op !== _this.selectedOption) {
                        _this.selectedOption = op;
                        op.selected = true;
                        _this.value = option.value;
                        _this.text = SelectComponent_1.getTextByElement(option.nativeElement);
                        _this.selectedIndex = index;
                        if (_this.onChange) {
                            _this.onChange(_this.value);
                        }
                        if (_this.onTouched) {
                            _this.onTouched();
                        }
                        _this.uiChange.emit(_this.value);
                    }
                }
                else {
                    op.selected = false;
                }
            });
        }));
        this.subs.push(this.options.changes.pipe(delay(0)).subscribe(function () {
            _this.updateByNgModel();
        }));
    };
    SelectComponent.prototype.ngAfterViewInit = function () {
        if (this.selectedOption) {
            this.text = SelectComponent_1.getTextByElement(this.selectedOption.nativeElement);
            this.changeDetectorRef.detectChanges();
        }
    };
    SelectComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    SelectComponent.prototype.showOptions = function () {
        if (!this.disabled && !this.readonly) {
            this.focus = true;
        }
    };
    SelectComponent.prototype.hideOptions = function () {
        this.focus = false;
    };
    SelectComponent.prototype.writeValue = function (value) {
        var _this = this;
        this.isWrite = true;
        this.value = value;
        if (this.options) {
            var selectedOption_1;
            this.options.forEach(function (item, index) {
                item.selected = false;
                if (item.value === value || "" + item.value === value || item.value === "" + value) {
                    selectedOption_1 = item;
                    _this.selectedIndex = index;
                }
            });
            if (selectedOption_1) {
                this.text = SelectComponent_1.getTextByElement(selectedOption_1.nativeElement);
                selectedOption_1.selected = true;
            }
            else {
                this.selectedOption = null;
                this.text = '';
            }
        }
        else {
            this.selectedOption = null;
            this.text = '';
        }
    };
    SelectComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SelectComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SelectComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    SelectComponent.prototype.updateByNgModel = function () {
        var _this = this;
        var selectedOption;
        this.options.forEach(function (item, index) {
            item.selected = false;
            if (item.value === _this.value || "" + item.value === _this.value || item.value === "" + _this.value) {
                selectedOption = item;
                _this.selectedIndex = index;
            }
        });
        if (selectedOption) {
            this.text = SelectComponent_1.getTextByElement(selectedOption.nativeElement);
            selectedOption.selected = true;
            this.selectedOption = selectedOption;
        }
        else {
            this.selectedIndex = -1;
            this.text = '';
            this.selectedOption = null;
        }
    };
    SelectComponent.prototype.updateBySelf = function () {
        var _this = this;
        var defaultOption;
        this.options.forEach(function (option, index) {
            if (option.selected) {
                defaultOption = option;
                _this.selectedIndex = index;
            }
        });
        if (!defaultOption) {
            defaultOption = this.options.toArray()[this.selectedIndex];
        }
        if (!defaultOption) {
            defaultOption = this.options.first;
            this.selectedIndex = 0;
        }
        if (defaultOption) {
            this.value = defaultOption.value;
            setTimeout(function () {
                if (!_this.isWrite) {
                    defaultOption.selected = true;
                }
            });
            this.selectedOption = defaultOption;
        }
    };
    var SelectComponent_1;
    SelectComponent.ctorParameters = function () { return [
        { type: String, decorators: [{ type: Inject, args: [UI_SELECT_ARROW_CLASSNAME,] }] },
        { type: SelectService },
        { type: ChangeDetectorRef }
    ]; };
    __decorate([
        ContentChildren(OptionComponent),
        __metadata("design:type", QueryList)
    ], SelectComponent.prototype, "options", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SelectComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SelectComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "placeholder", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "selectedIndex", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "cancelText", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "arrowIconClassName", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SelectComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-readonly'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SelectComponent.prototype, "readonly", null);
    SelectComponent = SelectComponent_1 = __decorate([
        Component({
            selector: 'ui-select',
            template: "<div class=\"ui-select-fixed\" (click)=\"showOptions()\">\n  <div class=\"ui-raw-input-wrap\">\n    <input class=\"ui-raw-input\" [disabled]=\"disabled\"\n           [attr.name]=\"name\"\n           [placeholder]=\"placeholder\"\n           readonly\n           [attr.id]=\"forId\"\n           type=\"text\"\n           [value]=\"text\">\n    <div class=\"ui-raw-input-mask\"></div>\n  </div>\n  <div class=\"ui-select-arrow\" *ngIf=\"arrowIconClassName\">\n    <span [ngClass]=\"arrowIconClassName\"></span>\n  </div>\n</div>\n<ui-action-sheet [show]=\"focus\" (click)=\"hideOptions()\" uiStopPropagation=\"touchmove\">\n  <div class=\"ui-select-modal\">\n    <div class=\"ui-select-options\">\n      <ng-content></ng-content>\n    </div>\n    <div class=\"ui-select-btn-wrap\" *ngIf=\"cancelText\">\n      <button type=\"button\" class=\"ui-select-btn\">{{ cancelText }}</button>\n    </div>\n  </div>\n</ui-action-sheet>\n",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: SelectComponent_1,
                    multi: true
                }, SelectService
            ]
        }),
        __param(0, Inject(UI_SELECT_ARROW_CLASSNAME)),
        __metadata("design:paramtypes", [String, SelectService,
            ChangeDetectorRef])
    ], SelectComponent);
    return SelectComponent;
}());

var SwitchComponent = /** @class */ (function () {
    function SwitchComponent() {
        this.uiChange = new EventEmitter();
        this.value = '';
        this.name = '';
        this._disabled = false;
        this._readonly = false;
        this._checked = false;
    }
    SwitchComponent_1 = SwitchComponent;
    Object.defineProperty(SwitchComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SwitchComponent.prototype, "readonly", {
        get: function () {
            return this._readonly;
        },
        set: function (isReadonly) {
            this._readonly = inputAttrToBoolean(isReadonly);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SwitchComponent.prototype, "checked", {
        get: function () {
            return this._checked;
        },
        set: function (isChecked) {
            this._checked = inputAttrToBoolean(isChecked);
        },
        enumerable: true,
        configurable: true
    });
    SwitchComponent.prototype.click = function () {
        if (this.disabled || this.readonly) {
            return;
        }
        this.checked = !this.checked;
        if (this.onChange) {
            this.onChange(this.checked);
        }
        if (this.onTouched) {
            this.onTouched(this.checked);
        }
        this.uiChange.emit(this.checked);
    };
    SwitchComponent.prototype.writeValue = function (value) {
        this.checked = !!value;
    };
    SwitchComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SwitchComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SwitchComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    var SwitchComponent_1;
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SwitchComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SwitchComponent.prototype, "forId", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SwitchComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], SwitchComponent.prototype, "name", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SwitchComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SwitchComponent.prototype, "readonly", null);
    __decorate([
        Input(),
        HostBinding('class.ui-checked'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SwitchComponent.prototype, "checked", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], SwitchComponent.prototype, "click", null);
    SwitchComponent = SwitchComponent_1 = __decorate([
        Component({
            selector: 'ui-switch',
            template: "<button type=\"button\" class=\"ui-switch\">\n  <div class=\"ui-switch-btn\"></div>\n</button>\n<input class=\"ui-input-raw\"\n       type=\"radio\"\n       [value]=\"value\"\n       [attr.id]=\"forId\"\n       [attr.name]=\"name\"\n       [disabled]=\"disabled\"\n       [checked]=\"checked\"\n       [readonly]=\"readonly\">",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: SwitchComponent_1,
                    multi: true
                }]
        })
    ], SwitchComponent);
    return SwitchComponent;
}());

var CheckboxRequiredValidator = /** @class */ (function () {
    function CheckboxRequiredValidator() {
    }
    CheckboxRequiredValidator_1 = CheckboxRequiredValidator;
    Object.defineProperty(CheckboxRequiredValidator.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value !== false && value !== null;
            if (this._onChange) {
                this._onChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    CheckboxRequiredValidator.prototype.validate = function (c) {
        if (this.required) {
            return c.value === true ? null : { 'required': true };
        }
        return null;
    };
    CheckboxRequiredValidator.prototype.registerOnValidatorChange = function (fn) {
        this._onChange = fn;
    };
    var CheckboxRequiredValidator_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], CheckboxRequiredValidator.prototype, "required", null);
    CheckboxRequiredValidator = CheckboxRequiredValidator_1 = __decorate([
        Directive({
            /* tslint:disable */
            selector: 'ui-input[required][type=checkbox]',
            /* tslint:enable */
            providers: [{
                    provide: NG_VALIDATORS,
                    useExisting: CheckboxRequiredValidator_1,
                    multi: true
                }]
        })
    ], CheckboxRequiredValidator);
    return CheckboxRequiredValidator;
}());
var SwitchRequiredValidator = /** @class */ (function () {
    function SwitchRequiredValidator() {
    }
    SwitchRequiredValidator_1 = SwitchRequiredValidator;
    Object.defineProperty(SwitchRequiredValidator.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value !== false && value !== null;
            if (this._onChange) {
                this._onChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    SwitchRequiredValidator.prototype.validate = function (c) {
        if (this.required) {
            return c.value === true ? null : { 'required': true };
        }
        return null;
    };
    SwitchRequiredValidator.prototype.registerOnValidatorChange = function (fn) {
        this._onChange = fn;
    };
    var SwitchRequiredValidator_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SwitchRequiredValidator.prototype, "required", null);
    SwitchRequiredValidator = SwitchRequiredValidator_1 = __decorate([
        Directive({
            /* tslint:disable */
            selector: 'ui-switch[required]',
            /* tslint:enable */
            providers: [{
                    provide: NG_VALIDATORS,
                    useExisting: SwitchRequiredValidator_1,
                    multi: true
                }]
        })
    ], SwitchRequiredValidator);
    return SwitchRequiredValidator;
}());
var RadioRequiredValidator = /** @class */ (function () {
    function RadioRequiredValidator() {
    }
    RadioRequiredValidator_1 = RadioRequiredValidator;
    Object.defineProperty(RadioRequiredValidator.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value !== false && value !== null;
            if (this._onChange) {
                this._onChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    RadioRequiredValidator.prototype.validate = function (c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    };
    RadioRequiredValidator.prototype.registerOnValidatorChange = function (fn) {
        this._onChange = fn;
    };
    var RadioRequiredValidator_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], RadioRequiredValidator.prototype, "required", null);
    RadioRequiredValidator = RadioRequiredValidator_1 = __decorate([
        Directive({
            /* tslint:disable */
            selector: 'ui-input[required][type=radio]',
            /* tslint:enable */
            providers: [{
                    provide: NG_VALIDATORS,
                    useExisting: RadioRequiredValidator_1,
                    multi: true
                }]
        })
    ], RadioRequiredValidator);
    return RadioRequiredValidator;
}());
var SelectRequiredValidator = /** @class */ (function () {
    function SelectRequiredValidator() {
    }
    SelectRequiredValidator_1 = SelectRequiredValidator;
    Object.defineProperty(SelectRequiredValidator.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value !== false && value !== null;
            if (this._onChange) {
                this._onChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    SelectRequiredValidator.prototype.validate = function (c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    };
    SelectRequiredValidator.prototype.registerOnValidatorChange = function (fn) {
        this._onChange = fn;
    };
    var SelectRequiredValidator_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SelectRequiredValidator.prototype, "required", null);
    SelectRequiredValidator = SelectRequiredValidator_1 = __decorate([
        Directive({
            /* tslint:disable */
            selector: 'ui-select[required]',
            /* tslint:enable */
            providers: [{
                    provide: NG_VALIDATORS,
                    useExisting: SelectRequiredValidator_1,
                    multi: true
                }]
        })
    ], SelectRequiredValidator);
    return SelectRequiredValidator;
}());
var DateRequiredValidator = /** @class */ (function () {
    function DateRequiredValidator() {
    }
    DateRequiredValidator_1 = DateRequiredValidator;
    Object.defineProperty(DateRequiredValidator.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value !== false && value !== null;
            if (this._onChange) {
                this._onChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    DateRequiredValidator.prototype.validate = function (c) {
        if (this.required) {
            return c.value !== '' && c.value !== undefined && c.value !== null ? null : { 'required': true };
        }
        return null;
    };
    DateRequiredValidator.prototype.registerOnValidatorChange = function (fn) {
        this._onChange = fn;
    };
    var DateRequiredValidator_1;
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DateRequiredValidator.prototype, "required", null);
    DateRequiredValidator = DateRequiredValidator_1 = __decorate([
        Directive({
            /* tslint:disable */
            selector: 'ui-input[type=date][required]',
            /* tslint:enable */
            providers: [{
                    provide: NG_VALIDATORS,
                    useExisting: DateRequiredValidator_1,
                    multi: true
                }]
        })
    ], DateRequiredValidator);
    return DateRequiredValidator;
}());

var SubmitDirective = /** @class */ (function () {
    function SubmitDirective(ngForm) {
        this.ngForm = ngForm;
        this.uiSubmit = new EventEmitter();
    }
    SubmitDirective.prototype.ngOnInit = function () {
        var _this = this;
        var group = this.ngForm.form;
        this.sub = this.ngForm.ngSubmit.subscribe(function (ev) {
            if (!_this.ngForm.valid) {
                _this.markAsTouched(group);
            }
            else {
                _this.uiSubmit.emit(ev);
            }
        });
    };
    SubmitDirective.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    SubmitDirective.prototype.markAsTouched = function (group) {
        var _this = this;
        Object.keys(group.controls).map(function (field) {
            var control = group.get(field);
            if (control instanceof FormControl) {
                control.markAsDirty({ onlySelf: true });
                control.markAsTouched({ onlySelf: true });
            }
            else if (control instanceof FormGroup) {
                _this.markAsTouched(control);
            }
        });
    };
    SubmitDirective.ctorParameters = function () { return [
        { type: NgForm, decorators: [{ type: Self }, { type: Optional }] }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SubmitDirective.prototype, "uiSubmit", void 0);
    SubmitDirective = __decorate([
        Directive({
            /*tslint:disable*/
            selector: '[uiSubmit]'
            /*tslint:enable*/
        }),
        __param(0, Self()), __param(0, Optional()),
        __metadata("design:paramtypes", [NgForm])
    ], SubmitDirective);
    return SubmitDirective;
}());

var ImageViewerController = /** @class */ (function () {
    function ImageViewerController() {
        this.imageSource = new Subject();
        this.displayEvent = new Subject();
        this.imageList = [];
        this.images = this.imageSource.asObservable();
        this.display = this.displayEvent.asObservable();
    }
    ImageViewerController.prototype.addImage = function (src, srcElement) {
        if (this.findIndex(src) === -1) {
            this.imageList.push({
                srcElement: srcElement,
                src: src,
                scale: 1
            });
            this.imageSource.next(this.imageList);
        }
    };
    ImageViewerController.prototype.removeImage = function (src) {
        var index = this.findIndex(src);
        if (index > -1) {
            this.imageList.splice(index, 1);
            this.imageSource.next(this.imageList);
        }
    };
    ImageViewerController.prototype.show = function (firstSrc) {
        this.displayEvent.next({
            firstSrc: firstSrc,
            show: true
        });
    };
    ImageViewerController.prototype.hide = function () {
        this.displayEvent.next({
            show: false
        });
    };
    ImageViewerController.prototype.findIndex = function (src) {
        for (var i = 0; i < this.imageList.length; i++) {
            if (this.imageList[i].src === src) {
                return i;
            }
        }
        return -1;
    };
    ImageViewerController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ImageViewerController);
    return ImageViewerController;
}());

var ImageViewableDirective = /** @class */ (function () {
    function ImageViewableDirective(imageViewerService, elementRef) {
        this.imageViewerService = imageViewerService;
        this.elementRef = elementRef;
    }
    Object.defineProperty(ImageViewableDirective.prototype, "src", {
        get: function () {
            return this._src;
        },
        set: function (v) {
            if (v && typeof v === 'string') {
                this.imageViewerService.removeImage(this._src);
                this._src = v;
                this.imageViewerService.addImage(v, this.elementRef.nativeElement);
            }
        },
        enumerable: true,
        configurable: true
    });
    ImageViewableDirective.prototype.ngOnDestroy = function () {
        this.imageViewerService.removeImage(this.src);
    };
    ImageViewableDirective.prototype.click = function () {
        this.imageViewerService.show(this.src);
    };
    ImageViewableDirective.ctorParameters = function () { return [
        { type: ImageViewerController },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String),
        __metadata("design:paramtypes", [String])
    ], ImageViewableDirective.prototype, "src", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ImageViewableDirective.prototype, "click", null);
    ImageViewableDirective = __decorate([
        Directive({
            selector: '[uiImageViewable]'
        }),
        __metadata("design:paramtypes", [ImageViewerController,
            ElementRef])
    ], ImageViewableDirective);
    return ImageViewableDirective;
}());

var ImageViewerComponent = /** @class */ (function () {
    function ImageViewerComponent(imageViewerService, renderer2, elementRef) {
        this.imageViewerService = imageViewerService;
        this.renderer2 = renderer2;
        this.elementRef = elementRef;
        this.isShow = false;
        this.bgColor = '';
        this.isShowSlide = false;
        this.images = [];
        this.viewIndex = 0;
        this.opacity = 0;
        this.subs = [];
    }
    Object.defineProperty(ImageViewerComponent.prototype, "transition", {
        get: function () {
            return this.bgColor ? 'none' : '';
        },
        enumerable: true,
        configurable: true
    });
    ImageViewerComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.imageViewerService.images.subscribe(function (images) {
            _this.images = images;
        }));
        this.subs.push(this.imageViewerService.display.subscribe(function (config) {
            _this.isShow = config.show;
            if (config.show) {
                _this.isShowSlide = true;
                var index = _this.findIndex(config.firstSrc);
                _this.viewIndex = index > -1 ? index : 0;
                _this.setupInitStyles();
            }
        }));
    };
    ImageViewerComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    ImageViewerComponent.prototype.click = function () {
        var _this = this;
        var current = this.images[this.viewIndex];
        if (current.srcElement) {
            var rect = current.srcElement.getBoundingClientRect();
            this.renderer2.setStyle(current.srcElement, 'opacity', 0);
            var fn_1 = this.renderer2.listen(this.imgElements.toArray()[this.viewIndex].nativeElement, 'transitionend', function () {
                _this.opacity = 0;
                _this.isShowSlide = false;
                _this.renderer2.setStyle(current.srcElement, 'opacity', 1);
                fn_1();
            });
            current.styles = {
                left: rect.left,
                top: rect.top,
                width: rect.width,
                height: rect.height,
                transition: 'all .3s'
            };
        }
        this.imageViewerService.hide();
    };
    ImageViewerComponent.prototype.pinch = function (ev, imageView) {
        if (ev.first) {
            var imgRect = this.imgElements.toArray()[this.viewIndex].nativeElement.getBoundingClientRect();
            imageView.styles.transformOrigin = ev.moveX - imgRect.left + "px " + (ev.moveY - imgRect.top) + "px";
        }
        if (ev.type === 'touchmove') {
            var transforms = [
                "scale(" + ev.scale + ")",
                "translate(" + ev.distanceX / ev.scale + "px, " + ev.distanceY / ev.scale + "px)"
            ];
            imageView.styles.transition = 'none';
            imageView.styles.transform = transforms.join(' ');
        }
        else if (ev.type === 'touchend') {
            var imgRect = this.imgElements.toArray()[this.viewIndex].nativeElement.getBoundingClientRect();
            imageView.styles = {
                width: imgRect.width,
                height: imgRect.height,
                left: imgRect.left,
                top: imgRect.top
            };
        }
        ev.srcEvent.stopPropagation();
    };
    ImageViewerComponent.prototype.drag = function (ev, imageView) {
        var _this = this;
        if (ev.firstDirection === 'down') {
            var scale = ev.cumulativeDistanceY < 0 ? 1 :
                (1 - ev.cumulativeDistanceY / 4 / Number(imageView.styles.height));
            var transforms = [
                "translate(" + ev.cumulativeDistanceX + "px, " + ev.cumulativeDistanceY + "px)",
                "scale(" + scale + ")"
            ];
            imageView.styles.transition = 'none';
            imageView.styles.transform = transforms.join(' ');
            var alpha = ev.cumulativeDistanceY < 0 ? 1 : (1 - ev.cumulativeDistanceY / Number(imageView.styles.height));
            this.bgColor = "rgba(0,0,0," + (alpha < 0.5 ? 0.5 : alpha) + ")";
            if (ev.type === 'touchend') {
                this.bgColor = '';
                ev.resetCumulative();
                if (ev.cumulativeDistanceY > this.elementRef.nativeElement.offsetHeight / 4) {
                    this.imageViewerService.hide();
                    var rect = imageView.srcElement.getBoundingClientRect();
                    var fn_2 = this.renderer2.listen(this.imgElements.toArray()[this.viewIndex].nativeElement, 'transitionend', function () {
                        _this.opacity = 0;
                        _this.isShowSlide = false;
                        _this.renderer2.setStyle(imageView.srcElement, 'opacity', 1);
                        fn_2();
                    });
                    imageView.styles = {
                        width: rect.width,
                        height: rect.height,
                        left: rect.left,
                        top: rect.top,
                        transition: 'all .3s'
                    };
                }
                else {
                    imageView.styles.transform = 'none';
                    imageView.styles.transition = 'all .3s';
                }
            }
        }
        else {
            ev.resetCumulative();
            ev.stop();
        }
    };
    ImageViewerComponent.prototype.setViewIndex = function (index) {
        this.viewIndex = index;
    };
    ImageViewerComponent.prototype.findIndex = function (src) {
        for (var i = 0; i < this.images.length; i++) {
            if (this.images[i].src === src) {
                return i;
            }
        }
        return -1;
    };
    ImageViewerComponent.prototype.setupInitStyles = function () {
        var _this = this;
        this.images.forEach(function (item, index) {
            item.styles = {};
            if (index === _this.viewIndex) {
                if (item.srcElement) {
                    var rect = item.srcElement.getBoundingClientRect();
                    item.styles = {
                        left: rect.left,
                        top: rect.top,
                        width: rect.width,
                        height: rect.height,
                        transition: 'all .3s'
                    };
                }
            }
            requestAnimationFrame(function () {
                if (item.srcElement) {
                    if (index === _this.viewIndex) {
                        _this.renderer2.setStyle(item.srcElement, 'opacity', 0);
                        var fn_3 = _this.renderer2.listen(_this.imgElements.toArray()[_this.viewIndex].nativeElement, 'transitionend', function () {
                            _this.renderer2.setStyle(item.srcElement, 'opacity', 1);
                            item.styles.transition = 'none';
                            fn_3();
                        });
                    }
                    var containerRect = _this.elementRef.nativeElement.getBoundingClientRect();
                    var contentWidth = item.srcElement.naturalWidth;
                    var contentHeight = item.srcElement.naturalHeight;
                    var proportion = (containerRect.width / containerRect.height) / (contentWidth / contentHeight);
                    item.defaultStyles = {};
                    if (contentHeight <= containerRect.height && contentWidth <= containerRect.width) {
                        item.defaultStyles.height = contentHeight;
                        item.defaultStyles.width = contentWidth;
                        item.defaultStyles.top = (containerRect.height - contentHeight) / 2;
                        item.defaultStyles.left = (containerRect.width - contentWidth) / 2;
                    }
                    else {
                        if (proportion > 1) {
                            // 高比宽大
                            var width = contentWidth * containerRect.height / contentHeight;
                            item.defaultStyles.height = containerRect.height;
                            item.defaultStyles.width = width;
                            item.defaultStyles.top = 0;
                            item.defaultStyles.left = (containerRect.width - width) / 2;
                        }
                        else {
                            // 高比宽小
                            var height = contentHeight * containerRect.width / contentWidth;
                            item.defaultStyles.height = height;
                            item.defaultStyles.width = containerRect.width;
                            item.defaultStyles.top = (containerRect.height - height) / 2;
                            item.defaultStyles.left = 0;
                        }
                    }
                    Object.assign(item.styles, item.defaultStyles);
                }
                requestAnimationFrame(function () {
                    _this.opacity = 1;
                });
            });
        });
    };
    ImageViewerComponent.ctorParameters = function () { return [
        { type: ImageViewerController },
        { type: Renderer2 },
        { type: ElementRef }
    ]; };
    __decorate([
        ViewChildren('img'),
        __metadata("design:type", QueryList)
    ], ImageViewerComponent.prototype, "imgElements", void 0);
    __decorate([
        HostBinding('class.ui-show'),
        __metadata("design:type", Object)
    ], ImageViewerComponent.prototype, "isShow", void 0);
    __decorate([
        HostBinding('style.backgroundColor'),
        __metadata("design:type", Object)
    ], ImageViewerComponent.prototype, "bgColor", void 0);
    __decorate([
        HostBinding('style.transition'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], ImageViewerComponent.prototype, "transition", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], ImageViewerComponent.prototype, "click", null);
    ImageViewerComponent = __decorate([
        Component({
            selector: 'ui-image-viewer',
            template: "<ui-slide (uiPlayed)=\"setViewIndex($event)\" *ngIf=\"isShowSlide\" [autoPlay]=\"false\" [initIndex]=\"viewIndex\">\n  <ui-slide-item class=\"ui-image-viewer-item\" [ngStyle]=\"{opacity: opacity}\"\n                 (uiPinch)=\"pinch($event, item)\" (uiPan)=\"drag($event, item)\"\n                 *ngFor=\"let item of images\">\n    <img #img\n         class=\"ui-image-viewer-img\"\n         [src]=\"item.src\"\n         [ngStyle]=\"{\n            width: item.styles.width + 'px',\n            height: item.styles.height + 'px',\n            left: item.styles.left + 'px',\n            top: item.styles.top + 'px',\n            transition: item.styles.transition,\n            transform: item.styles.transform,\n            transformOrigin: item.styles.transformOrigin\n           }\">\n  </ui-slide-item>\n</ui-slide>"
        }),
        __metadata("design:paramtypes", [ImageViewerController,
            Renderer2,
            ElementRef])
    ], ImageViewerComponent);
    return ImageViewerComponent;
}());

var ButtonsComponent = /** @class */ (function () {
    function ButtonsComponent() {
    }
    ButtonsComponent = __decorate([
        Component({
            selector: 'ui-buttons',
            template: "<ng-content></ng-content>"
        })
    ], ButtonsComponent);
    return ButtonsComponent;
}());

var RouteCacheController = /** @class */ (function () {
    function RouteCacheController() {
        this.hasHistorySource = new Subject();
        this.hasCache = this.hasHistorySource.asObservable();
    }
    RouteCacheController.prototype.isCacheCurrentView = function (x) {
        this.hasHistorySource.next(x);
    };
    RouteCacheController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], RouteCacheController);
    return RouteCacheController;
}());

var UIRouteReuseStrategy = /** @class */ (function () {
    function UIRouteReuseStrategy(routeCacheController) {
        var _this = this;
        this.routeCacheController = routeCacheController;
        this.caches = {};
        this.isCache = false;
        this.routeSequence = [];
        this.routeCacheController.hasCache.subscribe(function (b) {
            _this.isCache = b;
        });
    }
    UIRouteReuseStrategy_1 = UIRouteReuseStrategy;
    UIRouteReuseStrategy.getRouteIdentifier = function (route) {
        var urls = [];
        if (route.routeConfig.loadChildren) {
            return UIRouteReuseStrategy_1.getRouteIdentifier(route.firstChild);
        }
        var r = route;
        while (r.firstChild) {
            r = r.firstChild;
        }
        while (r && r.routeConfig) {
            urls.unshift(r.routeConfig.path);
            r = r.parent;
        }
        return urls.join('/');
    };
    // 是否缓存路由
    UIRouteReuseStrategy.prototype.shouldDetach = function (route) {
        return this.isCache;
    };
    // 存储路由
    UIRouteReuseStrategy.prototype.store = function (route, handle) {
        var url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        if (handle) {
            this.routeSequence.push(url);
            if (!this.caches[url]) {
                this.caches[url] = [];
            }
            this.caches[url].push(handle);
            return;
        }
        var caches = this.caches[url];
        caches.pop();
        if (caches.length === 0) {
            delete this.caches[url];
        }
    };
    // 是否还原当前路由
    UIRouteReuseStrategy.prototype.shouldAttach = function (route) {
        var url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        var lastUrl = this.routeSequence[this.routeSequence.length - 1];
        if (this.caches[url] && url === lastUrl) {
            this.routeSequence.pop();
            return true;
        }
        return false;
    };
    // 取得之前缓存的路由
    UIRouteReuseStrategy.prototype.retrieve = function (route) {
        if (!route.routeConfig || this.isCache) {
            return null;
        }
        var url = UIRouteReuseStrategy_1.getRouteIdentifier(route);
        var caches = this.caches[url] || [];
        return caches[caches.length - 1];
    };
    // 是否重用路由
    UIRouteReuseStrategy.prototype.shouldReuseRoute = function (future, current) {
        return future.routeConfig === current.routeConfig;
    };
    var UIRouteReuseStrategy_1;
    UIRouteReuseStrategy.ctorParameters = function () { return [
        { type: RouteCacheController }
    ]; };
    UIRouteReuseStrategy = UIRouteReuseStrategy_1 = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [RouteCacheController])
    ], UIRouteReuseStrategy);
    return UIRouteReuseStrategy;
}());

var TabController = /** @class */ (function () {
    function TabController() {
        this.tabIndexSource = new Subject();
        this.tabIndex = this.tabIndexSource.asObservable();
    }
    TabController.prototype.publishIndex = function (index) {
        this.tabIndexSource.next(index);
    };
    TabController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], TabController);
    return TabController;
}());

var RouterService = /** @class */ (function () {
    function RouterService() {
        this.parentTouchEvent = new Subject();
        this.activatedSource = new Subject();
        this.animationProgressSource = new Subject();
        this.moveBackProgressSource = new Subject();
        this.activated = this.activatedSource.asObservable();
        this.animationProgress = this.animationProgressSource.asObservable();
        this.moveBackProgress = this.moveBackProgressSource.asObservable();
        this.parentTouchProgress = this.parentTouchEvent.asObservable();
    }
    RouterService.prototype.publish = function (activateView) {
        this.activatedSource.next(activateView);
    };
    RouterService.prototype.publishAnimationProgress = function (progress) {
        this.animationProgressSource.next(progress);
    };
    RouterService.prototype.publishMoveBackProgress = function (progress) {
        this.moveBackProgressSource.next(progress);
    };
    RouterService.prototype.parentTouching = function (progress) {
        this.parentTouchEvent.next(progress);
    };
    RouterService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], RouterService);
    return RouterService;
}());

var ViewState;
(function (ViewState) {
    ViewState["Activate"] = "Activate";
    ViewState["ToStack"] = "ToStack";
    ViewState["Destroy"] = "Destroy";
    ViewState["Reactivate"] = "Reactivate";
    ViewState["Sleep"] = "Sleep";
})(ViewState || (ViewState = {}));
var UI_VIEW_INIT_STATE = new InjectionToken('UI_VIEW_INIT_STATE');
var UI_VIEW_INIT_TOUCH_BACK = new InjectionToken('UI_VIEW_INIT_TOUCH_BACK');
var ViewStateService = /** @class */ (function () {
    function ViewStateService() {
        this.stateSource = new Subject();
        this.progressSource = new Subject();
        this.touchEvent = new Subject();
        this.state = this.stateSource.asObservable();
        this.progress = this.progressSource.asObservable();
        this.touchProgress = this.touchEvent.asObservable();
    }
    ViewStateService.prototype.changeState = function (state) {
        this.stateSource.next(state);
    };
    ViewStateService.prototype.updateProgress = function (n) {
        this.progressSource.next(n);
    };
    ViewStateService.prototype.touching = function (progress) {
        this.touchEvent.next(progress);
    };
    ViewStateService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ViewStateService);
    return ViewStateService;
}());

/**
 * @whatItDoes 定义路由转声动画的帧数
 *
 * 你可以通过 provide 自定义帧数，帧数越多，转场动画的时间越长
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_ROUTER_ANIMATION_STEPS } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_ROUTER_ANIMATION_STEPS,
 *     useValue: 30
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
var UI_ROUTER_ANIMATION_STEPS = new InjectionToken('UI_ROUTER_ANIMATION_STEPS');
/**
 * @whatItDoes 定义 ui-back 组件左箭头的 html class 的值
 *
 * 你可以通过 provide 定义自己的 class
 *
 * @howToUse
 * ```typescript
 * import { NgModule } from '@angular/core';
 * import { UI_BACK_ICON_CLASSNAME } from '@tanbo/ui-native';
 *
 * @NgModule({
 *   ...
 *   providers: [{
 *     provide: UI_BACK_ICON_CLASSNAME,
 *     useValue: 'custom-classname'
 *   }]
 * })
 * export class AppModule {
 * }
 * ```
 */
var UI_BACK_ICON_CLASSNAME = new InjectionToken('UI_BACK_ICON_CLASSNAME');

var BackComponent = /** @class */ (function () {
    function BackComponent(document, steps, scale, iconName, state, location, router, appController, viewStateService) {
        this.document = document;
        this.steps = steps;
        this.scale = scale;
        this.state = state;
        this.location = location;
        this.router = router;
        this.appController = appController;
        this.viewStateService = viewStateService;
        this.icon = '';
        this.closeBackHandle = false;
        this.opacity = 1;
        this.subs = [];
        this.timer = null;
        this.icon = iconName || '';
    }
    BackComponent.prototype.resize = function () {
        this.docWidth = this.document.body.offsetWidth;
        this.translateDistance = this.docWidth / 2 - (this.leftDistance + 10 * this.scale) - this.contentWidth / 2;
    };
    BackComponent.prototype.click = function () {
        var _this = this;
        // 当用户点击组件时，触发视图返回上一页
        clearTimeout(this.timer);
        if (!this.closeBackHandle) {
            this.timer = setTimeout(function () {
                _this.appController.quit();
            }, 300);
            this.location.back();
        }
    };
    BackComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.router.events.subscribe(function (event) {
            if (event instanceof NavigationStart) {
                clearTimeout(_this.timer);
            }
        }));
        var steps = this.steps;
        var bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            var n = p / 50;
            var m;
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.translate = "translate3d(" + p / steps * _this.translateDistance + "px, 0, 0)";
                m = 1 - n;
                _this.opacity = m < 0 ? 0 : m;
            }
            else if (_this.state === ViewState.ToStack) {
                _this.translate = "translate3d(" + (-50 + 50 * p / steps) + "%, 0, 0)";
                m = progress * 2;
                _this.opacity = m > 1 ? 1 : m;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            var n = p / 50;
            var m;
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = 1;
                    var distance = _this.translateDistance - progress * _this.translateDistance;
                    _this.translate = "translate3d(" + distance + "px, 0, 0)";
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + _this.translateDistance * progress + "px, 0, 0)";
                    break;
                case ViewState.ToStack:
                    _this.translate = "translate3d(" + -progress * 100 + "%, 0, 0)";
                    m = 1 - n;
                    _this.opacity = m < 0 ? 0 : m;
                    break;
                case ViewState.Reactivate:
                    _this.translate = "translate3d(" + (-50 + progress * 50) + "%, 0, 0)";
                    m = progress * 2;
                    _this.opacity = m > 1 ? 1 : m;
                    break;
            }
        }));
    };
    BackComponent.prototype.ngAfterViewInit = function () {
        this.docWidth = this.document.body.offsetWidth;
        this.contentWidth = this.textElement.nativeElement.offsetWidth;
        this.leftDistance = this.textElement.nativeElement.offsetLeft;
        this.translateDistance = this.docWidth / 2 - (this.leftDistance + 10 * this.scale) - this.contentWidth / 2;
    };
    BackComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    BackComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: String, decorators: [{ type: Inject, args: [UI_BACK_ICON_CLASSNAME,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Location },
        { type: Router },
        { type: AppController },
        { type: ViewStateService }
    ]; };
    __decorate([
        ViewChild('text', { static: true }),
        __metadata("design:type", ElementRef)
    ], BackComponent.prototype, "textElement", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], BackComponent.prototype, "icon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], BackComponent.prototype, "closeBackHandle", void 0);
    __decorate([
        HostListener('window:resize'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], BackComponent.prototype, "resize", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], BackComponent.prototype, "click", null);
    BackComponent = __decorate([
        Component({
            selector: 'ui-back',
            template: "<div class=\"ui-back\">\n  <div class=\"ui-back-icon\">\n    <div [ngClass]=\"icon\"></div>\n  </div>\n  <div #text class=\"ui-back-text\" [ngStyle]=\"{transform: translate, webkitTransform: translate, opacity: opacity}\">\n    <ng-content></ng-content>\n  </div>\n</div>"
        }),
        __param(0, Inject(DOCUMENT)),
        __param(1, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __param(2, Inject(UI_SCREEN_SCALE)),
        __param(3, Inject(UI_BACK_ICON_CLASSNAME)),
        __param(4, Inject(UI_VIEW_INIT_STATE)),
        __metadata("design:paramtypes", [Object, Number, Number, String, String, Location,
            Router,
            AppController,
            ViewStateService])
    ], BackComponent);
    return BackComponent;
}());

var TabViewItemService = /** @class */ (function () {
    function TabViewItemService() {
        this.changeEvent = new Subject();
        this.stateSource = new Subject();
        this.progressSource = new Subject();
        this.touchEvent = new Subject();
        this.activateState = this.changeEvent.asObservable();
        this.state = this.stateSource.asObservable();
        this.progress = this.progressSource.asObservable();
        this.touchProgress = this.touchEvent.asObservable();
    }
    TabViewItemService.prototype.change = function (state) {
        this.changeEvent.next(state);
    };
    TabViewItemService.prototype.changeState = function (state) {
        this.stateSource.next(state);
    };
    TabViewItemService.prototype.updateProgress = function (n) {
        this.progressSource.next(n);
    };
    TabViewItemService.prototype.touching = function (progress) {
        this.touchEvent.next(progress);
    };
    TabViewItemService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], TabViewItemService);
    return TabViewItemService;
}());

var RouterComponent = /** @class */ (function () {
    function RouterComponent(parentContexts, routerService, resolver, routeCacheController, viewContainerRef, location, appController, tabViewItemService, state, initTouchBack, steps) {
        this.parentContexts = parentContexts;
        this.routerService = routerService;
        this.resolver = resolver;
        this.routeCacheController = routeCacheController;
        this.viewContainerRef = viewContainerRef;
        this.location = location;
        this.appController = appController;
        this.tabViewItemService = tabViewItemService;
        this.state = state;
        this.initTouchBack = initTouchBack;
        this.steps = steps;
        /* tslint:disable */
        this.activateEvents = new EventEmitter();
        this.deactivateEvents = new EventEmitter();
        /* tslint:enable */
        this.name = PRIMARY_OUTLET;
        this.views = [];
        this.subs = [];
        this.activated = null;
        this._activatedRoute = null;
        this.isMoveBack = false;
        this.isBack = false;
    }
    Object.defineProperty(RouterComponent.prototype, "openMoveBack", {
        get: function () {
            return this.views.length > 1 || this.initTouchBack;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RouterComponent.prototype, "isActivated", {
        get: function () {
            return !!this.activated;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RouterComponent.prototype, "component", {
        get: function () {
            if (!this.activated) {
                throw new Error('Outlet is not activated');
            }
            return this.activated.instance;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RouterComponent.prototype, "activatedRoute", {
        get: function () {
            if (!this.activated) {
                throw new Error('Outlet is not activated');
            }
            return this._activatedRoute;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RouterComponent.prototype, "activatedRouteData", {
        get: function () {
            if (this._activatedRoute) {
                return this._activatedRoute.snapshot.data;
            }
            return {};
        },
        enumerable: true,
        configurable: true
    });
    RouterComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.routeCacheController.hasCache.pipe(distinctUntilChanged()).subscribe(function (b) {
            _this.isBack = !b;
        }));
        if (this.tabViewItemService) {
            this.subs.push(this.tabViewItemService.activateState.subscribe(function (b) {
                if (b) {
                    _this.setViewState([ViewState.Activate]);
                }
                else {
                    _this.setViewState([ViewState.ToStack]);
                }
            }));
            this.subs.push(this.tabViewItemService.state.subscribe(function (state) {
                _this.setViewState([state]);
            }));
            this.subs.push(this.tabViewItemService.progress.subscribe(function (p) {
                _this.routerService.publishAnimationProgress(p);
            }));
            this.subs.push(this.tabViewItemService.touchProgress.subscribe(function (p) {
                _this.routerService.parentTouching(p);
            }));
        }
        this.parentContexts.onChildOutletCreated(this.name, this);
        if (!this.activated) {
            var context = this.parentContexts.getContext(this.name);
            if (context && context.route) {
                this.activateWith(context.route, context.resolver || null);
            }
        }
        // 订阅当前活动的组件
        this.subs.push(this.routerService.activated.subscribe(function (componentRef) {
            _this.activated = componentRef;
        }));
        var parentRouterService = this.viewContainerRef.parentInjector.get(RouterService, null);
        this.subs.push(this.routerService.moveBackProgress.subscribe(function (progress) {
            if (progress <= 0) {
                _this.setViewState([ViewState.Sleep, ViewState.Activate]);
            }
            else {
                _this.setViewState([ViewState.ToStack, ViewState.Activate]);
            }
            if (parentRouterService && _this.views.length <= 1) {
                parentRouterService.publishMoveBackProgress(progress);
            }
            else if (progress >= _this.steps && _this.openMoveBack) {
                _this.isMoveBack = true;
                _this.location.back();
            }
        }));
    };
    RouterComponent.prototype.ngOnDestroy = function () {
        this.parentContexts.onChildOutletDestroyed(this.name);
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    // 从子树分离时调用
    RouterComponent.prototype.detach = function () {
        if (!this.activated) {
            throw new Error('Outlet is not activated');
        }
        var comp = this.activated;
        this._activatedRoute = null;
        this.activated = null;
        return comp;
    };
    // 将以前分离的子树重新附加时调用
    RouterComponent.prototype.attach = function (ref, activatedRoute) {
        var _this = this;
        this.activated = ref;
        this._activatedRoute = activatedRoute;
        this.routeCacheController.isCacheCurrentView(true);
        if (this.isMoveBack) {
            this.views.pop();
            this.isMoveBack = false;
            this.setViewState([ViewState.Activate]);
            setTimeout(function () {
                _this.routerService.publishAnimationProgress(_this.steps);
            });
            return;
        }
        this.setViewState([ViewState.Reactivate, ViewState.Destroy]);
        setTimeout(function () {
            _this.setupRouterAnimation();
        });
    };
    RouterComponent.prototype.activateWith = function (activatedRoute, resolver) {
        this._activatedRoute = activatedRoute;
        var snapshot = activatedRoute._futureSnapshot;
        var component = snapshot.routeConfig.component;
        resolver = resolver || this.resolver;
        var childContexts = this.parentContexts.getOrCreateContext(this.name).children;
        if (this.isBack) {
            this.setViewState([ViewState.Destroy]);
            this.views.unshift({
                state: ViewState.Reactivate,
                componentFactory: resolver.resolveComponentFactory(component),
                childContexts: childContexts,
                activatedRoute: activatedRoute
            });
        }
        else {
            this.setViewState([ViewState.ToStack]);
            var state = void 0;
            if (this.state) {
                state = this.state;
                this.state = null;
            }
            else {
                state = ViewState.Activate;
            }
            this.views.push({
                state: state,
                componentFactory: resolver.resolveComponentFactory(component),
                childContexts: childContexts,
                activatedRoute: activatedRoute
            });
        }
        this.routeCacheController.isCacheCurrentView(true);
        this.setupRouterAnimation();
    };
    RouterComponent.prototype.deactivate = function () {
        this.activated = null;
        this._activatedRoute = null;
    };
    RouterComponent.prototype.setViewState = function (status) {
        var i = this.views.length - 1;
        while (i >= 0) {
            this.views[i].state = status.pop() || ViewState.Sleep;
            i--;
        }
    };
    RouterComponent.prototype.setupRouterAnimation = function () {
        var _this = this;
        if (this.views.length === 1) {
            this.routerService.publishAnimationProgress(this.steps);
            return;
        }
        var i = 0;
        this.appController.transitionStart();
        var fn = function () {
            if (i > _this.steps) {
                _this.appController.transitionEnd();
                switch (_this.views[_this.views.length - 1].state) {
                    case ViewState.Destroy:
                        _this.views.pop();
                        _this.setViewState([ViewState.Activate]);
                        break;
                    case ViewState.Activate:
                        _this.setViewState([ViewState.Sleep, ViewState.Activate]);
                        break;
                }
            }
            else {
                requestAnimationFrame(fn);
                _this.routerService.publishAnimationProgress(i);
                i++;
            }
        };
        fn();
    };
    RouterComponent.ctorParameters = function () { return [
        { type: ChildrenOutletContexts },
        { type: RouterService },
        { type: ComponentFactoryResolver },
        { type: RouteCacheController },
        { type: ViewContainerRef },
        { type: Location },
        { type: AppController },
        { type: TabViewItemService, decorators: [{ type: Optional }] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Boolean, decorators: [{ type: Optional }, { type: Inject, args: [UI_VIEW_INIT_TOUCH_BACK,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
    ]; };
    __decorate([
        Output('activate'),
        __metadata("design:type", Object)
    ], RouterComponent.prototype, "activateEvents", void 0);
    __decorate([
        Output('deactivate'),
        __metadata("design:type", Object)
    ], RouterComponent.prototype, "deactivateEvents", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RouterComponent.prototype, "name", void 0);
    RouterComponent = __decorate([
        Component({
            selector: 'ui-router',
            template: "<ui-view\n        *ngFor=\"let item of views; let i = index\"\n        [openMoveBack]=\"openMoveBack\"\n        [componentFactory]=\"item.componentFactory\"\n        [activatedRoute]=\"item.activatedRoute\"\n        [parentContexts]=\"item.childContexts\"\n        [state]=\"item.state\"\n        [ngStyle]=\"{zIndex: i}\">\n</ui-view>\n",
            providers: [
                RouterService
            ]
        }),
        __param(7, Optional()),
        __param(8, Optional()), __param(8, Inject(UI_VIEW_INIT_STATE)),
        __param(9, Optional()), __param(9, Inject(UI_VIEW_INIT_TOUCH_BACK)),
        __param(10, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __metadata("design:paramtypes", [ChildrenOutletContexts,
            RouterService,
            ComponentFactoryResolver,
            RouteCacheController,
            ViewContainerRef,
            Location,
            AppController,
            TabViewItemService, String, Boolean, Number])
    ], RouterComponent);
    return RouterComponent;
}());

var TabComponent = /** @class */ (function () {
    function TabComponent() {
    }
    TabComponent = __decorate([
        Component({
            selector: 'ui-tab',
            template: "<ng-content></ng-content>",
            providers: [
                TabController
            ]
        })
    ], TabComponent);
    return TabComponent;
}());

var TabBarItemComponent = /** @class */ (function () {
    function TabBarItemComponent() {
        this.uiSelected = new EventEmitter();
        this.active = false;
    }
    TabBarItemComponent.prototype.click = function () {
        this.uiSelected.emit();
    };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TabBarItemComponent.prototype, "uiSelected", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-active'),
        __metadata("design:type", Boolean)
    ], TabBarItemComponent.prototype, "active", void 0);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], TabBarItemComponent.prototype, "click", null);
    TabBarItemComponent = __decorate([
        Component({
            selector: 'ui-tab-bar-item',
            template: "<ng-content></ng-content>"
        })
    ], TabBarItemComponent);
    return TabBarItemComponent;
}());

var TabBarComponent = /** @class */ (function () {
    function TabBarComponent(tabService, viewStateService, state, steps) {
        this.tabService = tabService;
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.tabIndex = 0;
        this.subs = [];
    }
    TabBarComponent.prototype.ngOnInit = function () {
        var _this = this;
        var steps = this.steps;
        var bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.translate = "translate3d(" + p / steps * 100 + "%, 0, 0)";
            }
            else if (_this.state === ViewState.ToStack) {
                _this.translate = "translate3d(" + (-33 + 33 * p / steps) + "%, 0, 0)";
                _this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + (100 - progress * 100) + "%, 0, 0)";
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + progress * 100 + "%, 0, 0)";
                    break;
                case ViewState.ToStack:
                    _this.translate = "translate3d(" + progress * -33 + "%, 0, 0)";
                    _this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    var n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    _this.translate = n === 0 ? '' : "translate3d(" + n + "%, 0, 0)";
                    _this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    };
    TabBarComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        // 当用户点击或选中某一个按扭时，发布相应事件
        this.tabBarItems.forEach(function (item, index) {
            var sub = item.uiSelected.subscribe(function () {
                _this.tabService.publishIndex(index);
            });
            _this.subs.push(sub);
        });
        // 当用户切换tab时，显示/隐藏对应视图
        this.subs.push(this.tabService.tabIndex.subscribe(function (index) {
            _this.tabBarItems.forEach(function (item, i) {
                item.active = i === index;
            });
        }));
        this.tabService.publishIndex(this.tabIndex);
    };
    TabBarComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    TabBarComponent.ctorParameters = function () { return [
        { type: TabController },
        { type: ViewStateService },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
    ]; };
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", String)
    ], TabBarComponent.prototype, "translate", void 0);
    __decorate([
        HostBinding('style.opacity'),
        __metadata("design:type", Number)
    ], TabBarComponent.prototype, "opacity", void 0);
    __decorate([
        ContentChildren(TabBarItemComponent),
        __metadata("design:type", QueryList)
    ], TabBarComponent.prototype, "tabBarItems", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], TabBarComponent.prototype, "tabIndex", void 0);
    TabBarComponent = __decorate([
        Component({
            selector: 'ui-tab-bar',
            template: "<div class=\"ui-tab-bar-shadow\"></div>\n<div class=\"ui-tab-bar\">\n  <ng-content></ng-content>\n</div>"
        }),
        __param(2, Inject(UI_VIEW_INIT_STATE)),
        __param(3, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __metadata("design:paramtypes", [TabController,
            ViewStateService, String, Number])
    ], TabBarComponent);
    return TabBarComponent;
}());

var TabViewItemComponent = /** @class */ (function () {
    function TabViewItemComponent(changeDetectorRef, viewStateService, tabViewItemService) {
        this.changeDetectorRef = changeDetectorRef;
        this.viewStateService = viewStateService;
        this.tabViewItemService = tabViewItemService;
        this._active = false;
        this.isInit = false;
        this.subs = [];
    }
    Object.defineProperty(TabViewItemComponent.prototype, "active", {
        get: function () {
            return this._active;
        },
        set: function (value) {
            this._active = value;
            this.tabViewItemService.change(value);
            if (value) {
                this.isInit = true;
                this.changeDetectorRef.reattach();
            }
            else {
                this.changeDetectorRef.detach();
                if (this.isInit) {
                    this.changeDetectorRef.detectChanges();
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    TabViewItemComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            if (_this.active) {
                _this.tabViewItemService.changeState(state);
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            if (_this.active) {
                _this.tabViewItemService.updateProgress(p);
            }
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            if (_this.active) {
                _this.tabViewItemService.touching(p);
            }
        }));
    };
    TabViewItemComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    TabViewItemComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef },
        { type: ViewStateService },
        { type: TabViewItemService }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TabViewItemComponent.prototype, "name", void 0);
    __decorate([
        HostBinding('class.ui-active'),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], TabViewItemComponent.prototype, "active", null);
    TabViewItemComponent = __decorate([
        Component({
            selector: 'ui-tab-view-item',
            template: "<ng-content></ng-content>\n<ui-router [name]=\"name\"></ui-router>",
            providers: [
                TabViewItemService
            ]
        }),
        __metadata("design:paramtypes", [ChangeDetectorRef,
            ViewStateService,
            TabViewItemService])
    ], TabViewItemComponent);
    return TabViewItemComponent;
}());

var TabViewComponent = /** @class */ (function () {
    function TabViewComponent(tabService) {
        this.tabService = tabService;
        this.subs = [];
    }
    TabViewComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        // 订阅tab切换事件，如果发生切换，显示/隐藏对应视图
        this.subs.push(this.tabService.tabIndex.subscribe(function (index) {
            _this.tabViewItems.forEach(function (item, i) {
                item.active = i === index;
            });
        }));
        // 初始化时，默认为第一项
        var items = this.tabViewItems.toArray();
        if (items[0]) {
            items[0].active = true;
        }
    };
    TabViewComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    TabViewComponent.ctorParameters = function () { return [
        { type: TabController }
    ]; };
    __decorate([
        ContentChildren(TabViewItemComponent),
        __metadata("design:type", QueryList)
    ], TabViewComponent.prototype, "tabViewItems", void 0);
    TabViewComponent = __decorate([
        Component({
            selector: 'ui-tab-view',
            template: "<ng-content></ng-content>",
            providers: []
        }),
        __metadata("design:paramtypes", [TabController])
    ], TabViewComponent);
    return TabViewComponent;
}());

var ComponentHostDirective = /** @class */ (function () {
    function ComponentHostDirective(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
    ComponentHostDirective.ctorParameters = function () { return [
        { type: ViewContainerRef }
    ]; };
    ComponentHostDirective = __decorate([
        Directive({
            selector: '[uiComponentHost]'
        }),
        __metadata("design:paramtypes", [ViewContainerRef])
    ], ComponentHostDirective);
    return ComponentHostDirective;
}());

var ActionSheetService = /** @class */ (function () {
    function ActionSheetService() {
        this.showEvent = new Subject();
        this.onShow = this.showEvent.asObservable();
    }
    ActionSheetService.prototype.push = function (el) {
        this.showEvent.next(el);
    };
    ActionSheetService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ActionSheetService);
    return ActionSheetService;
}());

var ContentLoadingController = /** @class */ (function () {
    function ContentLoadingController() {
        this.showLoadingSource = new Subject();
        this.hideLoadingSource = new Subject();
        this.showLoading = this.showLoadingSource.asObservable();
        this.hideLoading = this.hideLoadingSource.asObservable();
    }
    ContentLoadingController.prototype.show = function (text) {
        this.showLoadingSource.next(text);
    };
    ContentLoadingController.prototype.hide = function () {
        this.hideLoadingSource.next();
    };
    ContentLoadingController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ContentLoadingController);
    return ContentLoadingController;
}());

var ActionSheetComponent = /** @class */ (function () {
    function ActionSheetComponent(actionSheetService, elementRef) {
        this.actionSheetService = actionSheetService;
        this.elementRef = elementRef;
        this.isPopUp = false;
        this._show = false;
        this.timer = null;
    }
    Object.defineProperty(ActionSheetComponent.prototype, "show", {
        get: function () {
            return this._show;
        },
        set: function (value) {
            var _this = this;
            clearTimeout(this.timer);
            if (value) {
                this._show = value;
                this.timer = setTimeout(function () {
                    _this.isPopUp = value;
                }, 20);
            }
            else {
                this.isPopUp = value;
                this.timer = setTimeout(function () {
                    _this._show = value;
                }, 300);
            }
        },
        enumerable: true,
        configurable: true
    });
    ActionSheetComponent.prototype.ngOnInit = function () {
        this.actionSheetService.push(this.elementRef.nativeElement);
    };
    ActionSheetComponent.ctorParameters = function () { return [
        { type: ActionSheetService },
        { type: ElementRef }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], ActionSheetComponent.prototype, "show", null);
    ActionSheetComponent = __decorate([
        Component({
            selector: 'ui-action-sheet',
            template: "<ui-mask [show]=\"isPopUp\">\n  <div class=\"ui-action-sheet\" [ngClass]=\"{'ui-show': show, 'ui-pop-up': isPopUp}\">\n    <ng-content></ng-content>\n  </div>\n</ui-mask>\n"
        }),
        __metadata("design:paramtypes", [ActionSheetService,
            ElementRef])
    ], ActionSheetComponent);
    return ActionSheetComponent;
}());

var ContentLoadingComponent = /** @class */ (function () {
    function ContentLoadingComponent(contentLoadingController) {
        this.contentLoadingController = contentLoadingController;
        this.show = false;
        this.text = '';
        this.fadeIn = false;
        this.fadeOut = false;
        this.subs = [];
        this.timer = null;
    }
    ContentLoadingComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.subs.push(this.contentLoadingController.showLoading.subscribe(function (text) {
            _this.fadeOut = false;
            _this.fadeIn = true;
            clearTimeout(_this.timer);
            _this.timer = setTimeout(function () {
                _this.show = true;
                _this.text = text;
            });
        }));
        this.subs.push(this.contentLoadingController.hideLoading.subscribe(function () {
            _this.fadeIn = false;
            _this.fadeOut = true;
            clearTimeout(_this.timer);
            _this.timer = setTimeout(function () {
                _this.show = false;
                _this.text = '';
            }, 150);
        }));
    };
    ContentLoadingComponent.prototype.ngOnDestroy = function () {
        clearTimeout(this.timer);
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    ContentLoadingComponent.ctorParameters = function () { return [
        { type: ContentLoadingController }
    ]; };
    __decorate([
        HostBinding('class.ui-content-loading-fadein'),
        __metadata("design:type", Boolean)
    ], ContentLoadingComponent.prototype, "fadeIn", void 0);
    __decorate([
        HostBinding('class.ui-content-loading-fadeout'),
        __metadata("design:type", Boolean)
    ], ContentLoadingComponent.prototype, "fadeOut", void 0);
    ContentLoadingComponent = __decorate([
        Component({
            selector: 'ui-content-loading',
            template: "<div class=\"ui-content-loading\" *ngIf=\"show\">\n  <ui-loading color=\"#fff\"></ui-loading>\n  <p class=\"ui-content-loading-text\">{{ text || '\u52A0\u8F7D\u4E2D\u2026' }}</p>\n</div>",
            animations: [
                trigger('loadingAnimate', [transition(':enter', animate('5s', keyframes([
                        style({
                            opacity: 0,
                            offset: 0
                        }),
                        style({
                            opacity: 1,
                            offset: 1
                        })
                    ]))), transition(':leave', animate('0.15s', keyframes([
                        style({
                            opacity: 1,
                            offset: 0
                        }),
                        style({
                            opacity: 0,
                            offset: 1
                        })
                    ])))])
            ]
        }),
        __metadata("design:paramtypes", [ContentLoadingController])
    ], ContentLoadingComponent);
    return ContentLoadingComponent;
}());

var FixedBarComponent = /** @class */ (function () {
    function FixedBarComponent() {
    }
    FixedBarComponent = __decorate([
        Component({
            selector: 'ui-fixed-bar',
            template: "<ng-content></ng-content>"
        })
    ], FixedBarComponent);
    return FixedBarComponent;
}());

var LoadingComponent = /** @class */ (function () {
    function LoadingComponent() {
        this.scale = 1;
        this.color = '';
    }
    Object.defineProperty(LoadingComponent.prototype, "size", {
        get: function () {
            return "scale(" + this.scale + ")";
        },
        enumerable: true,
        configurable: true
    });
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], LoadingComponent.prototype, "scale", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], LoadingComponent.prototype, "color", void 0);
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], LoadingComponent.prototype, "size", null);
    LoadingComponent = __decorate([
        Component({
            selector: 'ui-loading',
            template: "<div class=\"ui-circle-1\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-2\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-3\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-4\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-5\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-6\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-7\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-8\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-9\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-10\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-11\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>\n<div class=\"ui-circle-12\"><span [ngStyle]=\"{backgroundColor: color}\"></span></div>"
        })
    ], LoadingComponent);
    return LoadingComponent;
}());

var StopPropagationDirective = /** @class */ (function () {
    function StopPropagationDirective(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.callbacks = [];
    }
    StopPropagationDirective.prototype.ngOnInit = function () {
        var _this = this;
        if (typeof this.uiStopPropagation === 'string') {
            this.addEvent(this.uiStopPropagation || 'click');
        }
        else {
            this.uiStopPropagation.forEach(function (eventType) {
                _this.addEvent(eventType);
            });
        }
    };
    StopPropagationDirective.prototype.ngOnDestroy = function () {
        this.callbacks.forEach(function (fn) {
            fn();
        });
    };
    StopPropagationDirective.prototype.addEvent = function (eventType) {
        var fn = this.renderer.listen(this.elementRef.nativeElement, eventType, function (ev) {
            ev.stopPropagation();
        });
        this.callbacks.push(fn);
    };
    StopPropagationDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], StopPropagationDirective.prototype, "uiStopPropagation", void 0);
    StopPropagationDirective = __decorate([
        Directive({
            selector: '[uiStopPropagation]'
        }),
        __metadata("design:paramtypes", [ElementRef, Renderer2])
    ], StopPropagationDirective);
    return StopPropagationDirective;
}());

var ViewInjector = /** @class */ (function () {
    function ViewInjector(route, childContexts, initState, isOpenTouchBack, parent) {
        this.route = route;
        this.childContexts = childContexts;
        this.initState = initState;
        this.isOpenTouchBack = isOpenTouchBack;
        this.parent = parent;
    }
    ViewInjector.prototype.get = function (token, notFoundValue) {
        if (token === ActivatedRoute) {
            return this.route;
        }
        if (token === ChildrenOutletContexts) {
            return this.childContexts;
        }
        if (token === UI_VIEW_INIT_STATE) {
            return this.initState;
        }
        if (token === UI_VIEW_INIT_TOUCH_BACK) {
            return this.isOpenTouchBack;
        }
        return this.parent.get(token, notFoundValue);
    };
    return ViewInjector;
}());
var ViewComponent = /** @class */ (function () {
    function ViewComponent(viewStateService, viewContainerRef, _activatedRoute, changeDetectorRef, actionSheetService, elementRef, routerService) {
        this.viewStateService = viewStateService;
        this.viewContainerRef = viewContainerRef;
        this._activatedRoute = _activatedRoute;
        this.changeDetectorRef = changeDetectorRef;
        this.actionSheetService = actionSheetService;
        this.elementRef = elementRef;
        this.routerService = routerService;
        this.openMoveBack = false;
        this.subs = [];
    }
    Object.defineProperty(ViewComponent.prototype, "state", {
        get: function () {
            return this._state;
        },
        set: function (value) {
            if (value === this._state) {
                return;
            }
            // 根据视图不同状态，调用生命周期勾子
            this.viewStateService.changeState(value);
            if (this.childInstance) {
                switch (value) {
                    case ViewState.Activate:
                    case ViewState.Reactivate:
                        if (this._state !== ViewState.Activate && this._state !== ViewState.Reactivate) {
                            if (typeof this.childInstance['uiOnViewEnter'] === 'function') {
                                this.childInstance['uiOnViewEnter']();
                            }
                            this.routerService.publish(this.componentRef);
                        }
                        break;
                    case ViewState.ToStack:
                    case ViewState.Destroy:
                        if (typeof this.childInstance['uiOnViewLeave'] === 'function') {
                            this.childInstance['uiOnViewLeave']();
                        }
                        break;
                }
                if (value === ViewState.Sleep) {
                    this.changeDetectorRef.detach();
                }
                else {
                    this.changeDetectorRef.reattach();
                }
            }
            this._state = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ViewComponent.prototype, "sleep", {
        get: function () {
            return this.state === ViewState.Sleep;
        },
        enumerable: true,
        configurable: true
    });
    ViewComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (this.componentFactory) {
            var injector = new ViewInjector(this.activatedRoute || this._activatedRoute, this.parentContexts, this.state, this.openMoveBack, this.viewContainerRef.injector);
            this.componentRef = this.componentHost.viewContainerRef.createComponent(this.componentFactory, this.componentHost.viewContainerRef.length, injector);
            this.childInstance = this.componentRef.instance;
            this.routerService.publish(this.componentRef);
            if (typeof this.childInstance['uiOnViewEnter'] === 'function') {
                this.childInstance['uiOnViewEnter']();
            }
        }
        this.subs.push(this.routerService.animationProgress.subscribe(function (progress) {
            if (_this.state !== ViewState.Sleep) {
                _this.viewStateService.updateProgress(progress);
            }
        }));
        this.subs.push(this.routerService.moveBackProgress.subscribe(function (progress) {
            if (_this.state !== ViewState.Sleep && _this.openMoveBack) {
                _this.viewStateService.touching(progress);
            }
        }));
        this.subs.push(this.routerService.parentTouchProgress.subscribe(function (progress) {
            if (_this.state !== ViewState.Sleep) {
                _this.viewStateService.touching(progress);
            }
        }));
        this.subs.push(this.actionSheetService.onShow.subscribe(function (el) {
            _this.elementRef.nativeElement.appendChild(el);
        }));
    };
    ViewComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    ViewComponent.ctorParameters = function () { return [
        { type: ViewStateService },
        { type: ViewContainerRef },
        { type: ActivatedRoute },
        { type: ChangeDetectorRef },
        { type: ActionSheetService },
        { type: ElementRef },
        { type: RouterService }
    ]; };
    __decorate([
        ViewChild(ComponentHostDirective, { static: true }),
        __metadata("design:type", ComponentHostDirective)
    ], ViewComponent.prototype, "componentHost", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ViewComponent.prototype, "componentFactory", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ViewComponent.prototype, "openMoveBack", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ViewComponent.prototype, "parentContexts", void 0);
    __decorate([
        Input(),
        __metadata("design:type", ActivatedRoute)
    ], ViewComponent.prototype, "activatedRoute", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], ViewComponent.prototype, "state", null);
    __decorate([
        HostBinding('class.ui-sleep'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], ViewComponent.prototype, "sleep", null);
    ViewComponent = __decorate([
        Component({
            selector: 'ui-view',
            template: "<ng-template uiComponentHost></ng-template>\n<ui-content-loading></ui-content-loading>",
            providers: [
                ViewStateService,
                ContentLoadingController,
                PullDownRefreshController,
                PullUpLoadController,
                ActionSheetService
            ]
        }),
        __metadata("design:paramtypes", [ViewStateService,
            ViewContainerRef,
            ActivatedRoute,
            ChangeDetectorRef,
            ActionSheetService,
            ElementRef,
            RouterService])
    ], ViewComponent);
    return ViewComponent;
}());

var ContentComponent = /** @class */ (function () {
    function ContentComponent(viewStateService, elementRef, routerService, state, scale, renderer, steps) {
        this.viewStateService = viewStateService;
        this.elementRef = elementRef;
        this.routerService = routerService;
        this.state = state;
        this.scale = scale;
        this.renderer = renderer;
        this.steps = steps;
        this.subs = [];
        this.distanceX = 0;
    }
    ContentComponent.prototype.ngOnInit = function () {
        var _this = this;
        var steps = this.steps;
        var bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.translate = "translate3d(" + p / steps * 100 + "%, 0, 0)";
            }
            else if (_this.state === ViewState.ToStack) {
                _this.translate = "translate3d(" + (-33 + 33 * p / steps) + "%, 0, 0)";
                _this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + (100 - progress * 100) + "%, 0, 0)";
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + progress * 100 + "%, 0, 0)";
                    break;
                case ViewState.ToStack:
                    _this.translate = "translate3d(" + progress * -33 + "%, 0, 0)";
                    _this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    var n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    _this.translate = n === 0 ? '' : "translate3d(" + n + "%, 0, 0)";
                    _this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    };
    ContentComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    ContentComponent.prototype.touchStart = function (event) {
        var _this = this;
        if (navigator.userAgent.indexOf('web-2.0') > -1) {
            return;
        }
        var startPoint = event.touches[0];
        var startX = startPoint.pageX;
        var startY = startPoint.pageY;
        if (startX > 50 * this.scale && this.state !== ViewState.Sleep) {
            return;
        }
        cancelAnimationFrame(this.animationId);
        var unbindTouchMoveFn;
        var unbindTouchEndFn;
        var unbindTouchCancelFn;
        var unbindFn;
        var element = this.elementRef.nativeElement;
        var maxWidth = element.offsetWidth;
        var startTime = Date.now();
        this.maxWidth = maxWidth;
        var isBack = false;
        var progress = 0;
        unbindTouchMoveFn = this.renderer.listen(element, 'touchmove', function (moveEvent) {
            var movePoint = moveEvent.touches[0];
            var moveX = movePoint.pageX;
            var moveY = movePoint.pageY;
            var distanceY = moveY - startY;
            var distanceX = moveX - startX;
            if (distanceX < distanceY && !isBack) {
                unbindFn();
                return;
            }
            isBack = true;
            if (distanceX < 0) {
                distanceX = 0;
            }
            else if (distanceX > maxWidth) {
                distanceX = maxWidth;
            }
            _this.distanceX = Math.floor(distanceX);
            progress = distanceX / maxWidth * _this.steps;
            _this.routerService.publishMoveBackProgress(progress);
        });
        unbindFn = function () {
            unbindTouchMoveFn();
            unbindTouchEndFn();
            unbindTouchCancelFn();
            if (progress === 0) {
                return;
            }
            var endTime = Date.now();
            if (endTime - startTime < 100 && _this.distanceX > 50 * _this.scale) {
                _this.animationTo(maxWidth);
                return;
            }
            if (_this.distanceX < maxWidth / 3) {
                _this.animationTo(0);
            }
            else {
                _this.animationTo(maxWidth);
            }
        };
        unbindTouchEndFn = this.renderer.listen(element, 'touchend', unbindFn);
        unbindTouchCancelFn = this.renderer.listen(element, 'touchcancel', unbindFn);
    };
    ContentComponent.prototype.animationTo = function (target) {
        var _this = this;
        var l = target - this.distanceX;
        var oldDistanceX = this.distanceX;
        var i = 0;
        var fn = function () {
            i++;
            _this.distanceX = oldDistanceX + l * Easing.Cubic.Out(i / 10);
            _this.routerService.publishMoveBackProgress(_this.distanceX / _this.maxWidth * _this.steps);
            if (i < 10) {
                _this.animationId = requestAnimationFrame(fn);
            }
        };
        this.animationId = requestAnimationFrame(fn);
    };
    ContentComponent.ctorParameters = function () { return [
        { type: ViewStateService },
        { type: ElementRef },
        { type: RouterService },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: Renderer2 },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
    ]; };
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", String)
    ], ContentComponent.prototype, "translate", void 0);
    __decorate([
        HostBinding('style.opacity'),
        __metadata("design:type", Number)
    ], ContentComponent.prototype, "opacity", void 0);
    __decorate([
        HostListener('touchstart', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], ContentComponent.prototype, "touchStart", null);
    ContentComponent = __decorate([
        Component({
            selector: 'ui-content',
            template: "<div class=\"ui-content-shadow\"></div>\n<div class=\"ui-content\">\n  <ng-content></ng-content>\n</div>\n"
        }),
        __param(3, Inject(UI_VIEW_INIT_STATE)),
        __param(4, Inject(UI_SCREEN_SCALE)),
        __param(6, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __metadata("design:paramtypes", [ViewStateService,
            ElementRef,
            RouterService, String, Number, Renderer2, Number])
    ], ContentComponent);
    return ContentComponent;
}());

var FooterComponent = /** @class */ (function () {
    function FooterComponent(viewStateService, state, steps) {
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.subs = [];
    }
    FooterComponent.prototype.ngOnInit = function () {
        var _this = this;
        var steps = this.steps;
        var bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.translate = "translate3d(" + p / steps * 100 + "%, 0, 0)";
            }
            else if (_this.state === ViewState.ToStack) {
                _this.translate = "translate3d(" + (-33 + 33 * p / steps) + "%, 0, 0)";
                _this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + (100 - progress * 100) + "%, 0, 0)";
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + progress * 100 + "%, 0, 0)";
                    break;
                case ViewState.ToStack:
                    _this.translate = "translate3d(" + progress * -33 + "%, 0, 0)";
                    _this.opacity = 1 - 0.1 * p / steps;
                    break;
                case ViewState.Reactivate:
                    var n = -33 + progress * 33;
                    // 当dom元素的style有transform属性时，会导致子级元素 position: fixed 全屏失效
                    // 会跟着有定位的父级同样大小
                    _this.translate = n === 0 ? '' : "translate3d(" + n + "%, 0, 0)";
                    _this.opacity = 0.9 + 0.1 * progress;
                    break;
            }
        }));
    };
    FooterComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    FooterComponent.ctorParameters = function () { return [
        { type: ViewStateService },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
    ]; };
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", String)
    ], FooterComponent.prototype, "translate", void 0);
    __decorate([
        HostBinding('style.opacity'),
        __metadata("design:type", Number)
    ], FooterComponent.prototype, "opacity", void 0);
    FooterComponent = __decorate([
        Component({
            selector: 'ui-footer',
            template: "<div class=\"ui-footer-shadow\"></div>\n<div class=\"ui-footer\">\n  <ng-content></ng-content>\n</div>"
        }),
        __param(1, Inject(UI_VIEW_INIT_STATE)),
        __param(2, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __metadata("design:paramtypes", [ViewStateService, String, Number])
    ], FooterComponent);
    return FooterComponent;
}());

var HeaderComponent = /** @class */ (function () {
    function HeaderComponent(viewStateService, state, steps) {
        this.viewStateService = viewStateService;
        this.state = state;
        this.steps = steps;
        this.subs = [];
    }
    HeaderComponent.prototype.ngOnInit = function () {
        var _this = this;
        var steps = this.steps;
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.opacity = 1 - p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = p / steps;
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1 - p / steps;
                    break;
                case ViewState.ToStack:
                case ViewState.Reactivate:
                    _this.opacity = 1;
                    break;
            }
        }));
    };
    HeaderComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    HeaderComponent.ctorParameters = function () { return [
        { type: ViewStateService },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] }
    ]; };
    __decorate([
        HostBinding('style.opacity'),
        __metadata("design:type", Number)
    ], HeaderComponent.prototype, "opacity", void 0);
    HeaderComponent = __decorate([
        Component({
            selector: 'ui-header',
            template: "<ng-content></ng-content>\n"
        }),
        __param(1, Inject(UI_VIEW_INIT_STATE)),
        __param(2, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __metadata("design:paramtypes", [ViewStateService, String, Number])
    ], HeaderComponent);
    return HeaderComponent;
}());

var NavBarComponent = /** @class */ (function () {
    function NavBarComponent() {
    }
    NavBarComponent = __decorate([
        Component({
            selector: 'ui-navbar',
            template: "<ng-content></ng-content>"
        })
    ], NavBarComponent);
    return NavBarComponent;
}());

var PageComponent = /** @class */ (function () {
    function PageComponent() {
    }
    PageComponent = __decorate([
        Component({
            selector: 'ui-page',
            template: "<ng-content></ng-content>"
        })
    ], PageComponent);
    return PageComponent;
}());

var TitleComponent = /** @class */ (function () {
    function TitleComponent(steps, state, elementRef, viewStateService) {
        this.steps = steps;
        this.state = state;
        this.elementRef = elementRef;
        this.viewStateService = viewStateService;
        this.subs = [];
        this.contentWidth = 0;
    }
    TitleComponent.prototype.resize = function () {
        this.docWidth = this.elementRef.nativeElement.offsetWidth;
        this.translateDistance = this.docWidth / 2 - this.contentWidth / 2;
    };
    TitleComponent.prototype.ngOnInit = function () {
        var _this = this;
        var steps = this.steps;
        var bezier = new CubicBezier(.36, .66, .04, 1);
        this.subs.push(this.viewStateService.state.subscribe(function (state) {
            _this.state = state;
        }));
        this.subs.push(this.viewStateService.touchProgress.subscribe(function (p) {
            var translate;
            if (_this.state === ViewState.Activate || _this.state === ViewState.Reactivate) {
                _this.translate = "translate3d(" + p / steps * 100 * 0.7 + "%, 0, 0)";
            }
            else if (_this.state === ViewState.ToStack) {
                translate = -_this.translateDistance + _this.translateDistance * p / steps;
                _this.translate = "translate3d(" + translate + "px, 0, 0)";
                _this.opacity = 0.9 + 0.1 * p / steps;
            }
        }));
        this.subs.push(this.viewStateService.progress.subscribe(function (p) {
            var progress = bezier.update(p / steps);
            var n;
            var translate;
            switch (_this.state) {
                case ViewState.Activate:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + (70 - progress * 70) + "%, 0, 0)";
                    break;
                case ViewState.Destroy:
                    _this.opacity = 1;
                    _this.translate = "translate3d(" + progress * 70 + "%, 0, 0)";
                    break;
                case ViewState.ToStack:
                    _this.translate = "translate3d(" + -progress * _this.translateDistance + "px, 0, 0)";
                    n = 1 - progress * 1.3;
                    _this.opacity = n < 0 ? 0 : n;
                    break;
                case ViewState.Reactivate:
                    translate = -_this.translateDistance + _this.translateDistance * progress;
                    _this.translate = "translate3d(" + translate + "px, 0, 0)";
                    n = progress * 2;
                    _this.opacity = n > 1 ? 1 : n;
                    break;
            }
        }));
    };
    TitleComponent.prototype.ngAfterViewInit = function () {
        this.docWidth = this.elementRef.nativeElement.offsetWidth;
        this.contentWidth = this.contentElement.nativeElement.offsetWidth;
        this.translateDistance = this.docWidth / 2 - this.contentWidth / 2;
    };
    TitleComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    TitleComponent.ctorParameters = function () { return [
        { type: Number, decorators: [{ type: Inject, args: [UI_ROUTER_ANIMATION_STEPS,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [UI_VIEW_INIT_STATE,] }] },
        { type: ElementRef },
        { type: ViewStateService }
    ]; };
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", String)
    ], TitleComponent.prototype, "translate", void 0);
    __decorate([
        HostBinding('style.opacity'),
        __metadata("design:type", Number)
    ], TitleComponent.prototype, "opacity", void 0);
    __decorate([
        ViewChild('content', { static: true }),
        __metadata("design:type", ElementRef)
    ], TitleComponent.prototype, "contentElement", void 0);
    __decorate([
        HostListener('window:resize'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], TitleComponent.prototype, "resize", null);
    TitleComponent = __decorate([
        Component({
            selector: 'ui-title',
            template: "<div #content>\n  <ng-content></ng-content>\n</div>"
        }),
        __param(0, Inject(UI_ROUTER_ANIMATION_STEPS)),
        __param(1, Inject(UI_VIEW_INIT_STATE)),
        __metadata("design:paramtypes", [Number, String, ElementRef,
            ViewStateService])
    ], TitleComponent);
    return TitleComponent;
}());

var ListActivatedService = /** @class */ (function () {
    function ListActivatedService() {
        this.activatedComponentSource = new Subject();
        this.activatedComponent = this.activatedComponentSource.asObservable();
    }
    ListActivatedService.prototype.publish = function () {
        this.activatedComponentSource.next();
    };
    ListActivatedService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ListActivatedService);
    return ListActivatedService;
}());

var ListEventService = /** @class */ (function () {
    function ListEventService() {
        this.listOptionsSource = new Subject();
        this.listOptions = this.listOptionsSource.asObservable();
    }
    ListEventService.prototype.addOption = function (elementRef) {
        this.listOptionsSource.next(elementRef);
    };
    ListEventService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ListEventService);
    return ListEventService;
}());

var ListComponent = /** @class */ (function () {
    function ListComponent() {
    }
    ListComponent = __decorate([
        Component({
            selector: 'ui-list',
            template: "<ng-content></ng-content>"
        })
    ], ListComponent);
    return ListComponent;
}());

var ListHeaderComponent = /** @class */ (function () {
    function ListHeaderComponent() {
    }
    ListHeaderComponent = __decorate([
        Component({
            selector: 'ui-list-header',
            template: "<ng-content></ng-content>"
        })
    ], ListHeaderComponent);
    return ListHeaderComponent;
}());

var ListInnerComponent = /** @class */ (function () {
    function ListInnerComponent() {
    }
    ListInnerComponent = __decorate([
        Component({
            selector: 'ui-list-inner',
            template: "<ng-content></ng-content>"
        })
    ], ListInnerComponent);
    return ListInnerComponent;
}());

var ListItemComponent = /** @class */ (function () {
    function ListItemComponent() {
    }
    ListItemComponent = __decorate([
        Component({
            selector: 'ui-list-item',
            template: "<ng-content></ng-content>",
            providers: [
                // 提供一个服务，供 list-sliding 组件发起通信
                ListEventService
            ]
        })
    ], ListItemComponent);
    return ListItemComponent;
}());

var ListOptionComponent = /** @class */ (function () {
    function ListOptionComponent(listEventService, elementRef) {
        this.listEventService = listEventService;
        this.elementRef = elementRef;
    }
    ListOptionComponent.prototype.ngAfterViewInit = function () {
        // 发布当前组件的 dom 元素，以便让 list-sliding 组件在划动时，计算最大划动宽度
        this.listEventService.addOption(this.elementRef);
    };
    ListOptionComponent.ctorParameters = function () { return [
        { type: ListEventService },
        { type: ElementRef }
    ]; };
    ListOptionComponent = __decorate([
        Component({
            selector: 'ui-list-option',
            template: "<ng-content></ng-content>"
        }),
        __metadata("design:paramtypes", [ListEventService,
            ElementRef])
    ], ListOptionComponent);
    return ListOptionComponent;
}());

var ListSlidingComponent = /** @class */ (function () {
    function ListSlidingComponent(listEventService, scale, listActivatedService, renderer) {
        this.listEventService = listEventService;
        this.scale = scale;
        this.listActivatedService = listActivatedService;
        this.renderer = renderer;
        this.subs = [];
        this.distanceX = 0;
        this.refs = [];
        this.isFocus = false;
    }
    ListSlidingComponent.prototype.ngOnInit = function () {
        var _this = this;
        // 拿到当前 list-item 组件下，所有的 list-options 组件，以便在划动时，计算 dom 的宽度
        this.subs.push(this.listEventService.listOptions.subscribe(function (elementRef) {
            _this.refs.push(elementRef);
        }));
        // 订阅选中事件，并判断用户操作的 list-sliding 是不是自身，如果不是自身，则修改自身相应样式
        this.subs.push(this.listActivatedService.activatedComponent.subscribe(function () {
            if (!_this.isFocus) {
                _this.distanceX = 0;
                _this.transitionDuration = '';
                _this.transform = 'translateX(0px)';
            }
        }));
    };
    ListSlidingComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) {
            item.unsubscribe();
        });
    };
    ListSlidingComponent.prototype.touchstart = function (event) {
        var _this = this;
        this.isFocus = true;
        var touchPoint = event.touches[0];
        var startX = touchPoint.pageX;
        var startY = touchPoint.pageY;
        var oldDistanceX = this.distanceX;
        var startTime = Date.now();
        var maxDistance = 0;
        this.refs.forEach(function (item) {
            maxDistance += item.nativeElement.offsetWidth;
        });
        var isClick = true;
        var isScroll = true;
        // 设置当前元素的 css 动画时间为0，防止有迟滞感
        this.transitionDuration = '0s';
        var unBindTouchEndFn;
        var unBindTouchCancelFn;
        var unBindTouchMoveFn = this.renderer.listen('document', 'touchmove', function (moveEvent) {
            isClick = false;
            var newTouchPoint = moveEvent.touches[0];
            var moveX = newTouchPoint.pageX;
            var moveY = newTouchPoint.pageY;
            // 当用户触摸的纵向距离大于横向距离，能忽略此次划动，不触发各左滑动的动作
            if (isScroll && Math.abs(moveX - startX) < Math.abs(moveY - startY)) {
                unBindTouchCancelFn();
                unBindTouchMoveFn();
                unBindTouchEndFn();
                return;
            }
            // 计算向左滑动的距离
            _this.distanceX = moveX - startX + oldDistanceX;
            if (_this.distanceX > 0) {
                _this.distanceX = 0;
            }
            if (_this.distanceX < -maxDistance) {
                _this.distanceX = -maxDistance;
            }
            // 设置相应样式，并阻止事件冒泡和默认事件
            _this.transform = "translateX(" + _this.distanceX + "px)";
            isScroll = false;
            moveEvent.stopPropagation();
            return false;
        });
        var touchEndFn = function () {
            _this.isFocus = false;
            var endTime = Date.now();
            var distance = oldDistanceX - _this.distanceX;
            // 当用户触摸完成后，时间小于 100ms，并且距离大于20，则认为是惯性触摸
            if (endTime - startTime < 100 && Math.abs(distance) > 20 * _this.scale) {
                _this.distanceX = distance < 0 ? 0 : -maxDistance;
            }
            else {
                // 否则按 50% 区分向左还是向右
                _this.distanceX = _this.distanceX > maxDistance / -2 ? 0 : -maxDistance;
            }
            _this.transitionDuration = '';
            _this.transform = "translateX(" + _this.distanceX + "px)";
            unBindTouchCancelFn();
            unBindTouchMoveFn();
            unBindTouchEndFn();
        };
        unBindTouchEndFn = this.renderer.listen('document', 'touchend', touchEndFn);
        unBindTouchCancelFn = this.renderer.listen('document', 'touchcancel', touchEndFn);
        setTimeout(function () {
            if (isClick) {
                unBindTouchCancelFn();
                unBindTouchMoveFn();
                unBindTouchEndFn();
            }
        }, 100);
    };
    ListSlidingComponent.ctorParameters = function () { return [
        { type: ListEventService },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: ListActivatedService },
        { type: Renderer2 }
    ]; };
    __decorate([
        HostBinding('style.transitionDuration'),
        __metadata("design:type", String)
    ], ListSlidingComponent.prototype, "transitionDuration", void 0);
    __decorate([
        HostBinding('style.transform'),
        __metadata("design:type", String)
    ], ListSlidingComponent.prototype, "transform", void 0);
    __decorate([
        HostListener('touchstart', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], ListSlidingComponent.prototype, "touchstart", null);
    ListSlidingComponent = __decorate([
        Component({
            selector: 'ui-list-sliding',
            template: "<ng-content></ng-content>"
        }),
        __param(1, Inject(UI_SCREEN_SCALE)),
        __metadata("design:paramtypes", [ListEventService, Number, ListActivatedService,
            Renderer2])
    ], ListSlidingComponent);
    return ListSlidingComponent;
}());

var ListThumbnailComponent = /** @class */ (function () {
    function ListThumbnailComponent() {
    }
    ListThumbnailComponent = __decorate([
        Component({
            selector: 'ui-list-thumbnail',
            template: "<ng-content></ng-content>"
        })
    ], ListThumbnailComponent);
    return ListThumbnailComponent;
}());

var AlertController = /** @class */ (function () {
    function AlertController() {
        this.alertActionSource = new Subject();
        this.alertConfigSource = new Subject();
        this.alertAction = this.alertActionSource.asObservable();
        this.alertConfig = this.alertConfigSource.asObservable();
    }
    AlertController.prototype.show = function (params) {
        // 当接收到用户事件时，发布相应事件，并返回 promise，当用户点击确定时，resolve 当前 promise;
        var _this = this;
        this.alertConfigSource.next(typeof params === 'string' ? {
            content: params
        } : params);
        return new Promise(function (resolve) {
            var sub = _this.alertAction.subscribe(function () {
                resolve();
                sub.unsubscribe();
            });
        });
    };
    AlertController.prototype.publishAction = function () {
        // 发布用户点击弹出框的确认按扭事件
        this.alertActionSource.next();
    };
    AlertController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], AlertController);
    return AlertController;
}());

var DialogController = /** @class */ (function () {
    function DialogController() {
        this.actionEvent = new Subject();
        this.configSource = new Subject();
        this.action = this.actionEvent.asObservable();
        this.config = this.configSource.asObservable();
    }
    DialogController.prototype.show = function (params) {
        var _this = this;
        this.configSource.next(typeof params === 'string' ? {
            content: params
        } : params);
        return new Promise(function (resolve) {
            var sub = _this.action.subscribe(function (result) {
                resolve(result);
                sub.unsubscribe();
            });
        });
    };
    DialogController.prototype.publishAction = function (result) {
        this.actionEvent.next(result);
    };
    DialogController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], DialogController);
    return DialogController;
}());

var ModalController = /** @class */ (function () {
    function ModalController() {
        this.displayTemplate = new Subject();
        this.hideEvent = new Subject();
        this.template = this.displayTemplate.asObservable();
        this.onHide = this.hideEvent.asObservable();
    }
    ModalController.prototype.show = function (template) {
        this.displayTemplate.next(template);
    };
    ModalController.prototype.hide = function () {
        this.hideEvent.next();
    };
    ModalController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ModalController);
    return ModalController;
}());

var ToastType;
(function (ToastType) {
    ToastType["Default"] = "default";
    ToastType["Primary"] = "primary";
    ToastType["Info"] = "info";
    ToastType["Warning"] = "warning";
    ToastType["Success"] = "success";
    ToastType["Danger"] = "danger";
})(ToastType || (ToastType = {}));
var ToastController = /** @class */ (function () {
    function ToastController() {
        this.notifySource = new Subject();
        this.notify = this.notifySource.asObservable();
    }
    ToastController.prototype.push = function (config) {
        this.notifySource.next(config);
    };
    ToastController = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], ToastController);
    return ToastController;
}());

var AlertComponent = /** @class */ (function () {
    function AlertComponent(alertController, router) {
        this.alertController = alertController;
        this.router = router;
        this.show = false;
        this.title = '';
        this.content = '';
        this.btnText = '';
        this.subs = [];
    }
    Object.defineProperty(AlertComponent.prototype, "titleIsString", {
        get: function () {
            return typeof this.title === 'string';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AlertComponent.prototype, "contentIsString", {
        get: function () {
            return typeof this.content === 'string';
        },
        enumerable: true,
        configurable: true
    });
    AlertComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (this.router) {
            this.subs.push(this.router.events.subscribe(function (event) {
                if (event instanceof NavigationStart) {
                    _this.checked();
                }
            }));
        }
        // 订阅alert事件
        this.subs.push(this.alertController.alertConfig.subscribe(function (params) {
            // 设置状态，以弹出对话框
            _this.show = true;
            _this.title = params.title;
            _this.content = params.content;
            _this.btnText = params.btnText;
        }));
    };
    AlertComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    AlertComponent.prototype.checked = function () {
        // 当点击确认按扭时关闭弹窗
        this.show = false;
    };
    AlertComponent.prototype.hide = function () {
        // 当弹窗动画完成时，发布事件
        this.alertController.publishAction();
    };
    AlertComponent.ctorParameters = function () { return [
        { type: AlertController },
        { type: Router, decorators: [{ type: Optional }] }
    ]; };
    AlertComponent = __decorate([
        Component({
            selector: 'ui-alert',
            template: "<ui-dialog-base [show]=\"show\" (uiHide)=\"hide()\">\n  <ui-modal>\n    <ng-container *ngIf=\"title\">\n      <ui-modal-header *ngIf=\"titleIsString\">{{ title }}</ui-modal-header>\n      <ui-modal-header *ngIf=\"!titleIsString\">\n        <ng-container [ngTemplateOutlet]=\"title\"></ng-container>\n      </ui-modal-header>\n    </ng-container>\n    <ui-modal-inner *ngIf=\"contentIsString\">{{ content }}</ui-modal-inner>\n    <ui-modal-inner *ngIf=\"!contentIsString\">\n      <ng-container [ngTemplateOutlet]=\"content\"></ng-container>\n    </ui-modal-inner>\n    <ui-modal-footer class=\"ui-alert-footer\">\n      <button (click)=\"checked()\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnText || '\u786E\u5B9A' }}</button>\n    </ui-modal-footer>\n  </ui-modal>\n</ui-dialog-base>"
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [AlertController,
            Router])
    ], AlertComponent);
    return AlertComponent;
}());

var DialogComponent = /** @class */ (function () {
    function DialogComponent(dialogController, router) {
        this.dialogController = dialogController;
        this.router = router;
        this.show = false;
        this.title = '';
        this.content = '';
        this.btnsText = ['取消', '确认'];
        this.result = false;
        this.subs = [];
    }
    Object.defineProperty(DialogComponent.prototype, "titleIsString", {
        get: function () {
            return typeof this.title === 'string';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogComponent.prototype, "contentIsString", {
        get: function () {
            return typeof this.content === 'string';
        },
        enumerable: true,
        configurable: true
    });
    DialogComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (this.router) {
            this.subs.push(this.router.events.subscribe(function (event) {
                if (event instanceof NavigationStart) {
                    _this.show = false;
                }
            }));
        }
        // 订阅用户事件
        this.subs.push(this.dialogController.config.subscribe(function (params) {
            // 设置动画状态
            _this.show = true;
            // 赋值相应参数
            _this.title = params.title;
            _this.content = params.content;
            if (params.btnsText) {
                _this.btnsText = params.btnsText;
            }
        }));
    };
    DialogComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    DialogComponent.prototype.checked = function (result) {
        // 当用户点击按扭时，关闭弹窗
        this.show = false;
        this.result = result;
    };
    DialogComponent.prototype.hide = function () {
        // 当弹窗关闭动画完成时，发布相应事件
        this.dialogController.publishAction(this.result);
    };
    DialogComponent.ctorParameters = function () { return [
        { type: DialogController },
        { type: Router, decorators: [{ type: Optional }] }
    ]; };
    DialogComponent = __decorate([
        Component({
            selector: 'ui-dialog',
            template: "<ui-dialog-base [show]=\"show\" (uiHide)=\"hide()\">\n  <ui-modal>\n    <ng-container *ngIf=\"title\">\n      <ui-modal-header *ngIf=\"titleIsString\">{{ title }}</ui-modal-header>\n      <ui-modal-header *ngIf=\"!titleIsString\">\n        <ng-container [ngTemplateOutlet]=\"title\"></ng-container>\n      </ui-modal-header>\n    </ng-container>\n    <ui-modal-inner *ngIf=\"contentIsString\">{{ content }}</ui-modal-inner>\n    <ui-modal-inner *ngIf=\"!contentIsString\">\n      <ng-container [ngTemplateOutlet]=\"content\"></ng-container>\n    </ui-modal-inner>\n    <ui-modal-footer class=\"ui-dialog-footer\">\n      <button (click)=\"checked(false)\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnsText[0] }}</button>\n      <button (click)=\"checked(true)\" type=\"button\" class=\"ui-btn ui-btn-lg ui-btn-block\">{{ btnsText[1] }}</button>\n    </ui-modal-footer>\n  </ui-modal>\n</ui-dialog-base>"
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [DialogController,
            Router])
    ], DialogComponent);
    return DialogComponent;
}());

var DialogBaseComponent = /** @class */ (function () {
    function DialogBaseComponent() {
        this.uiHide = new EventEmitter();
        this.display = false;
        this._show = false;
    }
    Object.defineProperty(DialogBaseComponent.prototype, "show", {
        get: function () {
            return this._show;
        },
        set: function (value) {
            this._show = value;
            if (value) {
                this.display = true;
            }
        },
        enumerable: true,
        configurable: true
    });
    DialogBaseComponent.prototype.animationEnd = function () {
        this.uiHide.emit();
        this.display = false;
    };
    DialogBaseComponent.prototype.hide = function () {
        if (!this.show) {
            this.display = false;
        }
    };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], DialogBaseComponent.prototype, "uiHide", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], DialogBaseComponent.prototype, "show", null);
    DialogBaseComponent = __decorate([
        Component({
            selector: 'ui-dialog-base',
            template: "<ui-mask [show]=\"show\" (uiHide)=\"animationEnd()\">\n  <div class=\"ui-dialog-base\" *ngIf=\"display\" (animationend)=\"hide()\" [ngClass]=\"show ? 'ui-in' : 'ui-out'\">\n    <ng-content></ng-content>\n  </div>\n</ui-mask>"
        })
    ], DialogBaseComponent);
    return DialogBaseComponent;
}());

var MaskComponent = /** @class */ (function () {
    function MaskComponent() {
        this.uiHide = new EventEmitter();
        this.show = false;
    }
    MaskComponent.prototype.done = function () {
        if (!this.show) {
            this.uiHide.emit();
        }
    };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], MaskComponent.prototype, "uiHide", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], MaskComponent.prototype, "show", void 0);
    MaskComponent = __decorate([
        Component({
            selector: 'ui-mask',
            template: "<div [ngClass]=\"{'ui-show': show}\" class=\"ui-mask\" (webkitTransitionEnd)=\"done()\" (transitionend)=\"done()\"></div>\n<ng-content></ng-content>"
        })
    ], MaskComponent);
    return MaskComponent;
}());

var ModalComponent = /** @class */ (function () {
    function ModalComponent() {
    }
    ModalComponent = __decorate([
        Component({
            selector: 'ui-modal',
            template: "<ng-content></ng-content>\n"
        })
    ], ModalComponent);
    return ModalComponent;
}());

var ModalBaseComponent = /** @class */ (function () {
    function ModalBaseComponent(modalController, router) {
        this.modalController = modalController;
        this.router = router;
        this.isShow = false;
        this.subs = [];
    }
    ModalBaseComponent.prototype.ngOnInit = function () {
        var _this = this;
        if (this.router) {
            this.subs.push(this.router.events.subscribe(function (event) {
                if (event instanceof NavigationStart) {
                    _this.isShow = false;
                }
            }));
        }
        this.subs.push(this.modalController.template.subscribe(function (template) {
            _this.isShow = true;
            _this.template = template;
        }));
        this.subs.push(this.modalController.onHide.subscribe(function () {
            _this.isShow = false;
        }));
    };
    ModalBaseComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    ModalBaseComponent.prototype.animationEnd = function () {
        this.template = null;
    };
    ModalBaseComponent.ctorParameters = function () { return [
        { type: ModalController },
        { type: Router, decorators: [{ type: Optional }] }
    ]; };
    ModalBaseComponent = __decorate([
        Component({
            selector: 'ui-modal-base',
            template: "<ui-dialog-base [show]=\"isShow\" (uiHide)=\"animationEnd()\">\n  <ng-container [ngTemplateOutlet]=\"template\"></ng-container>\n</ui-dialog-base>"
        }),
        __param(1, Optional()),
        __metadata("design:paramtypes", [ModalController,
            Router])
    ], ModalBaseComponent);
    return ModalBaseComponent;
}());

var ModalFooterComponent = /** @class */ (function () {
    function ModalFooterComponent() {
    }
    ModalFooterComponent = __decorate([
        Component({
            selector: 'ui-modal-footer',
            template: "<ng-content></ng-content>"
        })
    ], ModalFooterComponent);
    return ModalFooterComponent;
}());

var ModalHeaderComponent = /** @class */ (function () {
    function ModalHeaderComponent() {
    }
    ModalHeaderComponent = __decorate([
        Component({
            selector: 'ui-modal-header',
            template: "<ng-content></ng-content>"
        })
    ], ModalHeaderComponent);
    return ModalHeaderComponent;
}());

var ModalInnerComponent = /** @class */ (function () {
    function ModalInnerComponent() {
    }
    ModalInnerComponent = __decorate([
        Component({
            selector: 'ui-modal-inner',
            template: "<ng-content></ng-content>"
        })
    ], ModalInnerComponent);
    return ModalInnerComponent;
}());

var ToastComponent = /** @class */ (function () {
    function ToastComponent(toastService) {
        this.toastService = toastService;
        this.messageList = [];
        this.fadeIn = false;
        this.fadeOut = false;
        this.timer = null;
    }
    ToastComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.toastService.notify.pipe(delay(0)).subscribe(function (config) {
            _this.fadeOut = false;
            _this.fadeIn = true;
            var _config = {};
            _config.rawConfig = config;
            switch (config.type) {
                case ToastType.Default:
                    _config.type = 'ui-default';
                    break;
                case ToastType.Primary:
                    _config.type = 'ui-primary';
                    break;
                case ToastType.Info:
                    _config.type = 'ui-info';
                    break;
                case ToastType.Success:
                    _config.type = 'ui-success';
                    break;
                case ToastType.Warning:
                    _config.type = 'ui-warning';
                    break;
                case ToastType.Danger:
                    _config.type = 'ui-danger';
                    break;
                default:
                    _config.type = 'ui-default';
            }
            _config.content = config.content || '';
            _config.time = config.time || 2000;
            _this.messageList.push(_config);
            _this.timer = setTimeout(function () {
                for (var i = 0; i < _this.messageList.length; i++) {
                    if (_this.messageList[i] === _config) {
                        _this.messageList.splice(i, 1);
                        return;
                    }
                }
                _this.fadeIn = false;
                _this.fadeOut = true;
            }, _config.time);
        });
    };
    ToastComponent.prototype.ngOnDestroy = function () {
        clearTimeout(this.timer);
    };
    ToastComponent.ctorParameters = function () { return [
        { type: ToastController }
    ]; };
    __decorate([
        HostBinding('class.ui-toast-fadein'),
        __metadata("design:type", Boolean)
    ], ToastComponent.prototype, "fadeIn", void 0);
    __decorate([
        HostBinding('class.ui-toast-fadeout'),
        __metadata("design:type", Boolean)
    ], ToastComponent.prototype, "fadeOut", void 0);
    ToastComponent = __decorate([
        Component({
            selector: 'ui-toast',
            template: "<div *ngFor=\"let config of messageList;\"\n     class=\"ui-toast\"\n     [ngClass]=\"config.type || 'ui-default'\">\n  {{ config.content }}\n</div>",
            animations: [
                trigger('toastState', [state('*', style({
                        opacity: 1
                    })), transition(':leave', animate('.15s', style({
                        opacity: 0,
                    }))), transition(':enter', [style({
                            opacity: 0,
                        }), animate('.15s', style({
                            opacity: 1,
                        }))])])
            ]
        }),
        __metadata("design:paramtypes", [ToastController])
    ], ToastComponent);
    return ToastComponent;
}());

var SegmentService = /** @class */ (function () {
    function SegmentService() {
        this.checkedSource = new Subject();
        this.onChecked = this.checkedSource.asObservable();
    }
    SegmentService.prototype.checked = function (option) {
        this.checkedSource.next(option);
    };
    SegmentService = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [])
    ], SegmentService);
    return SegmentService;
}());

function inputAttrToBoolean$1(value) {
    return value === '' || !!value;
}
var SegmentButtonComponent = /** @class */ (function () {
    function SegmentButtonComponent(elementRef, segmentService) {
        this.elementRef = elementRef;
        this.segmentService = segmentService;
        this.uiChecked = new EventEmitter();
        this.value = '';
        this._disabled = false;
        this._selected = false;
    }
    Object.defineProperty(SegmentButtonComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (isDisabled) {
            this._disabled = inputAttrToBoolean$1(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SegmentButtonComponent.prototype, "selected", {
        get: function () {
            return this._selected;
        },
        set: function (isSelected) {
            this._selected = inputAttrToBoolean$1(isSelected);
        },
        enumerable: true,
        configurable: true
    });
    SegmentButtonComponent.prototype.click = function () {
        if (!this.disabled) {
            this.segmentService.checked(this);
            this.uiChecked.emit(this);
        }
    };
    SegmentButtonComponent.prototype.ngAfterViewInit = function () {
        this.nativeElement = this.elementRef.nativeElement;
    };
    SegmentButtonComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: SegmentService }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SegmentButtonComponent.prototype, "uiChecked", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SegmentButtonComponent.prototype, "value", void 0);
    __decorate([
        Input(),
        HostBinding('class.ui-disabled'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SegmentButtonComponent.prototype, "disabled", null);
    __decorate([
        Input(),
        HostBinding('class.ui-selected'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SegmentButtonComponent.prototype, "selected", null);
    __decorate([
        HostListener('click'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], SegmentButtonComponent.prototype, "click", null);
    SegmentButtonComponent = __decorate([
        Component({
            selector: 'ui-segment-button',
            template: "<ng-content></ng-content>"
        }),
        __metadata("design:paramtypes", [ElementRef,
            SegmentService])
    ], SegmentButtonComponent);
    return SegmentButtonComponent;
}());

var SegmentComponent = /** @class */ (function () {
    function SegmentComponent(segmentService) {
        this.segmentService = segmentService;
        this.uiChange = new EventEmitter();
        this.selectedIndex = 0;
    }
    SegmentComponent_1 = SegmentComponent;
    SegmentComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        var defaultOption;
        this.options.forEach(function (option, index) {
            if (option.selected) {
                defaultOption = option;
                _this.selectedIndex = index;
            }
        });
        if (!defaultOption) {
            defaultOption = this.options.toArray()[this.selectedIndex];
        }
        if (!defaultOption) {
            defaultOption = this.options.first;
            this.selectedIndex = 0;
        }
        if (defaultOption) {
            this.value = defaultOption.value;
            defaultOption.selected = true;
            this.defaultOption = defaultOption;
        }
        this.sub = this.segmentService.onChecked.subscribe(function (option) {
            _this.options.forEach(function (op, index) {
                if (op === option) {
                    op.selected = true;
                    _this.value = option.value;
                    _this.selectedIndex = index;
                    if (_this.onChange) {
                        _this.onChange(_this.value);
                    }
                    if (_this.onTouched) {
                        _this.onTouched(_this.value);
                    }
                    _this.uiChange.emit(_this.value);
                }
                else {
                    op.selected = false;
                }
            });
        });
    };
    SegmentComponent.prototype.ngOnDestroy = function () {
        this.sub.unsubscribe();
    };
    SegmentComponent.prototype.writeValue = function (value) {
        var _this = this;
        this.value = value;
        if (this.options) {
            var selectedOption_1;
            this.options.forEach(function (item, index) {
                item.selected = false;
                if (item.value === value || "" + item.value === value || item.value === "" + value) {
                    selectedOption_1 = item;
                    _this.selectedIndex = index;
                }
            });
            if (selectedOption_1) {
                selectedOption_1.selected = true;
            }
        }
    };
    SegmentComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SegmentComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    var SegmentComponent_1;
    SegmentComponent.ctorParameters = function () { return [
        { type: SegmentService }
    ]; };
    __decorate([
        ContentChildren(SegmentButtonComponent),
        __metadata("design:type", QueryList)
    ], SegmentComponent.prototype, "options", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SegmentComponent.prototype, "uiChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], SegmentComponent.prototype, "selectedIndex", void 0);
    SegmentComponent = SegmentComponent_1 = __decorate([
        Component({
            selector: 'ui-segment',
            template: "<ng-content></ng-content>",
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: SegmentComponent_1,
                    multi: true
                }, SegmentService]
        }),
        __metadata("design:paramtypes", [SegmentService])
    ], SegmentComponent);
    return SegmentComponent;
}());

var SlideItemComponent = /** @class */ (function () {
    function SlideItemComponent() {
        this.state = '';
        this.animateProgress = 0;
    }
    Object.defineProperty(SlideItemComponent.prototype, "isIn", {
        get: function () {
            return this.state === 'in';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SlideItemComponent.prototype, "isOut", {
        get: function () {
            return this.state === 'out';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SlideItemComponent.prototype, "transform", {
        get: function () {
            return "translateX(" + this.animateProgress * 100 + "%)";
        },
        enumerable: true,
        configurable: true
    });
    __decorate([
        HostBinding('class.ui-in'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], SlideItemComponent.prototype, "isIn", null);
    __decorate([
        HostBinding('class.ui-out'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], SlideItemComponent.prototype, "isOut", null);
    __decorate([
        HostBinding('style.transform'),
        HostBinding('style.webkitTransform'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], SlideItemComponent.prototype, "transform", null);
    SlideItemComponent = __decorate([
        Component({
            selector: 'ui-slide-item',
            template: "<ng-content></ng-content>\n"
        })
    ], SlideItemComponent);
    return SlideItemComponent;
}());

var SlideComponent = /** @class */ (function () {
    function SlideComponent(elementRef, scale, appController) {
        this.elementRef = elementRef;
        this.scale = scale;
        this.appController = appController;
        this.uiPlayed = new EventEmitter();
        this.initIndex = 0;
        this.speed = 2000;
        this.steps = 35;
        this.progress = 0;
        this.oldProgress = 0;
        this.timer = null;
        this.subs = [];
        this._autoPlay = true;
    }
    Object.defineProperty(SlideComponent.prototype, "autoPlay", {
        get: function () {
            return this._autoPlay;
        },
        set: function (v) {
            this._autoPlay = v;
            if (v) {
                this.play();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SlideComponent.prototype, "index", {
        get: function () {
            return Math.ceil(this.progress - 0.5) % this.items.length;
        },
        enumerable: true,
        configurable: true
    });
    SlideComponent.prototype.ngOnInit = function () {
        this.progress = this.initIndex;
    };
    SlideComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.containerWidth = this.elementRef.nativeElement.offsetWidth;
        this.subs.push(this.appController.onResize.subscribe(function () {
            _this.containerWidth = _this.elementRef.nativeElement.offsetWidth;
        }));
        setTimeout(function () {
            _this.updateChildrenStyle(_this.progress);
            _this.play();
        });
        this.subs.push(this.items.changes.pipe(delay(0)).subscribe(function () {
            _this.updateChildrenStyle(0);
            _this.play();
        }));
    };
    SlideComponent.prototype.ngOnDestroy = function () {
        clearTimeout(this.timer);
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    SlideComponent.prototype.touch = function (event) {
        var _this = this;
        clearTimeout(this.timer);
        cancelAnimationFrame(this.animateId);
        event.srcEvent.stopPropagation();
        var firstDirection = event.firstDirection;
        var durationTime = event.durationTime;
        var element = this.elementRef.nativeElement;
        this.containerWidth = element.offsetWidth;
        var len = this.items.length;
        var distanceX = event.distanceX;
        var isFirstMoving = event.first;
        var autoUpdateStyle = function () {
            var min = 0;
            var max = 20;
            var p = _this.progress;
            var target;
            if (durationTime < 200 && Math.abs(distanceX) > 20 * _this.scale) {
                target = distanceX > 0 ? Math.floor(p) : Math.floor(p) + 1;
            }
            else {
                target = p % 1 > 0.5 ? Math.ceil(p) : Math.floor(p);
            }
            var distance = target - p;
            var updateStyle = function () {
                min++;
                _this.progress = (p + Easing.Cubic.Out(min / max) * distance) % len;
                _this.updateChildrenStyle(_this.progress);
                if (min < max) {
                    _this.animateId = requestAnimationFrame(updateStyle);
                }
                else {
                    _this.uiPlayed.emit(_this.progress);
                    _this.play();
                }
            };
            _this.animateId = requestAnimationFrame(updateStyle);
        };
        if (this.items.length < 2) {
            return;
        }
        if (firstDirection === 'up' || firstDirection === 'down') {
            this.touchAction = undefined;
            event.stop();
            return;
        }
        this.touchAction = 'pan-x';
        if (isFirstMoving) {
            this.oldProgress = this.progress;
        }
        this.progress = this.oldProgress - (distanceX) / this.containerWidth;
        if (this.progress < 0) {
            this.progress += len;
        }
        this.updateChildrenStyle(this.progress);
        if (event.type === 'touchend') {
            this.touchAction = undefined;
            autoUpdateStyle();
        }
    };
    SlideComponent.prototype.play = function () {
        var _this = this;
        clearTimeout(this.timer);
        if (!this.autoPlay) {
            return;
        }
        if (this.items.length < 2) {
            return;
        }
        this.timer = setTimeout(function () {
            _this.animate();
        }, this.speed);
    };
    SlideComponent.prototype.animate = function () {
        var _this = this;
        var progress = this.progress;
        var i = 0;
        var animateFn = function () {
            i++;
            _this.progress = (progress + Easing.Cubic.Out(i / _this.steps)) % _this.items.length;
            _this.updateChildrenStyle(_this.progress);
            if (i < _this.steps) {
                _this.animateId = requestAnimationFrame(animateFn);
            }
            else {
                _this.uiPlayed.emit(_this.progress);
                _this.play();
            }
        };
        this.animateId = requestAnimationFrame(animateFn);
    };
    SlideComponent.prototype.updateChildrenStyle = function (progress) {
        var currentIndex = Math.floor(progress);
        var p = progress % 1;
        var len = this.items.length;
        this.items.forEach(function (item, index) {
            if (index === currentIndex) {
                item.state = 'in';
                item.animateProgress = -p;
            }
            else if (index === (currentIndex + 1) % len) {
                item.state = 'out';
                item.animateProgress = -p + 1;
            }
            else {
                item.animateProgress = -1;
                item.state = '';
            }
        });
    };
    SlideComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: AppController }
    ]; };
    __decorate([
        ContentChildren(SlideItemComponent),
        __metadata("design:type", QueryList)
    ], SlideComponent.prototype, "items", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], SlideComponent.prototype, "uiPlayed", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SlideComponent.prototype, "initIndex", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SlideComponent.prototype, "speed", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SlideComponent.prototype, "steps", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean),
        __metadata("design:paramtypes", [Boolean])
    ], SlideComponent.prototype, "autoPlay", null);
    SlideComponent = __decorate([
        Component({
            selector: 'ui-slide',
            template: "<div class=\"ui-slide-group\" (uiPan)=\"touch($event)\" [style.touchAction]=\"touchAction\">\n  <ng-content></ng-content>\n</div>\n<div class=\"ui-slide-buttons\" *ngIf=\"items.length > 1\">\n  <div class=\"ui-slide-btn\" *ngFor=\"let item of items; let i = index\" [ngClass]=\"{'ui-active': i === index}\"></div>\n</div>"
        }),
        __param(1, Inject(UI_SCREEN_SCALE)),
        __metadata("design:paramtypes", [ElementRef, Number, AppController])
    ], SlideComponent);
    return SlideComponent;
}());

var TouchManager = /** @class */ (function () {
    function TouchManager() {
    }
    TouchManager.prototype.init = function (points) {
        var _this = this;
        var isLock = false;
        this.touchSubscription = fromEvent(this.elementRef.nativeElement, 'touchstart')
            .pipe(filter(function () { return !isLock; }), debounceTime(points === 1 ? 0 : 100), filter(function (ev) {
            return ev.touches.length === points;
        }), tap(function () {
            isLock = true;
        }))
            .subscribe(function (touchStartEvent) {
            _this.touchStart(touchStartEvent);
            var unSubMove = fromEvent(_this.elementRef.nativeElement, 'touchmove')
                .subscribe(function (touchMoveEvent) {
                _this.touchMove(touchMoveEvent, unListen);
            });
            var unSubEnd = merge(fromEvent(_this.elementRef.nativeElement, 'touchend'), fromEvent(_this.elementRef.nativeElement, 'touchcancel'))
                .subscribe(function (ev) {
                unListen();
                _this.touchEnd(ev);
            });
            var unListen = function () {
                isLock = false;
                unSubMove.unsubscribe();
                unSubEnd.unsubscribe();
            };
        });
    };
    TouchManager.prototype.destroy = function () {
        if (this.touchSubscription) {
            this.touchSubscription.unsubscribe();
        }
    };
    return TouchManager;
}());

var PanDirective = /** @class */ (function (_super) {
    __extends(PanDirective, _super);
    function PanDirective(elementRef) {
        var _this = _super.call(this) || this;
        _this.elementRef = elementRef;
        _this.uiPan = new EventEmitter();
        _this.cumulativeDistanceX = 0;
        _this.cumulativeDistanceY = 0;
        _this.distanceX = 0;
        _this.distanceY = 0;
        _this.oldMoveX = 0;
        _this.oldMoveY = 0;
        _this.direction = 'origin';
        _this.isFirst = true;
        return _this;
    }
    PanDirective.prototype.ngOnInit = function () {
        _super.prototype.init.call(this, 1);
    };
    PanDirective.prototype.ngOnDestroy = function () {
        _super.prototype.destroy.call(this);
    };
    PanDirective.prototype.touchStart = function (event) {
        var startPoint = event.touches[0];
        this.startX = startPoint.clientX;
        this.startY = startPoint.clientY;
        this.moveX = this.startX;
        this.moveY = this.startY;
        this.distanceX = 0;
        this.distanceY = 0;
        this.oldMoveX = 0;
        this.oldMoveY = 0;
        this.direction = 'origin';
        this.isFirst = true;
        this.startTime = event.timeStamp;
    };
    PanDirective.prototype.touchMove = function (event, unListen) {
        var _this = this;
        var point = event.touches[0];
        this.oldMoveX = this.moveX;
        this.oldMoveY = this.moveY;
        this.moveX = point.clientX;
        this.moveY = point.clientY;
        this.distanceX = this.moveX - this.startX;
        this.distanceY = this.moveY - this.startY;
        if (this.isFirst) {
            if (Math.abs(this.distanceX) > Math.abs(this.distanceY)) {
                this.direction = this.distanceX > 0 ? 'right' : 'left';
            }
            else {
                this.direction = this.distanceY > 0 ? 'down' : 'up';
            }
        }
        this.cumulativeDistanceX += this.moveX - this.oldMoveX;
        this.cumulativeDistanceY += this.moveY - this.oldMoveY;
        this.uiPan.emit({
            durationTime: event.timeStamp - this.startTime,
            first: this.isFirst,
            eventName: 'uiPan',
            firstDirection: this.direction,
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            srcEvent: event,
            type: 'touchmove',
            stop: unListen,
            resetCumulative: function () {
                _this.reset();
            }
        });
        this.isFirst = false;
    };
    PanDirective.prototype.touchEnd = function (event) {
        var _this = this;
        if (this.isFirst) {
            return;
        }
        this.uiPan.emit({
            durationTime: event.timeStamp - this.startTime,
            first: false,
            eventName: 'uiPan',
            firstDirection: this.direction,
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            srcEvent: event,
            type: 'touchend',
            /*tslint:disable*/
            stop: function noop() {
            },
            /*tslint:enable*/
            resetCumulative: function () {
                _this.reset();
            }
        });
    };
    PanDirective.prototype.reset = function () {
        this.cumulativeDistanceY = 0;
        this.cumulativeDistanceX = 0;
    };
    PanDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], PanDirective.prototype, "uiPan", void 0);
    PanDirective = __decorate([
        Directive({
            selector: '[uiPan]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], PanDirective);
    return PanDirective;
}(TouchManager));

var PinchDirective = /** @class */ (function (_super) {
    __extends(PinchDirective, _super);
    function PinchDirective(elementRef) {
        var _this = _super.call(this) || this;
        _this.elementRef = elementRef;
        _this.uiPinch = new EventEmitter();
        _this.cumulativeScale = 1;
        _this.scale = 1;
        _this.oldScale = 1;
        _this.distanceX = 0;
        _this.distanceY = 0;
        _this.cumulativeDistanceX = 0;
        _this.cumulativeDistanceY = 0;
        _this.oldMoveX = 0;
        _this.oldMoveY = 0;
        _this.isFirst = true;
        return _this;
    }
    PinchDirective.prototype.ngOnInit = function () {
        _super.prototype.init.call(this, 2);
    };
    PinchDirective.prototype.ngOnDestroy = function () {
        _super.prototype.destroy.call(this);
    };
    PinchDirective.prototype.touchStart = function (event) {
        var p1 = event.touches[0];
        var p2 = event.touches[1];
        var centerPoint = this.getCenterPoint(p1, p2);
        this.startX = centerPoint.x;
        this.startY = centerPoint.y;
        this.moveX = this.startX;
        this.moveY = this.startY;
        this.startDistance = this.getChordLength(p1, p2);
        this.scale = 1;
        this.isFirst = true;
        this.startTime = event.timeStamp;
        event.preventDefault();
    };
    PinchDirective.prototype.touchMove = function (event, unListen) {
        var _this = this;
        var p1 = event.touches[0];
        var p2 = event.touches[1];
        var centerPoint = this.getCenterPoint(p1, p2);
        this.oldMoveX = this.moveX;
        this.oldMoveY = this.moveY;
        this.moveX = centerPoint.x;
        this.moveY = centerPoint.y;
        this.distanceX = this.moveX - this.startX;
        this.distanceY = this.moveY - this.startY;
        this.cumulativeDistanceX += this.moveX - this.oldMoveX;
        this.cumulativeDistanceY += this.moveY - this.oldMoveY;
        var moveArea = this.getChordLength(p1, p2);
        this.oldScale = this.scale;
        this.scale = moveArea / this.startDistance;
        this.uiPinch.emit({
            durationTime: event.timeStamp - this.startTime,
            first: this.isFirst,
            eventName: 'uiPinch',
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            stop: unListen,
            srcEvent: event,
            type: 'touchmove',
            resetCumulative: function () {
                _this.cumulativeScale = 1;
                _this.cumulativeDistanceX = 0;
                _this.cumulativeDistanceY = 0;
            },
            scale: this.scale,
            cumulativeScale: this.cumulativeScale * this.scale
        });
        this.isFirst = false;
    };
    PinchDirective.prototype.touchEnd = function (event) {
        var _this = this;
        if (this.isFirst) {
            return;
        }
        this.cumulativeScale = this.cumulativeScale * this.scale;
        this.uiPinch.emit({
            durationTime: event.timeStamp - this.startTime,
            first: false,
            eventName: 'uiPinch',
            startX: this.startX,
            startY: this.startY,
            moveX: this.moveX,
            moveY: this.moveY,
            distanceX: this.distanceX,
            distanceY: this.distanceY,
            cumulativeDistanceX: this.cumulativeDistanceX,
            cumulativeDistanceY: this.cumulativeDistanceY,
            /*tslint:disable*/
            stop: function noop() {
            },
            /*tslint:enable*/
            srcEvent: event,
            type: 'touchend',
            resetCumulative: function () {
                _this.cumulativeScale = 1;
            },
            scale: this.scale,
            cumulativeScale: this.cumulativeScale
        });
    };
    PinchDirective.prototype.getChordLength = function (p1, p2) {
        var x = Math.abs(p1.clientX - p2.clientX);
        var y = Math.abs(p1.clientY - p2.clientY);
        return Math.sqrt(x * x + y * y);
    };
    PinchDirective.prototype.getCenterPoint = function (p1, p2) {
        return {
            x: (p1.clientX + p2.clientX) / 2,
            y: (p1.clientY + p2.clientY) / 2
        };
    };
    PinchDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], PinchDirective.prototype, "uiPinch", void 0);
    PinchDirective = __decorate([
        Directive({
            selector: '[uiPinch]'
        }),
        __metadata("design:paramtypes", [ElementRef])
    ], PinchDirective);
    return PinchDirective;
}(TouchManager));

var UIModalModule = /** @class */ (function () {
    function UIModalModule() {
    }
    UIModalModule = __decorate([
        NgModule({
            imports: [
                CommonModule
            ],
            declarations: [
                AlertComponent,
                DialogComponent,
                DialogBaseComponent,
                MaskComponent,
                ModalComponent,
                ModalBaseComponent,
                ModalFooterComponent,
                ModalHeaderComponent,
                ModalInnerComponent,
                ToastComponent
            ],
            exports: [
                AlertComponent,
                DialogComponent,
                DialogBaseComponent,
                MaskComponent,
                ModalComponent,
                ModalBaseComponent,
                ModalFooterComponent,
                ModalHeaderComponent,
                ModalInnerComponent,
                ToastComponent
            ]
        })
    ], UIModalModule);
    return UIModalModule;
}());

var UIOtherModule = /** @class */ (function () {
    function UIOtherModule() {
    }
    UIOtherModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                UIModalModule
            ],
            declarations: [
                ActionSheetComponent,
                ContentLoadingComponent,
                FixedBarComponent,
                LoadingComponent,
                StopPropagationDirective
            ],
            exports: [
                ActionSheetComponent,
                ContentLoadingComponent,
                FixedBarComponent,
                LoadingComponent,
                StopPropagationDirective
            ]
        })
    ], UIOtherModule);
    return UIOtherModule;
}());

var UIRouterModule = /** @class */ (function () {
    function UIRouterModule() {
    }
    UIRouterModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                UIOtherModule
            ],
            declarations: [
                BackComponent,
                RouterComponent,
                TabComponent,
                TabBarComponent,
                TabBarItemComponent,
                TabViewComponent,
                TabViewItemComponent,
                ViewComponent,
                ComponentHostDirective
            ],
            exports: [
                BackComponent,
                RouterComponent,
                TabComponent,
                TabBarComponent,
                TabBarItemComponent,
                TabViewComponent,
                TabViewItemComponent,
                ViewComponent
            ]
        })
    ], UIRouterModule);
    return UIRouterModule;
}());

var UITouchModule = /** @class */ (function () {
    function UITouchModule() {
    }
    UITouchModule = __decorate([
        NgModule({
            declarations: [
                PanDirective,
                PinchDirective,
            ],
            exports: [
                PanDirective,
                PinchDirective,
            ]
        })
    ], UITouchModule);
    return UITouchModule;
}());

var UISlideModule = /** @class */ (function () {
    function UISlideModule() {
    }
    UISlideModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                UITouchModule
            ],
            declarations: [
                SlideComponent,
                SlideItemComponent
            ],
            exports: [
                SlideComponent,
                SlideItemComponent
            ]
        })
    ], UISlideModule);
    return UISlideModule;
}());

var UIImageViewerModule = /** @class */ (function () {
    function UIImageViewerModule() {
    }
    UIImageViewerModule = __decorate([
        NgModule({
            imports: [
                UISlideModule,
                CommonModule,
                UITouchModule
            ],
            declarations: [
                ImageViewerComponent,
                ImageViewableDirective
            ],
            exports: [
                ImageViewerComponent,
                ImageViewableDirective
            ]
        })
    ], UIImageViewerModule);
    return UIImageViewerModule;
}());

var AppComponent = /** @class */ (function () {
    function AppComponent(document, env, scale, routeCacheController, location, appController, listActivatedService) {
        this.document = document;
        this.env = env;
        this.scale = scale;
        this.routeCacheController = routeCacheController;
        this.location = location;
        this.appController = appController;
        this.listActivatedService = listActivatedService;
        /**
         * 根据不同的屏幕宽度，设置 html 元素的 font-size
         * 主要用于 rem 布局，默认为 10
         */
        this.baseFontSize = 10;
        /**
         * 屏幕 X 轴默认逻辑像素
         */
        this.screenWidth = 375;
        /**
         * angular 路由的 name
         */
        this.name = PRIMARY_OUTLET;
        this.showMask = false;
        this.subs = [];
    }
    Object.defineProperty(AppComponent.prototype, "isIphone", {
        // 根据不同环境，需要把页面头部增高相对尺寸，以显示时间，电池电量等信息，这里通过样式来控制
        get: function () {
            return this.env === BrowserENV.iphone;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppComponent.prototype, "isIphoneX", {
        get: function () {
            return this.env === BrowserENV.iphoneX;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppComponent.prototype, "isAndroid", {
        get: function () {
            return this.env === BrowserENV.android;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppComponent.prototype, "isDefault", {
        get: function () {
            return this.env === BrowserENV.default;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AppComponent.prototype, "defaultDocWidth", {
        get: function () {
            return this.screenWidth * this.scale;
        },
        enumerable: true,
        configurable: true
    });
    AppComponent.prototype.ngOnInit = function () {
        var _this = this;
        // 根据当前应用是否在在路由动画转场中时，切换遮罩的显示与隐藏，防止多次点击
        this.subs.push(this.appController.transition.subscribe(function (b) {
            _this.showMask = b;
        }));
        // 当屏幕大小改变时，变更环境变量
        this.subs.push(this.appController.onResize.subscribe(function (env) {
            _this.env = env;
        }));
        // 当路由地址变更时，设置路由缓存
        this.subs.push(this.location.subscribe(function () {
            _this.routeCacheController.isCacheCurrentView(false);
        }));
        this.htmlElement = this.document.querySelector('html');
        this.resize();
    };
    AppComponent.prototype.ngOnDestroy = function () {
        this.subs.forEach(function (item) { return item.unsubscribe(); });
    };
    AppComponent.prototype.resize = function () {
        // 通过当前视口的宽度，动态设置 css rem 值的大小
        if (!this.htmlElement) {
            return;
        }
        var docWidth = this.htmlElement.getBoundingClientRect().width;
        var scale = docWidth / this.defaultDocWidth;
        this.htmlElement.style.fontSize = scale * this.baseFontSize + "px";
        this.appController.resize(getDeviceType());
    };
    AppComponent.prototype.documentTouchStart = function () {
        // 当页面有 list 组件时，如果其中某一项已打开 sliding，当用户点击其它地方时，需要发布全局事件，来收起 sliding
        this.listActivatedService.publish();
    };
    AppComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: undefined, decorators: [{ type: Inject, args: [UI_BROWSER_ENV,] }] },
        { type: Number, decorators: [{ type: Inject, args: [UI_SCREEN_SCALE,] }] },
        { type: RouteCacheController },
        { type: Location },
        { type: AppController },
        { type: ListActivatedService }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], AppComponent.prototype, "baseFontSize", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], AppComponent.prototype, "screenWidth", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], AppComponent.prototype, "name", void 0);
    __decorate([
        HostBinding('class.ui-iphone'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], AppComponent.prototype, "isIphone", null);
    __decorate([
        HostBinding('class.ui-iphone-x'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], AppComponent.prototype, "isIphoneX", null);
    __decorate([
        HostBinding('class.ui-android'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], AppComponent.prototype, "isAndroid", null);
    __decorate([
        HostBinding('class.ui-default'),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [])
    ], AppComponent.prototype, "isDefault", null);
    __decorate([
        HostListener('window:resize'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], AppComponent.prototype, "resize", null);
    __decorate([
        HostListener('document:touchstart'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], AppComponent.prototype, "documentTouchStart", null);
    AppComponent = __decorate([
        Component({
            selector: 'ui-app',
            template: "<ng-content></ng-content>\n<ui-router [name]=\"name\"></ui-router>\n<ui-image-viewer></ui-image-viewer>\n<ui-modal-base></ui-modal-base>\n<ui-alert></ui-alert>\n<ui-toast></ui-toast>\n<ui-dialog></ui-dialog>\n<div class=\"ui-app-mask\" [class.ui-show]=\"showMask\"></div>"
        }),
        __param(0, Inject(DOCUMENT)),
        __param(1, Inject(UI_BROWSER_ENV)),
        __param(2, Inject(UI_SCREEN_SCALE)),
        __metadata("design:paramtypes", [Object, String, Number, RouteCacheController,
            Location,
            AppController,
            ListActivatedService])
    ], AppComponent);
    return AppComponent;
}());

var UIAppModule = /** @class */ (function () {
    function UIAppModule() {
    }
    UIAppModule = __decorate([
        NgModule({
            imports: [
                UIRouterModule,
                UIModalModule,
                CommonModule,
                UIImageViewerModule
            ],
            declarations: [
                AppComponent
            ],
            exports: [
                AppComponent
            ]
        })
    ], UIAppModule);
    return UIAppModule;
}());

var UICollectionModule = /** @class */ (function () {
    function UICollectionModule() {
    }
    UICollectionModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                UITouchModule
            ],
            declarations: [
                CollectionComponent,
                CollectionItemComponent
            ],
            exports: [
                CollectionComponent,
                CollectionItemComponent
            ]
        })
    ], UICollectionModule);
    return UICollectionModule;
}());

var UIFlexModule = /** @class */ (function () {
    function UIFlexModule() {
    }
    UIFlexModule = __decorate([
        NgModule({
            declarations: [
                FlexComponent,
                FlexItemComponent
            ],
            exports: [
                FlexComponent,
                FlexItemComponent
            ]
        })
    ], UIFlexModule);
    return UIFlexModule;
}());

var UIFormsModule = /** @class */ (function () {
    function UIFormsModule() {
    }
    UIFormsModule = __decorate([
        NgModule({
            imports: [
                CommonModule,
                UIOtherModule,
                FormsModule,
                UIFlexModule
            ],
            declarations: [
                // 组件
                ButtonComponent,
                CheckboxComponent,
                DateComponent,
                OptionComponent,
                RadioComponent,
                PickerComponent,
                PickerColumnComponent,
                RangeComponent,
                SelectComponent,
                SwitchComponent,
                // 指令
                CheckboxRequiredValidator,
                SwitchRequiredValidator,
                RadioRequiredValidator,
                SelectRequiredValidator,
                DateRequiredValidator,
                SubmitDirective
            ],
            exports: [
                // 组件
                ButtonComponent,
                CheckboxComponent,
                DateComponent,
                OptionComponent,
                PickerComponent,
                PickerColumnComponent,
                RadioComponent,
                RangeComponent,
                SelectComponent,
                SwitchComponent,
                // 指令
                CheckboxRequiredValidator,
                SwitchRequiredValidator,
                RadioRequiredValidator,
                SelectRequiredValidator,
                DateRequiredValidator,
                SubmitDirective
            ],
            providers: [
                RadioStateService, {
                    provide: UI_SELECT_ARROW_CLASSNAME,
                    useValue: 'ui-caret'
                }
            ]
        })
    ], UIFormsModule);
    return UIFormsModule;
}());

var UILayoutModule = /** @class */ (function () {
    function UILayoutModule() {
    }
    UILayoutModule = __decorate([
        NgModule({
            declarations: [
                ButtonsComponent,
                ContentComponent,
                FooterComponent,
                HeaderComponent,
                NavBarComponent,
                PageComponent,
                TitleComponent
            ],
            exports: [
                ButtonsComponent,
                ContentComponent,
                FooterComponent,
                HeaderComponent,
                NavBarComponent,
                PageComponent,
                TitleComponent
            ]
        })
    ], UILayoutModule);
    return UILayoutModule;
}());

var UIListModule = /** @class */ (function () {
    function UIListModule() {
    }
    UIListModule = __decorate([
        NgModule({
            declarations: [
                ListComponent,
                ListHeaderComponent,
                ListInnerComponent,
                ListItemComponent,
                ListOptionComponent,
                ListSlidingComponent,
                ListThumbnailComponent
            ],
            exports: [
                ListComponent,
                ListHeaderComponent,
                ListInnerComponent,
                ListItemComponent,
                ListOptionComponent,
                ListSlidingComponent,
                ListThumbnailComponent
            ]
        })
    ], UIListModule);
    return UIListModule;
}());

var UIScrollModule = /** @class */ (function () {
    function UIScrollModule() {
    }
    UIScrollModule = __decorate([
        NgModule({
            imports: [
                CommonModule
            ],
            declarations: [
                RefresherComponent,
                ScrollComponent,
                ScrollTabComponent,
                ScrollTabButtonComponent
            ],
            exports: [
                RefresherComponent,
                ScrollComponent,
                ScrollTabComponent,
                ScrollTabButtonComponent
            ]
        })
    ], UIScrollModule);
    return UIScrollModule;
}());

var UISegmentModule = /** @class */ (function () {
    function UISegmentModule() {
    }
    UISegmentModule = __decorate([
        NgModule({
            declarations: [
                SegmentComponent,
                SegmentButtonComponent
            ],
            exports: [
                SegmentComponent,
                SegmentButtonComponent
            ]
        })
    ], UISegmentModule);
    return UISegmentModule;
}());

var UINativeModule = /** @class */ (function () {
    function UINativeModule() {
    }
    UINativeModule_1 = UINativeModule;
    UINativeModule.forRoot = function () {
        return {
            ngModule: UINativeModule_1,
            providers: [{
                    provide: RouteReuseStrategy,
                    useClass: UIRouteReuseStrategy
                }, {
                    provide: UI_BACK_ICON_CLASSNAME,
                    useValue: 'ui-icon-arrow-back'
                }, {
                    provide: UI_ROUTER_ANIMATION_STEPS,
                    useValue: navigator.userAgent.indexOf('web-2.0') > -1 ? 1 : 30
                }, {
                    provide: UI_DO_REFRESH_DISTANCE,
                    useValue: 40
                }, {
                    provide: UI_DO_LOAD_DISTANCE,
                    useValue: 600
                }, {
                    provide: UI_BROWSER_ENV,
                    useFactory: getDeviceType
                }, {
                    provide: UI_SCREEN_SCALE,
                    useValue: 1
                },
                AppController,
                RouteCacheController,
                AlertController,
                DialogController,
                ListActivatedService,
                ModalController,
                ToastController,
                ViewStateService,
                ImageViewerController
            ]
        };
    };
    var UINativeModule_1;
    UINativeModule = UINativeModule_1 = __decorate([
        NgModule({
            exports: [
                UIAppModule,
                UICollectionModule,
                UIFlexModule,
                UIFormsModule,
                UIImageViewerModule,
                UILayoutModule,
                UIListModule,
                UIModalModule,
                UIOtherModule,
                UIRouterModule,
                UIScrollModule,
                UISegmentModule,
                UISlideModule,
                UITouchModule
            ]
        })
    ], UINativeModule);
    return UINativeModule;
}());

/*
 * Public API Surface of ui-native
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ActionSheetComponent, ActionSheetService, AlertComponent, AlertController, AppController, BackComponent, BrowserENV, ButtonComponent, ButtonsComponent, CheckboxComponent, CheckboxRequiredValidator, CollectionComponent, CollectionItemComponent, ComponentHostDirective, ContentComponent, ContentLoadingComponent, ContentLoadingController, DateComponent, DateRequiredValidator, DialogBaseComponent, DialogComponent, DialogController, FixedBarComponent, FlexComponent, FlexItemComponent, FooterComponent, HeaderComponent, ImageViewableDirective, ImageViewerComponent, ImageViewerController, ListActivatedService, ListComponent, ListEventService, ListHeaderComponent, ListInnerComponent, ListItemComponent, ListOptionComponent, ListSlidingComponent, ListThumbnailComponent, LoadingComponent, MaskComponent, ModalBaseComponent, ModalComponent, ModalController, ModalFooterComponent, ModalHeaderComponent, ModalInnerComponent, NavBarComponent, OptionComponent, PageComponent, PanDirective, PickerColumnComponent, PickerComponent, PickerService, PinchDirective, PullDownRefreshController, PullUpLoadController, RadioComponent, RadioRequiredValidator, RangeComponent, RefresherComponent, RouteCacheController, RouterComponent, RouterService, ScrollComponent, ScrollTabButtonComponent, ScrollTabComponent, SegmentButtonComponent, SegmentComponent, SelectComponent, SelectRequiredValidator, SlideComponent, SlideItemComponent, StopPropagationDirective, SubmitDirective, SwitchComponent, SwitchRequiredValidator, TabBarComponent, TabBarItemComponent, TabComponent, TabController, TabViewComponent, TabViewItemComponent, TitleComponent, ToastComponent, ToastController, ToastType, UINativeModule, UIRouteReuseStrategy, UI_BACK_ICON_CLASSNAME, UI_BROWSER_ENV, UI_DO_LOAD_DISTANCE, UI_DO_REFRESH_DISTANCE, UI_ROUTER_ANIMATION_STEPS, UI_SCREEN_SCALE, UI_SELECT_ARROW_CLASSNAME, UI_VIEW_INIT_STATE, UI_VIEW_INIT_TOUCH_BACK, ViewComponent, ViewState, ViewStateService, getDeviceType, inputAttrToBoolean, SelectService as ɵa, RadioStateService as ɵb, TabViewItemService as ɵc, ScrollTabService as ɵd, SegmentService as ɵe, TouchManager as ɵf, UIAppModule as ɵg, UIRouterModule as ɵh, UIOtherModule as ɵi, UIModalModule as ɵj, UIImageViewerModule as ɵk, UISlideModule as ɵl, UITouchModule as ɵm, AppComponent as ɵn, UICollectionModule as ɵo, UIFlexModule as ɵp, UIFormsModule as ɵq, UILayoutModule as ɵr, UIListModule as ɵs, UIScrollModule as ɵt, UISegmentModule as ɵu };
//# sourceMappingURL=cloudbae-frontend-ui-native.js.map
